package com.alpha.lagouapk.bean.aiinterview

import android.os.Parcelable
import android.text.TextUtils
import com.alpha.lagouapk.manager.CMKVUtils
import com.lagou.commonlibs.utils.ListUtil
import kotlinx.android.parcel.Parcelize

/**
 * Time：2024/7/29 14:20:54
 * Author：wangjunwei
 * Description：
 * 表示 AI 音色选择的响应。
 */
@Parcelize
data class AiTimbreSelectRes(
    /**
     * 默认的声音值。
     */
    val defaultVoiceValue: String,
    
    /**
     * 声音类型的列表。
     */
    val voiceTypeList: List<VoiceTimbreType>
) :Parcelable {

    /**
     * 表示一种声音类型。
     */
    @Parcelize
    data class VoiceTimbreType (
        /**
         * 声音的类型（例如：男生、女生）。
         */
        val type: String,

        /**
         * 该类型下的声音列表。
         */
        val voiceList: List<VoiceTimbre>
    ) : Parcelable {
        /**
         * 表示具体的音色。
         */
        @Parcelize
        data class VoiceTimbre(
            /**
             * 表示声音图标的 URL。
             */
            val icon: String,
            
            /**
             * 声音的名称。
             */
            val name: String,
            
            /**
             * 声音的值。
             */
            val value: String,
            
            /**
             * 声音文件的 URL。
             */
            val voiceUrl: String,

            /**
             * 声音文件类型：
             * 1：男生
             * 2：女生
             * 3：口音
             */
            val type: String?,

            /**
             * 是否选中状态
             */
            var isSelected: Boolean
        ) : Parcelable {
            override fun equals(other: Any?): Boolean {
                if (this === other) return true
                if (javaClass != other?.javaClass) return false

                if (other is VoiceTimbre) {
                    return icon == other.icon
                            && name == other.name
                            && value == other.value
                            && voiceUrl == other.voiceUrl
                            && type == other.type
                            && isSelected == other.isSelected
                }

                return super.equals(other)
            }
        }
    }

    /**
     * 推荐音色列表
     */
    fun getRecommendTimbreList(): List<VoiceTimbreType.VoiceTimbre> {
        val recommendTimbreList = mutableListOf<VoiceTimbreType.VoiceTimbre>()

        val realDefaultVoiceValue = CMKVUtils.getSelectTimbre()?.value

        voiceTypeList.forEachIndexed { index, voiceTimbreType ->
            if (!ListUtil.isEmpty(voiceTimbreType.voiceList)){
                //找出默认选中的音色，并设置选中状态
                for((index, voiceTimbre) in voiceTimbreType.voiceList.withIndex()){
                    if (TextUtils.equals(voiceTimbre.value, realDefaultVoiceValue ?: defaultVoiceValue)) {
                        voiceTimbre.isSelected = true
                        break
                    }
                }
                recommendTimbreList.addAll(voiceTimbreType.voiceList)
            }
        }
        return recommendTimbreList
    }

    fun getMaleTimbreList(): List<VoiceTimbreType.VoiceTimbre> {

        return getGoalTimbreList(TimbreCategory.male)
    }

    fun getFemaleTimbreList(): List<VoiceTimbreType.VoiceTimbre> {

        return getGoalTimbreList(TimbreCategory.female)
    }

    /**
     * 获取口音音色列表
     */
    fun getAccentTimbreList(): List<VoiceTimbreType.VoiceTimbre> {

        return getGoalTimbreList(TimbreCategory.accents)
    }

    private fun getGoalTimbreList(timbreCategory: TimbreCategory): List<VoiceTimbreType.VoiceTimbre> {
        if (ListUtil.isEmpty(voiceTypeList)){
            return mutableListOf()
        }
        val realDefaultVoiceValue = CMKVUtils.getSelectTimbre()?.value

        val timbreList = voiceTypeList.first {
            TextUtils.equals(
                it.type,
                timbreCategory.displayName
            )
        }.voiceList
        for (voiceTimbre in timbreList) {
            if (TextUtils.equals(voiceTimbre.value, realDefaultVoiceValue ?: defaultVoiceValue)) {
                voiceTimbre.isSelected = true
                break
            }
        }
        return timbreList
    }
}

/**
 * 音色类型枚举值
 */
enum class TimbreCategory(val displayName: String) {
    recommend("推荐"),
    male("男声"),
    female("女声"),
    accents("口音")
}