package com.alpha.lagouapk.manager.net

import android.content.Context
import android.text.TextUtils
import com.ApiService
import com.alpha.lagouapk.LagouApplication
import com.alpha.lagouapk.bean.aiinterview.AIInterviewChatRes
import com.alpha.lagouapk.bean.aiinterview.AIInterviewCreateRes
import com.alpha.lagouapk.bean.aiinterview.AiChatInfoRes
import com.alpha.lagouapk.bean.aiinterview.AiInterviewAsrRes
import com.alpha.lagouapk.bean.aiinterview.AiInterviewFloatEntryRes
import com.alpha.lagouapk.bean.aiinterview.AiInterviewSessionListRes
import com.alpha.lagouapk.bean.aiinterview.AiInterviewSessionListRes.AiSessionItem
import com.alpha.lagouapk.bean.aiinterview.AiInterviewTtsRes
import com.alpha.lagouapk.bean.aiinterview.AiRelateMsgItemType
import com.alpha.lagouapk.bean.aiinterview.AiTimbreSelectRes
import com.alpha.lagouapk.bean.resume.AiImproveContent
import com.alpha.lagouapk.manager.AiManager
import com.lagou.baselib.analytic.AnalyticHelper
import com.lagou.baselib.analytic.bean.json.ClickInfo
import com.lagou.baselib.download.DownloadManager
import com.lagou.baselib.download.SimpleDownloadListener
import com.lagou.baselib.net.httpnetback.AdapterBaseCallBack
import com.lagou.baselib.net.httpnetback.BaseCallback
import com.lagou.baselib.net.httpnetback.BaseEventCallback
import com.lagou.baselib.net.httpnetback.OnAdapterCommonListenerImpl
import com.lagou.chat.chatlibrary.manager.MediaManager
import com.lagou.commonlibs.engine.ThreadUtil
import com.lagou.commonlibs.inter.OnSingleMethodListener
import com.lagou.commonlibs.utils.ListUtil
import com.lagou.commonlibs.utils.LogUtils
import com.lagou.commonlibs.utils.SDCardUtils
import com.lagou.commonlibs.utils.StringUtil
import com.lagou.socketchannel.http.HttpCallback
import com.lagou.socketchannel.http.HttpHelper
import com.lagou.socketchannel.http.HttpRequest
import com.lagou.socketchannel.http.HttpResponse
import com.lagou.socketchannel.http.event.HttpEventHelper
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.withContext
import java.io.File

/**
 * Time：2024/6/18 17:17:00
 * Author：wangjunwei
 * Description：AI 网络请求管理类
 *
 */
class AiNetManager {
    companion object {

        const val ASR_URL = "https://gate.lagou.com/v1/entry/ai/chat/asr"

        /**
         * 文字转语音的路径
         */
        @kotlin.jvm.JvmField
        var tts_output_dir: String? = null

        /**
         * 文字录音路径
         */
        @kotlin.jvm.JvmField
        var record_output_dir: String? = null

        @JvmStatic
        fun getTtsOutDir(context: Context?): String? {
            if (context == null){
                return tts_output_dir
            }

            if (TextUtils.isEmpty(tts_output_dir)){
                tts_output_dir = SDCardUtils.getVideoPath(LagouApplication.getLagouApplication()) + "/ai/interview/tts"
            }

            return tts_output_dir
        }

        /**
         * 获取录制音频的目录
         * @param context
         * @return
         */
        @JvmStatic
        fun getRecordOutDir(context: Context?): String? {
            if (context == null) {
                return record_output_dir
            }
            if (TextUtils.isEmpty(record_output_dir)) {
                record_output_dir = SDCardUtils.getVideoPath(context) + "/ai/interview/record"
            }
            return record_output_dir
        }

        /**
         * 1 获取Ai 面试的会话列表
         */
        fun fetchAiInterviewSessionList(
            onCommonListListener: OnAdapterCommonListenerImpl<List<AiInterviewSessionListRes?>?,
                    HttpResponse<List<AiInterviewSessionListRes?>?>>
        ) {

            if (!onCommonListListener.isActive) {
                return
            }

            //onCommonListListener.showLoading()
            ApiService.api.fetchAiInterviewSessionList().enqueue(object :
                AdapterBaseCallBack<List<AiInterviewSessionListRes?>>(onCommonListListener) {
                override fun onAdapterSuccess(
                    request: HttpRequest<*>,
                    response: HttpResponse<List<AiInterviewSessionListRes?>?>
                ) {
                    super.onAdapterSuccess(request, response)

                    onCommonListListener.onSuccess(response.content)
                }
            })
        }

        /**
         * 获取Ai面试指定sessionId会话列表
         */
        fun fetchAiInterviewSessionList(sessionId: Int, onCommonListListener: OnAdapterCommonListenerImpl<List<AiInterviewSessionListRes?>?,
                    HttpResponse<List<AiInterviewSessionListRes?>?>>
        ) {

            if (!onCommonListListener.isActive) {
                return
            }
            ApiService.api.fetchAiInterviewSessionList(sessionId).enqueue(object :
                AdapterBaseCallBack<List<AiInterviewSessionListRes?>>(onCommonListListener) {
                override fun onAdapterSuccess(
                    request: HttpRequest<*>,
                    response: HttpResponse<List<AiInterviewSessionListRes?>?>
                ) {
                    super.onAdapterSuccess(request, response)

                    onCommonListListener.onSuccess(response.content)
                }
            })
        }


        /**
         * 展开接口返回的所有会话列表数据，方便页面展示
         * @param content 后端返回所有会话合集
         */
        fun handleNetSessionList(content: List<AiInterviewSessionListRes?>): List<AiSessionItem> {
            val allAiInterviewSessionList = mutableListOf<AiSessionItem>()

            content.filter { it != null }.forEach {
                it?.sessionItemList?.forEachIndexed { index, aiSessionItem ->
                    aiSessionItem.apply {
                        sessionId = it.sessionId
                        sessionState = it.sessionState
                        positionId = it.positionId
                    }

                    //1 判断广告列表是否为空，如果不为空则单独放到一个新的AiSession中
                    if (!ListUtil.isEmpty(aiSessionItem.itemRelateMsgList)){
                        //2 构造广告aiSessionItem
                        val adsSessionItem = AiSessionItem()
                        adsSessionItem.itemRelateMsgList = aiSessionItem.itemRelateMsgList
                        adsSessionItem.msgType = aiSessionItem.itemRelateMsgList[0]?.msgType ?: AiRelateMsgItemType.COMMON
                        adsSessionItem.tag = HttpEventHelper.getInstance().clientTag

                        //3 原始的消息session广告列表置空&add
                        aiSessionItem.itemRelateMsgList = null
                        allAiInterviewSessionList.add(aiSessionItem)

                        //4 新的广告aiSession add到List中
                        allAiInterviewSessionList.add(adsSessionItem)

                        LogUtils.i("completePackageDataHandleAd", "获取sessionList列表时，广告数据展开，itemRelateMsgList ${adsSessionItem.itemRelateMsgList}")
                    }else {
                        allAiInterviewSessionList.add(aiSessionItem)
                    }
                }
            }

            return allAiInterviewSessionList
        }

        fun chatInterviewAdd(
            answer: String?,
            originalTag: Int?,
            positionId: Int,
            sessionId: Int,
            voiceMessage: Boolean = false,
            baseEventCallback: BaseEventCallback<AIInterviewChatRes?>?
        ): HttpRequest<AIInterviewChatRes> {
            return chatInterview(
                false,
                "interview",
                answer,
                originalTag,
                positionId,
                sessionId,
                voiceMessage,
                baseEventCallback
            )
        }

        /**
         * 2 AI 面试接口
         * @param stream 是否全量数据：true: 每包数据是全量数据，最后一包是完整结；false: 每包数据是增量数据，所有包内容拼接是完整
         * @param businessType interview 模拟面试
         * @param answer C 回答（点开始面试后，answer无需赋值）
         * @param positionId 职务id
         * @param sessionId
         * @param voiceMessage 是否是语音消息
         */
        fun chatInterview(
            stream: Boolean,
            businessType: String,
            answer: String?,
            originalTag: Int?,
            positionId: Int,
            sessionId: Int,
            voiceMessage: Boolean = false,
            baseEventCallback: BaseEventCallback<AIInterviewChatRes?>?
        ): HttpRequest<AIInterviewChatRes> {

            return ApiService.api.chatInterview(stream, businessType, answer, positionId, sessionId, voiceMessage)
                .parseRawType()
                .tag(originalTag ?: HttpEventHelper.getInstance().clientTag)
                .enqueueEvent(baseEventCallback)
        }


        /**
         * 上传语音文件到服务端，转化为文本
         * @param type pcm/wav/mp3/opus，缺省为： mp3
         */
        fun uploadVoiceToText(
            voicePath: String?,
            originalTag: Int?,
            callback: HttpCallback<AiInterviewAsrRes?>
        ): HttpRequest<AiInterviewAsrRes>? {
            if (TextUtils.isEmpty(voicePath)) return null

            val clickInfo = ClickInfo()
            clickInfo.apply {
                page_id = "m1uiq"
                address_id = "k1uk5"
                addClick_props("duration", MediaManager.getAudioDuration(voicePath))
            }
            AnalyticHelper.getInstance().sendClickInfo(clickInfo)

            LogUtils.i("AiInterviewNetManager", "+++++ 发送的音频文件：voicePath = $voicePath")

            val httpRequest = HttpHelper
                .post<AiInterviewAsrRes>()
                .url(ASR_URL)
                .query("type", "amr")
                .bodyFileFromData(File(voicePath))
                .tag(originalTag ?: HttpEventHelper.getInstance().clientTag)
                .enqueue(callback)

            return httpRequest
        }

        /**
         * 转换文本为音频，支持Ai生成文本语音播报的功能
         * @param type pclagoum/wav/mp3/opus
         */
        fun convertTextVoice(text: String?, voiceType: String? = "BV001_streaming",
                             baseCallback: BaseCallback<AiInterviewTtsRes?>){
            if (TextUtils.isEmpty(text)){
                return
            }
            ApiService.api.convertTextVoice("mp3", voiceType, text).enqueue(object : BaseCallback<AiInterviewTtsRes?>() {
                override fun onSuccess(request: HttpRequest<*>, responese: HttpResponse<AiInterviewTtsRes?>) {

                    if (responese.content == null){
                        baseCallback.onSuccess(request, responese)
                        return
                    }

                    ThreadUtil.io{
                        responese.content?.convertVoiceFile()
                        ThreadUtil.ui {
                            baseCallback.onSuccess(request, responese)
                        }
                    }
                }

                override fun onFailure(
                    request: HttpRequest<*>?,
                    response: HttpResponse<AiInterviewTtsRes?>?
                ) {
                    super.onFailure(request, response)

                    baseCallback.onFailure(request, response)
                }
            })
        }


        /**
         * Jd 页AI面试入口是否显示
         */
        @JvmStatic
        fun getAiInterviewFloatShow(positionId: Int, onSingleMethodListener: OnSingleMethodListener<AiInterviewFloatEntryRes?>){
            ApiService.api.getAiInterviewFloatShow(positionId).enqueue(object : BaseCallback<AiInterviewFloatEntryRes>(false){
                override fun onSuccess(
                    request: HttpRequest<*>?,
                    responese: HttpResponse<AiInterviewFloatEntryRes>?
                ) {
                    onSingleMethodListener.onCallBack(responese?.content)
                }

                override fun onFailure(
                    request: HttpRequest<*>?,
                    response: HttpResponse<AiInterviewFloatEntryRes>?
                ) {
                    super.onFailure(request, response)
                    onSingleMethodListener.onCallBack(null)
                }
            })
        }

        /**
         * AI 面试创建会话
         */
        fun createAiInterviewSession(aiInterviewSessionListRes: AiInterviewSessionListRes?,
                                     onSingleListener: OnSingleMethodListener<AIInterviewCreateRes>?){
            if (aiInterviewSessionListRes == null){
                return
            }
            ApiService.api.createAiInterviewSession(aiInterviewSessionListRes.positionId).enqueue(object :
                BaseCallback<AIInterviewCreateRes>() {
                override fun onSuccess(request: HttpRequest<*>, httpResponse: HttpResponse<AIInterviewCreateRes?>) {
                    onSingleListener?.onCallBack(httpResponse.content)
                }
            })
        }

        /**
         * 下载对应的音色文件
         */
        fun downloadTimbreFile(url: String, downloadListener: SimpleDownloadListener){
            val destDir = SDCardUtils.getMusicPath(LagouApplication.getLagouApplication())
            val timbreFileName = StringUtil.md5(url)
            DownloadManager
                .newTask(url)
                .dest(destDir)
                .fileName(timbreFileName)
                .dontOverwrite()
                .listener(downloadListener)
                .start()
        }

        /**
         * 获取可选择音色列表
         */
        suspend fun getTimbreListData(): AiTimbreSelectRes? {
            return withContext(Dispatchers.IO){
                ApiService.api.timbreListData
                    .callback(object : BaseCallback<AiTimbreSelectRes?>() {
                        override fun onSuccess(
                            p0: HttpRequest<*>?,
                            p1: HttpResponse<AiTimbreSelectRes?>?
                        ) {
                        }
                    })
                    .execute()?.content
            }
        }

        /***
         * 获取Ai面试的配置信息
         */
        @JvmStatic
        fun getAiChatInfo(){
            ApiService.api.aiChatInfo.enqueue(object : BaseCallback<AiChatInfoRes>(false) {
                override fun onSuccess(
                    request: HttpRequest<*>?,
                    response: HttpResponse<AiChatInfoRes?>
                ) {
                    AiManager.saveAiChatConfigInfo(response.content)
                }
            })
        }

        /**
         * Ai 润色实体接口
         */
        @JvmStatic
        fun postAiImproveContent(text: String?, type: Int, callback: BaseCallback<AiImproveContent?>){
            ApiService.api.postAiImproveContent(text, type).enqueue(callback)
        }
    }

}