package com.alpha.lagouapk.ui.dialog.ai

import android.os.Bundle
import android.text.method.ScrollingMovementMethod
import android.view.Gravity
import android.view.View
import androidx.lifecycle.Observer
import androidx.lifecycle.ViewModelProvider
import androidx.lifecycle.lifecycleScope
import com.alpha.lagouapk.R
import com.alpha.lagouapk.bean.resume.AiImproveConfig
import com.alpha.lagouapk.bean.resume.AiImproveContent
import com.alpha.lagouapk.bean.resume.AiImproveType.Companion.getRequestType
import com.alpha.lagouapk.manager.CUserManager
import com.alpha.lagouapk.manager.net.AiNetManager.Companion.postAiImproveContent
import com.alpha.lagouapk.ui.dialog.BaseBottomDialogFragment
import com.lagou.baselib.BaseActivity
import com.lagou.baselib.net.httpnetback.BaseCallback
import com.lagou.commonlibs.ui.dialog.BaseDialogFragment
import com.lagou.commonlibs.utils.ResourceUtils
import com.lagou.commonlibs.view.MaxWeightLayout.LayoutParams
import com.lagou.socketchannel.http.HttpRequest
import com.lagou.socketchannel.http.HttpResponse
import kotlinx.android.synthetic.main.dialog_ai_improve.image_improve_close
import kotlinx.android.synthetic.main.dialog_ai_improve.tv_append
import kotlinx.android.synthetic.main.dialog_ai_improve.tv_improve_after
import kotlinx.android.synthetic.main.dialog_ai_improve.tv_improve_before
import kotlinx.android.synthetic.main.dialog_ai_improve.tv_improve_content
import kotlinx.android.synthetic.main.dialog_ai_improve.tv_improve_success_limit
import kotlinx.android.synthetic.main.dialog_ai_improve.tv_re_polish
import kotlinx.android.synthetic.main.dialog_ai_improve.tv_replace
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext

/**
 * Time：2025/3/31 15:32:57
 * Author：wangjunwei
 * Description：
 * Ai 润色弹窗
 */
class AiImproveDialog : BaseDialogFragment() {

    companion object {
        private const val KEY_AI_INFO = "ai_info"
        private const val KEY_AI_REQUEST_TYPE = "ai_request_type"

        /**
         * ai 内容
         */
        @JvmStatic
        fun getInstance(aiImproveContent: AiImproveContent?, type: Int): AiImproveDialog {

            val args = Bundle()
            args.putParcelable(KEY_AI_INFO, aiImproveContent)
            args.putInt(KEY_AI_REQUEST_TYPE, type)
            val aiImproveDialog = AiImproveDialog()
            aiImproveDialog.arguments = args
            return aiImproveDialog
        }
    }


    private var aiImproveContent: AiImproveContent? = null

    /**
     * 请求类型
     */
    private var type: Int = 0

    /**
     * 原始内容
     */
    private var originalContent: String? = null
    private val aiImproveViewModel: AiImproveViewModel by lazy {
        ViewModelProvider(requireActivity()).get(AiImproveViewModel::class.java)
    }

    override fun getLayoutId(): Int {
        return R.layout.dialog_ai_improve
    }

    override fun fillUI() {
        super.fillUI()
        aiImproveContent = arguments?.getParcelable(KEY_AI_INFO)
        type = arguments?.getInt(KEY_AI_REQUEST_TYPE) ?: 0

        tv_improve_after.isSelected = true
        tv_improve_before.isSelected = false
        tv_improve_content.movementMethod = ScrollingMovementMethod()
        tv_improve_content.text = aiImproveContent?.result

        viewLifecycleOwner.lifecycleScope.launch(Dispatchers.IO) {
            val aiImproveConfig: AiImproveConfig = CUserManager.getInstance().aiImproveConfig
            aiImproveConfig.apply {
                withContext(Dispatchers.Main) {
                    //设置标题和副标题
                    if (aiImproveConfig.totalNum <= 0){
                        tv_improve_success_limit.visibility = View.INVISIBLE
                    }else{
                        tv_improve_success_limit.visibility = View.VISIBLE
                        tv_improve_success_limit.text =
                            ResourceUtils.getString(R.string.ai_improve_times, aiImproveConfig.totalNum)
                    }
                }
            }
        }
    }

    override fun initListener() {
        super.initListener()
        aiImproveViewModel.originalLiveData.observe(this,
            Observer<String> { content -> originalContent = content })

        //1 关闭弹窗
        image_improve_close.setOnClickListener {
            dismiss()
        }

        //2 润色前润色后
        tv_improve_after.setOnClickListener {
            tv_improve_after.isSelected = !tv_improve_after.isSelected
            tv_improve_before.isSelected = !tv_improve_before.isSelected
            tv_re_polish.visibility = View.VISIBLE

            // 设置润色后的内容
            tv_improve_content.setText(aiImproveContent?.result)
            tv_improve_content.scrollTo(0, 0)
        }
        tv_improve_before.setOnClickListener {
            tv_improve_after.isSelected = !tv_improve_after.isSelected
            tv_improve_before.isSelected = !tv_improve_before.isSelected
            tv_re_polish.visibility = View.INVISIBLE

            // 设置润色前的内容
            tv_improve_content.setText(aiImproveContent?.originalContent)
            tv_improve_content.scrollTo(0, 0)
        }

        //3 重新润色
        tv_re_polish.setOnClickListener {
            //重新润色逻辑
            postAiImproveContent()
        }

        //4 补充到内容下方
        tv_append.setOnClickListener {
            aiImproveContent?.result?.let {
                aiImproveViewModel.appendImproveContent(it)
            }
            dismiss()
        }

        //5 替换为此内容
        tv_replace.setOnClickListener {
            aiImproveContent?.result?.let {
                aiImproveViewModel.replaceImproveContent(it)
            }
            dismiss()
        }
    }

    override fun getWidth(): Int {
        return LayoutParams.MATCH_PARENT
    }

    /**
     * 简历润色
     */
    private fun postAiImproveContent() {
        showLoading()

        aiImproveContent?.originalContent.let {
            postAiImproveContent(
                it, getRequestType(type),
                object : BaseCallback<AiImproveContent?>() {
                    override fun onSuccess(
                        request: HttpRequest<*>,
                        response: HttpResponse<AiImproveContent?>
                    ) {
                        hideLoading()
                        if (!isAdded) {
                            return
                        }
                        aiImproveContent?.let {
                            it.result = response.content?.result
                        }
                        tv_improve_content.setText(aiImproveContent?.result)
                    }

                    override fun onFailure(
                        request: HttpRequest<*>?,
                        response: HttpResponse<AiImproveContent?>?
                    ) {
                        super.onFailure(request, response)
                        hideLoading()
                    }
                })
        }
    }

//    class CustomScrollingMovementMethod(private val parent: ViewGroup) : ScrollingMovementMethod() {
//
//        override fun onTouchEvent(
//            widget: TextView,
//            buffer: Spannable,
//            event: MotionEvent
//        ): Boolean {
//            when (event.action) {
//                MotionEvent.ACTION_DOWN -> {
//                    // 请求父 View 不拦截事件
//                    parent.requestDisallowInterceptTouchEvent(true)
//                }
//                MotionEvent.ACTION_MOVE -> {
//                    var dy = 0f
//                    if (event.historySize > 0) {
//                        dy = event.getY() - event.getHistoricalY(0)
//                    }
//                    val isAtTop = !widget.canScrollVertically(-1)
//                    val isAtBottom = !widget.canScrollVertically(1)
//                    if ((isAtTop && dy > 0) || (isAtBottom && dy < 0) || dy == 0.toFloat()) {
//                        Log.i("CustomScrollingMovementMethod", "isAtTop = $isAtTop  isAtBottom = $isAtBottom  dy = $dy  让父View接管事件")
//                        // 如果 TextView 滑动到顶部且向上滑，或者滑动到底部且向下滑，让父 View 拦截
//                        parent.requestDisallowInterceptTouchEvent(false)
//                        return false
//                    } else {
//                        Log.i("CustomScrollingMovementMethod", "不让父View接管事件")
//                        // 否则不让父 View 拦截
//                        parent.requestDisallowInterceptTouchEvent(true)
//                    }
//                }
//                MotionEvent.ACTION_UP, MotionEvent.ACTION_CANCEL -> {
//                    // 手指抬起或事件取消时，恢复父 View 的默认拦截状态
//                    parent.requestDisallowInterceptTouchEvent(false)
//                    return false
//                }
//            }
//            return super.onTouchEvent(widget, buffer, event)
//        }
//    }

    fun showLoading() {
        if (activity is BaseActivity) {
            val baseActivity = activity as BaseActivity
            if (baseActivity.isActivityDestroyed) {
                return
            }

            baseActivity.showLoading()
        }
    }

    fun hideLoading() {
        if (activity is BaseActivity) {
            val baseActivity = activity as BaseActivity
            if (baseActivity.isActivityDestroyed) {
                return
            }

            baseActivity.hideLoading()
        }
    }

    override fun getGravity(): Int {
        return Gravity.BOTTOM
    }
}