package com.alpha.lagouapk.ui.home.v2

import android.content.Context
import android.graphics.Color
import android.graphics.drawable.GradientDrawable
import android.util.ArrayMap
import android.util.AttributeSet
import android.view.View
import android.widget.FrameLayout
import android.widget.HorizontalScrollView
import android.widget.ImageView
import android.widget.LinearLayout
import com.alpha.lagouapk.R
import com.alpha.lagouapk.bean.SelectOption
import com.alpha.lagouapk.bean.config.FilterData
import com.alpha.lagouapk.bean.config.HomeActivityConfig
import com.alpha.lagouapk.bean.config.HomeItemActivityConfig
import com.alpha.lagouapk.ui.home.v2.filter.AggregateFilterView
import com.alpha.lagouapk.ui.home.v2.filter.FilterLevelView
import com.alpha.lagouapk.ui.home.v2.filter.FlatFilterView
import com.alpha.lagouapk.ui.home.v2.filter.HomeFilterDialog
import com.alpha.lagouapk.ui.home.v2.filter.NavAggregateFilterView
import com.alpha.lagouapk.ui.home.v2.filter.NavigateFilterView
import com.alpha.lagouapk.ui.home.v2.filter.OverseaActivityView
import com.alpha.lagouapk.ui.home.v2.filter.OverseaTabView
import com.alpha.lagouapk.ui.home.v2.filter.SelectOptionWrap
import com.alpha.lagouapk.ui.home.v2.filter.inter.FilterViewType
import com.alpha.lagouapk.ui.home.v2.filter.inter.SelectType
import com.common.ui.HorizontalScrollViewCompat
import com.lagou.annotation.aspect.SingleClick
import com.lagou.baselib.analytic.AnalyticHelper
import com.lagou.baselib.analytic.bean.json.ShowLogInfo
import com.lagou.baselib.analytic.definition.c.PageId
import com.lagou.commonlibs.imageload.ImageLoadUtils
import com.lagou.commonlibs.imageload.glide.transform.RoundTransform
import com.lagou.commonlibs.utils.BaseUtils
import com.lagou.commonlibs.utils.DeviceUtils
import com.lagou.commonlibs.utils.ImageUrlUtils
import com.lagou.commonlibs.utils.ListUtil
import com.lagou.commonlibs.utils.LogUtils
import com.lagou.commonlibs.utils.ResourceUtils

/**
 * 首页出海活动头部view
 */
class HomeOverseaHeadView @JvmOverloads constructor(
    context: Context,
    attrs: AttributeSet? = null,
    defStyleAttr: Int = 0
) : FrameLayout(context, attrs, defStyleAttr) {
    private lateinit var mFilterViewContainer: LinearLayout
    private lateinit var mIvOverseaCover: ImageView
    private lateinit var mActivityContainer:LinearLayout
    private lateinit var mSvActivity:HorizontalScrollViewCompat
    private lateinit var mSvFilter:HorizontalScrollView
    private var mSelectOptionMap: HashMap<String?, SelectOptionWrap?> = HashMap(2)
    private var mOnConfirmListener: ((HashMap<String?, List<String>?>) -> Unit)? = null
    private var mActivityCode:String? = null
    private var mThemeColor:String? = null
    private var mOnActivityChanged:((String?)->Unit)? = null
    private var mActivityCodeList:List<String?>? = null
    private var mActivityVisibilityMap:ArrayMap<Int,Boolean> = ArrayMap(4)//每个活动是否在屏幕内，即是否可见
    companion object{
        const val MAX_SHOW_ACTIVITY_COUNT_IN_SCREEN = 3//一屏最多展示三个入口
    }
    init {
        inflate(context, R.layout.view_oversear_head,this)
        initView()
    }

    private fun initView() {
        mFilterViewContainer = findViewById(R.id.ll_filter_container)
        mIvOverseaCover = findViewById(R.id.iv_oversea_activity)
        mActivityContainer = findViewById(R.id.ll_activity_container)
        mSvActivity = findViewById(R.id.hsv_activity_container)
        mSvFilter = findViewById(R.id.hsv_oversea_head)
       val params = (mIvOverseaCover.layoutParams as LinearLayout.LayoutParams).apply {
           width = DeviceUtils.getScreenWidth() - ResourceUtils.getDimenSize(R.dimen.dp_30)
           height = (width * 165f / 345).toInt()
       }
        mIvOverseaCover.layoutParams = params
        mSvActivity.setOnScrollChangeListener(object:HorizontalScrollViewCompat.OnScrollChangeListener{
            override fun onScrollChanged(
                scrollView: HorizontalScrollView?, l: Int, t: Int, oldl: Int, oldt: Int) {
                checkChildViewsVisibility()
            }

        })
    }

    /**
     * 检查活动入口的可见性
     */
    private fun checkChildViewsVisibility() {
        val scrollViewLocation = IntArray(2)
        mSvActivity.getLocationOnScreen(scrollViewLocation)
        val scrollViewLeft = scrollViewLocation[0]
        val scrollViewRight = scrollViewLeft + mSvActivity.width
        for(i in 0 until mActivityContainer.childCount){
            val childView = mActivityContainer.getChildAt(i)
            val viewLocation = IntArray(2)
            childView.getLocationOnScreen(viewLocation)
            val viewLeft = viewLocation[0]
            val viewRight = viewLeft + childView.width
            val isVisible = viewRight > scrollViewLeft && viewLeft < scrollViewRight
            if (mActivityVisibilityMap[i] != true && isVisible){
                //从不可见到可见上报
                AnalyticHelper.getInstance().sendShowInfo(ShowLogInfo().setPage_Id(PageId.ea00).setAddress_Id("m1ujf").addShowProp("activityid",mActivityCodeList?.getOrNull(i)))
            }
            mActivityVisibilityMap[i] = isVisible
        }
    }

    fun setHeadData(config: HomeActivityConfig?):HomeOverseaHeadView{
        mActivityCode = config?.activities?.first()?.activityCode
        mThemeColor = config?.activities?.first()?.themeColor
        mActivityCodeList = config?.activities?.map { it.activityCode }
        showActivityListUI(config?.activities)
        return this
    }

    /**
     * 活动列表
     */
    private fun showActivityListUI(activities: List<HomeItemActivityConfig>?) {
        if (ListUtil.size(activities) <= 1){
            mSvActivity.visibility = View.GONE
            return
        }
        mActivityContainer.removeAllViews()
        mSvActivity.visibility = View.VISIBLE
        val activityCount = activities!!.size
        val itemGap = if (activityCount == 2) ResourceUtils.getDimenSize(R.dimen.dp_14) else ResourceUtils.getDimenSize(R.dimen.dp_6)
        val itemWidth = calItemViewWidth(activityCount)
        activities.forEachIndexed { index, homeItemActivityConfig ->
            val activityEntryView = OverseaActivityView(context)
            activityEntryView.layoutParams = LinearLayout.LayoutParams(itemWidth,LinearLayout.LayoutParams.WRAP_CONTENT).apply {
                rightMargin = if (index < activityCount - 1) itemGap else 0
            }
            activityEntryView.tag = homeItemActivityConfig.activityCode
            activityEntryView.setData(homeItemActivityConfig)
            activityEntryView.setOnClickListener {
                if (mActivityCode == homeItemActivityConfig.activityCode){
                    return@setOnClickListener
                }
                mActivityCode = homeItemActivityConfig.activityCode
                mThemeColor = homeItemActivityConfig.themeColor
                setActivityItemBackground()
                mOnActivityChanged?.invoke(mActivityCode)
                AnalyticHelper.getInstance().sendClickInfo(PageId.ea00,"m1ujf", arrayOf("activityid"), arrayOf(mActivityCode))
            }
            mActivityContainer.addView(activityEntryView)
        }
        mSvActivity.post { checkChildViewsVisibility() }
        setActivityItemBackground()
    }

    private fun calItemViewWidth(activityCount:Int): Int {
        val availableWidth = DeviceUtils.getScreenWidth() - ResourceUtils.getDimenSize(R.dimen.dp_30)
        val itemGap = if (activityCount == 2) ResourceUtils.getDimenSize(R.dimen.dp_14) else ResourceUtils.getDimenSize(R.dimen.dp_6)
        val activityLogicCount = MAX_SHOW_ACTIVITY_COUNT_IN_SCREEN.coerceAtMost(activityCount)
        return (availableWidth - (activityLogicCount - 1) * itemGap) / activityLogicCount
    }

    /**
     * 设置活动入口背景
     */
    private fun setActivityItemBackground(){
        val activityThemeColor = if (BaseUtils.checkColorStrIsValid(mThemeColor)) Color.parseColor(mThemeColor) else ResourceUtils.getColor(R.color.color_00b38a)
        val alpha = (255 * 0.05).toInt()
        val red = Color.red(activityThemeColor)
        val green = Color.green(activityThemeColor)
        val blue = Color.blue(activityThemeColor)
        val childCount = mActivityContainer.childCount
        for(i in 0 until childCount){
            val gradientDrawable = GradientDrawable()
            gradientDrawable.shape = GradientDrawable.RECTANGLE
            gradientDrawable.cornerRadius = ResourceUtils.getDimenSize(R.dimen.dp_2).toFloat()
            gradientDrawable.setStroke(ResourceUtils.getDimenSize(R.dimen.dp_05), ResourceUtils.getColor(R.color.color_e8e9eb))
            val childView = mActivityContainer.getChildAt(i)
            if (childView.tag == mActivityCode){
                gradientDrawable.setColor(Color.argb(alpha,red,green,blue))
                (childView as OverseaActivityView).setSelectColor(activityThemeColor)
            }else{
                gradientDrawable.setColor(ResourceUtils.getColor(R.color.color_ffffff))
                (childView as OverseaActivityView).setNormalColor()
            }
            childView.background = gradientDrawable
        }
    }

    /**
     * tab点击事件
     */
    @SingleClick
    private fun onTabClick(tabView:OverseaTabView,filterData: FilterData){
        val filterView = getFilterContentView(filterData.viewStyle)?.apply {
            val dialog = HomeFilterDialog(context,this)
            dialog.show()
            this.setFilterData(filterData,mSelectOptionMap[filterData.selectKey])
            this.setOnCloseListener { dialog.dismiss() }
            this.setOnConfirmListener { selectOptionWrap:SelectOptionWrap? ->
                mSelectOptionMap[filterData.selectKey] = selectOptionWrap
                setTabStyle(tabView,filterData,selectOptionWrap)
                dialog.dismiss()
                var selectMap:HashMap<String?,List<String>?> = HashMap(2)
                mSelectOptionMap.entries.forEach { entry ->
                    when(val selectWrap = entry.value){
                        is SelectOptionWrap.Single -> {
                            val selectOption:SelectOption =  selectWrap.option
                            selectMap[selectOption.key] = selectOption.selectOptions?.map { filterOptionData -> filterOptionData.selectValue ?: "" }
                        }
                        is SelectOptionWrap.Multiple ->{
                            val selectOptionList =  selectWrap.option
                            selectOptionList.forEach {
                                selectMap[it.key] = it.selectOptions?.map { filterOptionData -> filterOptionData.selectValue ?: "" }
                            }
                        }
                    }
                }
                LogUtils.d("Leon","confirm ${selectMap}")
                mOnConfirmListener?.invoke(selectMap)
            }
        }
        AnalyticHelper.getInstance().sendClickInfo(PageId.ea00,"z1uj3", arrayOf("entrance","activityid"), arrayOf(filterData.selectName,mActivityCode))
    }

    fun setOnConfirmListener(onConfirmListener:((Map<String?,List<String>?>) -> Unit)?):HomeOverseaHeadView{
        this.mOnConfirmListener = onConfirmListener
        return this
    }


    /**
     * 设置tab样式
     */
    private fun setTabStyle(tabView:OverseaTabView, filterData: FilterData, selectOptionWrap: SelectOptionWrap?) {
        var tabTitle = filterData.selectName
        when(filterData.viewStyle){
            FilterViewType.FLAT.type,FilterViewType.NAVIGATE.type -> {
                val selectOptions = (selectOptionWrap as? SelectOptionWrap.Single)?.option?.selectOptions
                if (selectOptions.isNullOrEmpty().not()){
                    tabTitle = if (filterData.selectType == SelectType.MULTI_CHOICE.type) "${filterData.selectName} ${selectOptions!!.size}" else selectOptions!![0].selectTitle
                }
            }
            FilterViewType.AGGREGATE.type,FilterViewType.AGGREGATE_NO_NAV.type -> {
                //聚合，支持跨级多选
                val selectOptions = (selectOptionWrap as? SelectOptionWrap.Multiple)?.option
                val selectOptionCount = selectOptions?.sumBy { it.selectOptions?.size ?: 0 } ?: 0
                if (selectOptionCount > 0){
                    tabTitle = "${filterData.selectName} $selectOptionCount"
                }
            }
        }
        val isDefault = tabTitle == filterData.selectName
        val activityThemeColor = if (BaseUtils.checkColorStrIsValid(mThemeColor)) Color.parseColor(mThemeColor) else ResourceUtils.getColor(R.color.color_00b38a)
        tabView.setTitleColor(if (isDefault) ResourceUtils.getColor(R.color.color_333) else activityThemeColor)
        tabView.setTitle(tabTitle)
        tabView.setIndicatorColor(if (isDefault) ResourceUtils.getColor(R.color.color_333) else activityThemeColor)
        tabView.setBackgroundStyle(activityThemeColor)
        tabView.setIcon(filterData.selectLogo)
    }


    /**
     * 根据筛选器层级选择对应的筛选器内容区
     * 目前只适配最多两层
     */
    private fun getFilterContentView(viewType:String?): FilterLevelView?{
        var filterContentView: FilterLevelView? = null
        when(viewType){
            FilterViewType.FLAT.type -> {
                filterContentView = FlatFilterView(context)
            }
            FilterViewType.NAVIGATE.type -> {
                filterContentView = NavigateFilterView(context)
            }
            FilterViewType.AGGREGATE.type ->{
                filterContentView = NavAggregateFilterView(context)
            }
            FilterViewType.AGGREGATE_NO_NAV.type ->{
                filterContentView = AggregateFilterView(context)
            }
        }
        return filterContentView
    }

    fun setOnActivityChangedListener(onActivityChangedListener:((String?)->Unit)?){
        this.mOnActivityChanged = onActivityChangedListener
    }

    /**
     * 设置banner和筛选项
     */
    fun showBannerAndFilter(config: HomeItemActivityConfig?) {
        if (config == null){
            mIvOverseaCover.visibility = View.GONE
            mSvFilter.visibility = View.GONE
            return
        }
        mIvOverseaCover.visibility = View.VISIBLE
        mSvFilter.visibility = View.VISIBLE
        ImageLoadUtils.with(context)
            .load<String>(ImageUrlUtils.getLagouImageUrl(config.bannerImg))
            .bitmapTransform(RoundTransform(4))
            .dontAnimate()
            .into(mIvOverseaCover)
        mFilterViewContainer.removeAllViews()
        val options = config.selectConfigList
        mFilterViewContainer.visibility = if (options.isNullOrEmpty()) View.GONE else View.VISIBLE
        options?.forEachIndexed { index, filterData ->
            val tabMarginRight =
                if (index != options.size - 1) ResourceUtils.getDimenSize(R.dimen.dp_15) else 0
            val tabView = OverseaTabView(context).apply {
                layoutParams = LinearLayout.LayoutParams(
                    LinearLayout.LayoutParams.WRAP_CONTENT,
                    LinearLayout.LayoutParams.WRAP_CONTENT
                ).apply { marginEnd = tabMarginRight }
            }
            setTabStyle(tabView, filterData, mSelectOptionMap[filterData.selectKey])
            mFilterViewContainer.addView(tabView)
            tabView.setOnClickListener { onTabClick(tabView, filterData) }
        }
    }

    fun resetCacheSelectMap(){
        mSelectOptionMap.clear()
    }
}