package com.alpha.lagouapk.ui.home.v2.filter

import android.content.Context
import android.util.AttributeSet
import android.widget.Button
import android.widget.LinearLayout
import androidx.recyclerview.widget.RecyclerView
import com.alpha.lagouapk.R
import com.alpha.lagouapk.bean.SelectOption
import com.alpha.lagouapk.bean.config.FilterData
import com.alpha.lagouapk.bean.config.FilterOptionData
import com.lagou.commonlibs.utils.ResourceUtils
import com.lagou.commonlibs.view.LTitleBarLayout

/**
 * 聚合筛选器，包含期望薪资、工作经验、学历要求等
 */
class AggregateFilterView  @JvmOverloads constructor(
    context: Context, attrs: AttributeSet? = null,
    defStyleAttr: Int = 0
) : FilterLevelView(context, attrs, defStyleAttr){
    private lateinit var mTitleBar:LTitleBarLayout
    private lateinit var mBtnReset: Button
    private lateinit var mBtnConfirm: Button
    private lateinit var mModuleFilterViewContainer:LinearLayout
    init {
        inflate(context, R.layout.filter_aggregate, this)
        this.initView()
        this.initListener()
    }

    private fun initListener() {
        mTitleBar.setLeftImgListener {
            mOnCloseClickListener?.invoke()
        }
        mBtnReset.setOnClickListener {
            val childCount =  mModuleFilterViewContainer.childCount
            for (i in 0 until childCount){
                val childView = mModuleFilterViewContainer.getChildAt(i) as ModuleFilterView
                childView.resetSelect()
            }
            showSelectNum(0)
        }
        mBtnConfirm.setOnClickListener {
            var selectOptionWrap:SelectOptionWrap? = null
            val childCount =  mModuleFilterViewContainer.childCount
            var selectOptionList:MutableList<SelectOption>? = null
            for (i in 0 until childCount){
                val childView = mModuleFilterViewContainer.getChildAt(i) as ModuleFilterView
                if (childView.getSelectOption().isNotEmpty()) {
                    val selectOption = SelectOption(childView.getSelectOptionKey(), childView.getSelectOption())
                    if (selectOptionList == null){
                        selectOptionList = mutableListOf()
                    }
                    selectOptionList.add(selectOption)
                }
            }
            if (selectOptionList.isNullOrEmpty().not()){
                selectOptionWrap = SelectOptionWrap.Multiple(selectOptionList!!)
            }
            mOnConfirmClickListener?.invoke(selectOptionWrap)
        }

    }

    private fun initView() {
        mTitleBar = findViewById(R.id.title_bar_filter)
        mModuleFilterViewContainer = findViewById(R.id.ll_module_filter_view)
        mBtnReset = findViewById(R.id.filter_clear)
        mBtnConfirm = findViewById(R.id.filter_confirm)
    }

    override fun setFilterData(filterData: FilterData?, selectOption: SelectOptionWrap?) {
        mTitleBar.setCenterTvTxt(filterData?.selectTitle)
        addModuleFilterView(filterData?.aggregationSelectConfigs,selectOption)
        setSelectNum()
    }

    private fun addModuleFilterView(filterDataList: List<FilterData>?, selectOptionWrap: SelectOptionWrap?) {
        mModuleFilterViewContainer.removeAllViews()
        if (filterDataList.isNullOrEmpty()) return
        for (i in 0 until filterDataList!!.size){
            val modelFilterView = ModuleFilterView(context)
            modelFilterView.layoutParams = LinearLayout.LayoutParams(RecyclerView.LayoutParams.MATCH_PARENT,RecyclerView.LayoutParams.WRAP_CONTENT).apply {
                bottomMargin = if (i == filterDataList.size - 1) 0 else ResourceUtils.getDimenSize(R.dimen.dp_30)
            }
            val filterData = filterDataList[i]
            modelFilterView.setFilterOptionData(filterData.selectKey,filterData.selectTitle,filterData.selectType,
                filterData.selectValuesConfigs,getSelectFilterOptionByKey(filterData.selectKey,selectOptionWrap)){
                setSelectNum()
            }
            mModuleFilterViewContainer.addView(modelFilterView)
        }
    }

    /**
     * 每个module 选中项
     */
    private fun getSelectFilterOptionByKey(key:String?,selectOptionWrap: SelectOptionWrap?):List<FilterOptionData>?{
        return (selectOptionWrap as? SelectOptionWrap.Multiple)?.option?.find { it.key == key }?.selectOptions
    }

    private fun setSelectNum(){
        var selectCount = 0
        val childCount =  mModuleFilterViewContainer.childCount
        for (i in 0 until childCount){
            val childView = mModuleFilterViewContainer.getChildAt(i) as ModuleFilterView
            selectCount += childView.getSelectOption().size
        }
        //筛选项点击回调
        showSelectNum(selectCount)
    }

    /**
     * 展示选中的数量
     */
    private fun showSelectNum(selectNum:Int) {
        mBtnConfirm.text = if (selectNum == 0) ResourceUtils.getString(R.string.ok) else "${
            ResourceUtils.getString(R.string.ok)
        } $selectNum"
    }
}