package com.alpha.lagouapk.ui.home.v2.filter

import android.content.Context
import android.graphics.Rect
import android.util.AttributeSet
import android.view.View
import android.widget.LinearLayout
import android.widget.ScrollView
import android.widget.TextView
import androidx.recyclerview.widget.GridLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.alpha.lagouapk.R
import com.alpha.lagouapk.bean.config.FilterOptionData
import com.alpha.lagouapk.ui.home.v2.filter.adapter.ModuleFilterAdapter
import com.alpha.lagouapk.ui.home.v2.filter.inter.SelectType
import com.lagou.commonlibs.utils.ResourceUtils

/**
 * 一个模块的筛选器，如薪资、学历、工作经验等
 */
class ModuleFilterView @JvmOverloads constructor(
    context: Context, attrs: AttributeSet? = null,
    defStyleAttr: Int = 0
) : LinearLayout(context, attrs, defStyleAttr) {
    private lateinit var mTvTitle: TextView
    private lateinit var mRecyclerView: RecyclerView
    private lateinit var mLLTitleContainer:LinearLayout
    private lateinit var mTvSingleChoiceLabel:TextView
    private lateinit var mFilterAdapter:ModuleFilterAdapter
    private var mOnSelectOptionNumCallback:((Int)->Unit)? = null
    private var mSelectType:String = SelectType.SINGLE_CHOICE.type
    private var mSelectKey:String? = null
    private val mOptionItemMinWidth = ResourceUtils.getDimenSize(R.dimen.dp_100)
    init {
        inflate(context, R.layout.module_filter_view,this)
        orientation = LinearLayout.VERTICAL
        this.initView()
    }

    private fun initView() {
        mTvTitle = findViewById(R.id.tv_filter_title)
        mRecyclerView = findViewById(R.id.recycler_view_filter)
        mTvSingleChoiceLabel = findViewById(R.id.tv_filter_title_single_choice)
        mLLTitleContainer = findViewById(R.id.ll_title_container)
        mFilterAdapter = ModuleFilterAdapter(context){
            //点击item的回调
            this.mOnSelectOptionNumCallback?.invoke(mFilterAdapter.getSelectOption().size)
        }
        mRecyclerView.post {
            //计算列数
            val availableWidth = mRecyclerView.width - mRecyclerView.paddingLeft - mRecyclerView.paddingRight
            var column = 0
            val itemGap = ResourceUtils.getDimenSize(R.dimen.dp_10)
            column = if (3 * mOptionItemMinWidth + 2 * itemGap <= availableWidth){ 3 }else{ 2 }//3列能展示开展示3列，否则展示2列
            val layoutManager = GridLayoutManager(context, column)
            mRecyclerView.layoutManager = layoutManager
            mRecyclerView.addItemDecoration(GridSpaceItemDecoration(column,itemGap))
            mRecyclerView.adapter = mFilterAdapter
        }

    }

    fun setFilterOptionData(selectKey:String?,selectTitle:String?,selectType:String?, optionData:List<FilterOptionData>?,selectOptionData:List<FilterOptionData>?, onSelectOptionNumCallback:((Int)->Unit)?){
        this.mOnSelectOptionNumCallback = onSelectOptionNumCallback
        mTvTitle.text = selectTitle
        mLLTitleContainer.visibility = if (selectTitle.isNullOrEmpty()) View.GONE else View.VISIBLE
        this.mSelectType = selectType ?: SelectType.SINGLE_CHOICE.type
        this.mSelectKey = selectKey
        mTvSingleChoiceLabel.visibility = if (mSelectType == SelectType.SINGLE_CHOICE.type) View.VISIBLE else View.GONE
        mFilterAdapter.setFilterOptionData(optionData,selectOptionData,mSelectType)
    }

    fun setInitSelectNum(){
        this.mOnSelectOptionNumCallback?.invoke(mFilterAdapter.getSelectOption().size)
    }

    /**
     * 重置
     */
    fun resetSelect() {
        mFilterAdapter.resetSelect()
        mFilterAdapter.notifyDataSetChanged()
    }

    /**
     * 选中的筛选项
     */
    fun getSelectOption(): List<FilterOptionData> = mFilterAdapter.getSelectOption()

    /**
     * 筛选对应的key
     */
    fun getSelectOptionKey():String? = mSelectKey

    class GridSpaceItemDecoration(private val columnCount:Int,private val space:Int):RecyclerView.ItemDecoration(){
        override fun getItemOffsets(
            outRect: Rect,
            view: View,
            parent: RecyclerView,
            state: RecyclerView.State
        ) {
            val position = parent.getChildAdapterPosition(view) // item position
            val layoutManager = parent.layoutManager as GridLayoutManager
            val spanSizeLookup = layoutManager.spanSizeLookup
            val spanIndex = spanSizeLookup.getSpanIndex(position, columnCount)
            val spanSize = spanSizeLookup.getSpanSize(position)
            val totalSpanSize = spanSizeLookup.getSpanGroupIndex(position,columnCount)
            outRect.left = spanIndex * space / columnCount
            outRect.right = space - (spanSize + spanIndex) * space / columnCount
            outRect.bottom = 0
            if (totalSpanSize > 0){
                outRect.top = space
            }

        }
    }
}