package com.alpha.lagouapk.ui.home.v2.filter

import android.content.Context
import android.util.AttributeSet
import android.widget.Button
import android.widget.LinearLayout
import androidx.core.content.ContextCompat
import androidx.core.widget.NestedScrollView
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.alpha.lagouapk.R
import com.alpha.lagouapk.bean.SelectOption
import com.alpha.lagouapk.bean.config.FilterData
import com.alpha.lagouapk.bean.config.FilterOptionData
import com.alpha.lagouapk.ui.home.v2.filter.adapter.NavigateLeftFilterAdapter
import com.lagou.commonlibs.utils.ResourceUtils
import com.lagou.commonlibs.view.LTitleBarLayout

/**
 * 带导航聚合筛选器，包含期望薪资、工作经验、学历要求等
 */
class NavAggregateFilterView  @JvmOverloads constructor(
    context: Context, attrs: AttributeSet? = null,
    defStyleAttr: Int = 0
) : FilterLevelView(context, attrs, defStyleAttr){
    private lateinit var mTitleBar:LTitleBarLayout
    private lateinit var mBtnReset: Button
    private lateinit var mBtnConfirm: Button
    private lateinit var mModuleFilterViewContainer:LinearLayout
    private lateinit var mRecyclerViewLeftNav:RecyclerView
    private lateinit var mRvFilterContent:NestedScrollView
    private lateinit var mNavAdapter: NavigateLeftFilterAdapter
    private var mFilterDataList: List<FilterData>? = null
    private val mChildTop:MutableList<Int> = mutableListOf()//每个module与顶部的距离，用于滑动时锚点
    private var mAnchorIndex = 0
    init {
        inflate(context, R.layout.filter_nav_aggregate, this)
        this.initView()
        this.initListener()
    }

    private fun initListener() {
        mTitleBar.setLeftImgListener {
            mOnCloseClickListener?.invoke()
        }
        mBtnReset.setOnClickListener {
            val childCount =  mModuleFilterViewContainer.childCount
            for (i in 0 until childCount){
                val childView = mModuleFilterViewContainer.getChildAt(i) as ModuleFilterView
                childView.resetSelect()
            }
            showSelectNum(0)
        }
        mBtnConfirm.setOnClickListener {
            var selectOptionWrap:SelectOptionWrap? = null
            val childCount =  mModuleFilterViewContainer.childCount
            var selectOptionList:MutableList<SelectOption>? = null
            for (i in 0 until childCount){
                val childView = mModuleFilterViewContainer.getChildAt(i) as ModuleFilterView
                if (childView.getSelectOption().isNotEmpty()) {
                    val selectOption = SelectOption(childView.getSelectOptionKey(), childView.getSelectOption())
                    if (selectOptionList == null){
                        selectOptionList = mutableListOf()
                    }
                    selectOptionList.add(selectOption)
                }
            }
            if (selectOptionList.isNullOrEmpty().not()){
                selectOptionWrap = SelectOptionWrap.Multiple(selectOptionList!!)
            }
            mOnConfirmClickListener?.invoke(selectOptionWrap)
        }
        mRvFilterContent.setOnScrollChangeListener(onScrollChangeListener)
    }

    private val onScrollChangeListener= NestedScrollView.OnScrollChangeListener { _: NestedScrollView?, _: Int, scrollY: Int, _: Int, _: Int ->
        val newAnchorIndex = getAnchorIndex(scrollY)
        if (mAnchorIndex != newAnchorIndex){
            mAnchorIndex = newAnchorIndex
            mNavAdapter.updateSelectPosition(mAnchorIndex)
        }
    }

    private fun getAnchorIndex(scrollY:Int):Int{
        val childCount = mChildTop.size
        for (i in 0 until childCount){
            if (i == childCount - 1){
                return i
            }
            when(scrollY){
                in mChildTop[i]..mChildTop[i+1] -> return i
            }
        }
       return 0
    }

    private fun initView() {
        mTitleBar = findViewById(R.id.title_bar_filter)
        mModuleFilterViewContainer = findViewById(R.id.ll_module_filter_view)
        mBtnReset = findViewById(R.id.filter_clear)
        mBtnConfirm = findViewById(R.id.filter_confirm)
        mRecyclerViewLeftNav = findViewById(R.id.recycler_nav_aggregate_filter_nav)
        mRvFilterContent = findViewById(R.id.rv_nav_aggregate_filter_content)
        mRecyclerViewLeftNav.layoutManager = LinearLayoutManager(context)
        mNavAdapter = NavigateLeftFilterAdapter(context, ContextCompat.getColorStateList(context, R.color.color_666_2_00b38a)){
            //导航栏点击
            var specIndex:Int = 0
            run out@{
                this.mFilterDataList?.forEachIndexed { index, filterData ->
                    if (filterData.selectName == it.selectTitle){
                        specIndex = index
                        return@out
                    }
                }
            }
            scrollToSpecModuleFilter(specIndex)
        }
        mRecyclerViewLeftNav.adapter = mNavAdapter
    }

    /**
     * 根据左侧的分类，滑动到指定的筛选模块
     */
    private fun scrollToSpecModuleFilter(selectIndex:Int){
        mRvFilterContent.setOnScrollChangeListener(null as NestedScrollView.OnScrollChangeListener?)
        mAnchorIndex = selectIndex
        mRvFilterContent.scrollTo(0,mModuleFilterViewContainer.getChildAt(selectIndex).top)
        mRvFilterContent.setOnScrollChangeListener(onScrollChangeListener)
    }

    override fun setFilterData(filterData: FilterData?, selectOptionWrap: SelectOptionWrap?) {
        this.mFilterDataList = filterData?.aggregationSelectConfigs
        //第一个有选中的module
        val selectOption:SelectOption? = (selectOptionWrap as? SelectOptionWrap.Multiple)?.option?.first { it.selectOptions.isNullOrEmpty().not() }
        val firstSelectIndex = 0.coerceAtLeast(filterData?.aggregationSelectConfigs?.indexOfFirst { it.selectKey == selectOption?.key } ?: 0)
        val filterOptionDataList = filterData?.aggregationSelectConfigs?.map { FilterOptionData().apply { selectTitle = it.selectName } }
        mNavAdapter.setFilterOptionData(filterOptionDataList,firstSelectIndex)
        addModuleFilterView(selectOptionWrap)
        mRvFilterContent.post {
            for (i in 0 until mModuleFilterViewContainer.childCount){
              mChildTop.add(mModuleFilterViewContainer.getChildAt(i).top)
            }
            scrollToSpecModuleFilter(firstSelectIndex)

        }
        setSelectNum()
    }

    private fun addModuleFilterView(selectOptionWrap: SelectOptionWrap?) {
        mModuleFilterViewContainer.removeAllViews()
        if (mFilterDataList.isNullOrEmpty()) return
        for (i in 0 until mFilterDataList!!.size){
            val modelFilterView = ModuleFilterView(context)
            modelFilterView.layoutParams = LinearLayout.LayoutParams(RecyclerView.LayoutParams.MATCH_PARENT,RecyclerView.LayoutParams.WRAP_CONTENT).apply {
                bottomMargin = if (i == mFilterDataList!!.size - 1) 0 else ResourceUtils.getDimenSize(R.dimen.dp_30)
            }
            val filterData = mFilterDataList!![i]
            modelFilterView.setFilterOptionData(filterData.selectKey,filterData.selectTitle,filterData.selectType,
                filterData.selectValuesConfigs,getSelectFilterOptionByKey(filterData.selectKey,selectOptionWrap)){
                setSelectNum()
            }
            mModuleFilterViewContainer.addView(modelFilterView)
        }
    }

    /**
     * 每个module 选中项
     */
    private fun getSelectFilterOptionByKey(key:String?,selectOptionWrap: SelectOptionWrap?):List<FilterOptionData>?{
        return (selectOptionWrap as? SelectOptionWrap.Multiple)?.option?.find { it.key == key }?.selectOptions
    }

    private fun setSelectNum(){
        var selectCount = 0
        val childCount =  mModuleFilterViewContainer.childCount
        for (i in 0 until childCount){
            val childView = mModuleFilterViewContainer.getChildAt(i) as ModuleFilterView
            selectCount += childView.getSelectOption().size
        }
        //筛选项点击回调
        showSelectNum(selectCount)
    }

    /**
     * 展示选中的数量
     */
    private fun showSelectNum(selectNum:Int) {
        mBtnConfirm.text = if (selectNum == 0) ResourceUtils.getString(R.string.ok) else "${
            ResourceUtils.getString(R.string.ok)
        } $selectNum"
    }
}