package com.alpha.lagouapk.ui.home.v2.filter

import android.content.Context
import android.util.AttributeSet
import android.view.View
import android.widget.Button
import androidx.core.content.ContextCompat
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.alpha.lagouapk.R
import com.alpha.lagouapk.bean.SelectOption
import com.alpha.lagouapk.bean.config.FilterData
import com.alpha.lagouapk.bean.config.FilterOptionData
import com.alpha.lagouapk.ui.home.v2.filter.adapter.NavigateLeftFilterAdapter
import com.alpha.lagouapk.ui.home.v2.filter.adapter.SuggestAdapter
import com.alpha.lagouapk.ui.home.v2.filter.inter.SelectType
import com.common.afterTextChanged
import com.lagou.commonlibs.engine.ThreadUtil
import com.lagou.commonlibs.utils.InputMethodUtil
import com.lagou.commonlibs.utils.ResourceUtils
import com.lagou.commonlibs.view.ClearEditText
import com.lagou.commonlibs.view.LTitleBarLayout
import com.lagou.commonlibs.view.recyclerview.HfRecyclerView

/**
 * 二级数据的筛选器
 * 如城市、智能的筛选等
 * 不支持跨级多选，同一级可多选
 */
class NavigateFilterView @JvmOverloads constructor(
    context: Context, attrs: AttributeSet? = null,
    defStyleAttr: Int = 0
) : FilterLevelView(context, attrs, defStyleAttr) {
    private lateinit var mTitleBar: LTitleBarLayout
    private lateinit var mRvFirst: RecyclerView
    private lateinit var mModuleFilterView: ModuleFilterView
    private lateinit var mBtnReset: Button
    private lateinit var mBtnConfirm: Button
    private lateinit var mFirstAdapter: NavigateLeftFilterAdapter
    private lateinit var mEditText:ClearEditText
    private lateinit var mRvSuggest:HfRecyclerView
    private var mFirstSelectTitle:String? = null //选中的一级类的标题
    private var mFilterData:FilterData? = null
    private lateinit var mSuggestAdapter:SuggestAdapter
    init {
        inflate(context, R.layout.filter_level_two, this)
        this.initView()
        this.initListener()
    }

    private fun initListener() {
        mTitleBar.setLeftImgListener {
            mOnCloseClickListener?.invoke()
        }
        mBtnReset.setOnClickListener {
            mModuleFilterView.resetSelect()
            showSelectNum(0)
        }
        mBtnConfirm.setOnClickListener {
            var selectOptionWrap:SelectOptionWrap? = null
            if (mModuleFilterView.getSelectOption().isNotEmpty()) {
                selectOptionWrap = SelectOptionWrap.Single(SelectOption(this.mFilterData?.selectKey, mModuleFilterView.getSelectOption(),this.mFirstSelectTitle))
            }
            mOnConfirmClickListener?.invoke(selectOptionWrap)
        }
        mEditText.afterTextChanged {
            getFilterOptionByInput(it?.toString()){key,result->
                mRvSuggest.visibility = if (key.isNullOrEmpty()) View.GONE else View.VISIBLE
                mSuggestAdapter.setSuggest(key,result)
            }
        }
    }

    /**
     * 根据输入值 查询筛选条件
     */
    private fun getFilterOptionByInput(key:String?,resultCallback:(String?,List<FilterOptionData>?) -> Unit){
        if (key.isNullOrEmpty()){
            resultCallback.invoke(null,null)
            return
        }
        val result:MutableList<FilterOptionData> = mutableListOf()
        ThreadUtil.compute {
            this.mFilterData?.selectValuesConfigs?.forEach {parentOption ->
                val parentTitle = parentOption.selectTitle
                parentOption.childSelectValues.forEach {childOption->
                   if (childOption.selectTitle?.toLowerCase()?.contains(key!!.toLowerCase()) == true){
                       childOption.parentTitle = parentTitle
                       result.add(childOption)
                   }
                }
            }
            ThreadUtil.ui { resultCallback.invoke(key,result) }
        }
    }

    private fun initView() {
        mTitleBar = findViewById(R.id.title_bar_filter)
        mRvFirst = findViewById(R.id.rv_first_filter)
        mModuleFilterView = findViewById(R.id.navigate_module_filter_view)
        mBtnReset = findViewById(R.id.filter_clear)
        mBtnConfirm = findViewById(R.id.filter_confirm)
        mEditText = findViewById(R.id.edit_navigate_filter)
        mRvSuggest = findViewById(R.id.rv_search_suggest)
        mRvFirst.layoutManager = LinearLayoutManager(context)
        mRvFirst.setHasFixedSize(true)
        mFirstAdapter = NavigateLeftFilterAdapter(context,
            ContextCompat.getColorStateList(context, R.color.selector_province_filter_color)) {
            this.mFirstSelectTitle = it.selectTitle
            mModuleFilterView.setFilterOptionData(mFilterData?.selectKey,null,this.mFilterData?.selectType,it.childSelectValues,null,onItemClickCallback)
            showSelectNum(0)
        }
        mRvFirst.adapter = mFirstAdapter

        mSuggestAdapter = SuggestAdapter(context){
            //suggest点击回调
            InputMethodUtil.hideSystemKeyBoard(context,mEditText)
            val isFirstSelectTitleChanged =  this.mFirstSelectTitle != it.parentTitle
            this.mFirstSelectTitle = it.parentTitle
            mEditText.setText("")
            val selectOptionList = mutableListOf<FilterOptionData>()
            if (isFirstSelectTitleChanged.not()){
                if (it.isExclusive){
                    //互斥，先清除所有
                    mModuleFilterView.resetSelect()
                }
                selectOptionList.addAll(mModuleFilterView.getSelectOption())
            }
            val isDuplication = selectOptionList.any { filterOptionData -> filterOptionData.selectValue == it.selectValue }
            if (isDuplication.not() && it.selectValue.isNullOrEmpty().not()){
                selectOptionList.add(it)
            }
            scrollToSelect(selectOptionList)
            showSelectNum(selectOptionList.size)
        }
        mRvSuggest.layoutManager = LinearLayoutManager(context)
        mRvSuggest.addItemDecoration(SuggestItemDecoration())
        mRvSuggest.adapter = mSuggestAdapter
        mRvSuggest.setEmptyView(R.layout.item_filter_empty)
    }


    override fun setFilterData(filterData: FilterData?, selectOptionWrap: SelectOptionWrap?) {
        this.mFilterData = filterData
        val selectOption:SelectOption? = (selectOptionWrap as? SelectOptionWrap.Single)?.option
        this.mFirstSelectTitle = if (selectOption?.cateName.isNullOrEmpty()) mFilterData?.selectValuesConfigs?.first()?.selectTitle else selectOption?.cateName
        mTitleBar.setCenterTvTxt(filterData?.selectTitle)
        mEditText.visibility = if (filterData?.isSupportSearch == true) View.VISIBLE else View.GONE
        scrollToSelect(selectOption?.selectOptions)
        mModuleFilterView.setInitSelectNum()
    }

    /**
     * 滚动到指定位置并选中筛选项
     */
    private fun scrollToSelect(selectOptionFilterData:List<FilterOptionData>?){
        val firstSelectIndex = 0.coerceAtLeast(mFilterData?.selectValuesConfigs?.indexOfFirst { it.selectTitle == this.mFirstSelectTitle } ?: 0)
        mFirstAdapter.setFilterOptionData(mFilterData?.selectValuesConfigs,firstSelectIndex)
        mModuleFilterView.setFilterOptionData(mFilterData?.selectKey,null, mFilterData?.selectType,
            mFilterData?.selectValuesConfigs?.getOrNull(firstSelectIndex)?.childSelectValues,
            selectOptionFilterData,onItemClickCallback)
        (mRvFirst.layoutManager as? LinearLayoutManager)?.scrollToPositionWithOffset(firstSelectIndex,0)
    }


    private val onItemClickCallback:((Int)->Unit)? = { selectNum->
        showSelectNum(if (mFilterData?.selectType == SelectType.MULTI_CHOICE.type) selectNum else 0)
    }

    /**
     * 展示选中的数量
     */
    private fun showSelectNum(selectNum: Int) {
        mBtnConfirm.text = if (selectNum == 0) ResourceUtils.getString(R.string.ok) else "${
            ResourceUtils.getString(R.string.ok)
        } $selectNum"
    }
}