package com.alpha.lagouapk.ui.home.v2.filter

import android.content.Context
import android.graphics.drawable.GradientDrawable
import android.util.AttributeSet
import android.view.View
import android.widget.FrameLayout
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.TextView
import androidx.annotation.ColorInt
import com.alpha.lagouapk.R
import com.lagou.commonlibs.imageload.ImageLoadUtils
import com.lagou.commonlibs.utils.ResourceUtils

class OverseaTabView @JvmOverloads constructor(
    context: Context,
    attrs: AttributeSet? = null,
    defStyleAttr: Int = 0
) : FrameLayout(context, attrs, defStyleAttr) {
    private lateinit var  tabIcon: ImageView
    private lateinit var  tabTitle: TextView
    private lateinit var  tabIndicator: ImageView
    private lateinit var  tabInnerContainer: LinearLayout
    init {
        inflate(context, R.layout.layout_oversea_tab,this)
        initView()
    }

    private fun initView() {
        tabIcon = findViewById(R.id.iv_filter_tab_icon)
        tabTitle = findViewById(R.id.tv_filter_tab_title)
        tabIndicator = findViewById(R.id.iv_filter_tab_indicator)
        tabInnerContainer = findViewById(R.id.ll_filter_tab_inner)
    }

    /**
     * 设置标题颜色
     */
    fun setTitleColor(@ColorInt color: Int):OverseaTabView {
        tabTitle.setTextColor(color)
        return this
    }
    /**
     * 设置标题
     */
    fun setTitle(title:String?):OverseaTabView{
        tabTitle.text = title
        return this
    }

    /**
     * 设置箭头的颜色
     */
    fun setIndicatorColor(@ColorInt color: Int):OverseaTabView {
        tabIndicator.setColorFilter(color)
        return this
    }

    /**
     * 设置背景
     */
    fun setBackgroundStyle(@ColorInt color: Int):OverseaTabView {
        val drawable = GradientDrawable()
        drawable.shape = GradientDrawable.RECTANGLE
        drawable.cornerRadius = ResourceUtils.getDimenSize(R.dimen.dp_18).toFloat()
        drawable.setColor(color)
        background = drawable
        return this
    }

    fun setIcon(url:String?):OverseaTabView{
        if (url.isNullOrEmpty()) tabIcon.visibility = View.GONE else View.VISIBLE
        ImageLoadUtils.with(context).load(url).into(tabIcon)
        return this
    }
}