package com.alpha.lagouapk.ui.home.v2.filter.adapter

import android.content.Context
import android.util.SparseBooleanArray
import android.view.LayoutInflater
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.TextView
import androidx.recyclerview.widget.GridLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.alpha.lagouapk.R
import com.alpha.lagouapk.bean.config.FilterOptionData
import com.alpha.lagouapk.ui.home.v2.filter.holder.ButtonFilterOptionViewHolder
import com.alpha.lagouapk.ui.home.v2.filter.holder.SeekBarFilterOptionViewHolder
import com.alpha.lagouapk.ui.home.v2.filter.inter.FilterOptionType
import com.alpha.lagouapk.ui.home.v2.filter.inter.SelectType
import com.lagou.commonlibs.utils.ResourceUtils

/**
 * 模块筛选器适配器
 */
class ModuleFilterAdapter(private val mContext: Context,
                          private val onItemClickListener:(()->Unit)?): RecyclerView.Adapter<RecyclerView.ViewHolder>() {
    private var filterOptionDataList:List<FilterOptionData>? = null
    private var mSelectType:String = SelectType.SINGLE_CHOICE.type
    private val optionSelectMap: SparseBooleanArray = SparseBooleanArray()
    companion object{
        const val SALARY_MIN = 1
        const val SALARY_MAX = 100
    }
    override fun onAttachedToRecyclerView(recyclerView: RecyclerView) {
        super.onAttachedToRecyclerView(recyclerView)
        val manager = recyclerView.layoutManager
        if (manager is GridLayoutManager){
            manager.spanSizeLookup = object : GridLayoutManager.SpanSizeLookup() {
                override fun getSpanSize(position: Int): Int {
                    return when (getItemViewType(position)) {
                        R.layout.item_filter_option_seek -> {
                            manager.spanCount
                        }
                        else -> {
                           1
                        }
                    }
                }
            }
        }
    }

    fun setFilterOptionData(filterOptionData:List<FilterOptionData>?, selectOptionData:List<FilterOptionData>?,selectType:String){
        this.filterOptionDataList = filterOptionData
        this.mSelectType = selectType
        initSelectOption(selectOptionData)
        notifyDataSetChanged()
    }

    private fun initSelectOption(selectData:List<FilterOptionData>?) {
        this.optionSelectMap.clear()
        val selectValue:List<String>? = selectData?.map { it.selectValue ?: "" }
        this.filterOptionDataList?.forEachIndexed { index, filterOptionData ->
            if (filterOptionData.style == FilterOptionType.BUTTON.type){
                val isSelect = selectValue?.contains(filterOptionData.selectValue) == true
                this.optionSelectMap.put(index,isSelect)
            }else{
                this.optionSelectMap.put(index,false)
            }
        }
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): RecyclerView.ViewHolder {
        val itemView = LayoutInflater.from(mContext).inflate(viewType, parent, false)
        var holder: RecyclerView.ViewHolder
        if (viewType == R.layout.item_filter_option_seek) {
            holder = SeekBarFilterOptionViewHolder(itemView)
            val seekbarOptionData:FilterOptionData? = filterOptionDataList?.firstOrNull() { it.style == FilterOptionType.SEEK_BAR.type }
            holder.seekBarSalary.setOnRangeSeekBarChangeListener{ _, minValue, maxValue ->
                seekbarOptionData?.selectValueMin = minValue
                seekbarOptionData?.selectValueMax = maxValue
                val defaultMinValue = seekbarOptionData?.slipMin ?: SALARY_MIN
                val defaultMaxValue = seekbarOptionData?.slipMax ?: SALARY_MAX
                (holder as SeekBarFilterOptionViewHolder).setSeekBarTopText(minValue,maxValue,defaultMinValue,defaultMaxValue,seekbarOptionData?.slipUnit ?: "")
                if(mSelectType == SelectType.SINGLE_CHOICE.type){
                   //单选的时候需要先清空筛选项
                    if (optionSelectMap.size() > 0){
                        val index = this.optionSelectMap.indexOfValue(true)
                        if (index > -1){
                            val position = optionSelectMap.keyAt(index)
                            notifyItemChanged(position)
                        }
                        this.optionSelectMap.clear()
                    }

               }
                onItemClickListener?.invoke()//回显数量
            }

        } else {
            holder = ButtonFilterOptionViewHolder(itemView)
            itemView.setOnClickListener {
                onItemClick(holder.adapterPosition, holder.tvOption, holder.ivOptionIcon)
            }
        }
        return holder
    }

    override fun getItemViewType(position: Int): Int {
        val itemStyle = filterOptionDataList?.get(position)?.style
        return if (itemStyle == FilterOptionType.SEEK_BAR.type) R.layout.item_filter_option_seek else R.layout.item_filter_option_simple
    }

    override fun getItemCount(): Int = filterOptionDataList?.size ?: 0

    override fun onBindViewHolder(holder: RecyclerView.ViewHolder, position: Int) {
        if (holder is ButtonFilterOptionViewHolder){
            holder.bindButton(filterOptionDataList,position, optionSelectMap.get(position))
        }else if (holder is SeekBarFilterOptionViewHolder){
            holder.bindSeekBar(filterOptionDataList,position)
        }
    }


    private fun onItemClick(position:Int,tvOption:TextView,ivOptionIcon:ImageView){
        var isExclusive = filterOptionDataList?.get(position)?.isExclusive == true//是否互斥
        this.optionSelectMap.put(position,optionSelectMap.get(position).not())
        tvOption.isSelected = optionSelectMap.get(position)
        ivOptionIcon.alpha = if (tvOption.isSelected) 1f else 0.3f
        if (tvOption.isSelected){
            if (mSelectType == SelectType.SINGLE_CHOICE.type || isExclusive){
                //单选或者与其他选项互斥
                val size = itemCount
                for (i in 0 until size) {
                    if (i != position){
                        optionSelectMap.put(i,false)
                    }
                }
            }else{
                //多选，需要将其他互斥项清空
                filterOptionDataList?.forEachIndexed { index, optionData ->
                    if (optionData.isExclusive){
                        optionSelectMap.put(index,false)
                    }
                }
            }
            notifyDataSetChanged()
        }
        if (mSelectType == SelectType.SINGLE_CHOICE.type){
            //单选的时候，点击按钮需要更新seekbar的显示，多显的时候，seekbar保持原样
            var seekbarOptionData:FilterOptionData? = filterOptionDataList?.firstOrNull() { it.style == FilterOptionType.SEEK_BAR.type }
            if (seekbarOptionData != null){
                val selectMinValue = filterOptionDataList?.get(position)?.selectValueMin ?: SALARY_MIN
                val selectMaxValue = filterOptionDataList?.get(position)?.selectValueMax ?: SALARY_MAX
                seekbarOptionData.selectValueMin = if (tvOption.isSelected) selectMinValue else SALARY_MIN
                seekbarOptionData.selectValueMax = if (tvOption.isSelected) selectMaxValue else SALARY_MAX
                notifyDataSetChanged()
            }

        }
        onItemClickListener?.invoke()
    }



    fun resetSelect(){
        this.optionSelectMap.clear()
        val mapSize = this.optionSelectMap.size()
        for (i in 0 until mapSize){
            this.optionSelectMap.put(i,false)
        }
        filterOptionDataList?.filter { it.style == FilterOptionType.SEEK_BAR.type }?.forEach {
            it.selectValueMax = it.slipMax
            it.selectValueMin = it.slipMin
        }
    }

    fun getSelectOption(): List<FilterOptionData> {
        val selectOption = mutableListOf<FilterOptionData>()
        this.filterOptionDataList?.forEachIndexed { index, optionData ->
            if(optionData.style == FilterOptionType.BUTTON.type){
                if (optionSelectMap.get(index) && optionData.selectValue.isNullOrEmpty().not()){
                    if (isDuplication(selectOption,optionData).not()){
                        selectOption.add(optionData)
                    }

                }
            }else if (optionData.style == FilterOptionType.SEEK_BAR.type){
                if (optionData.selectValueMin != optionData.slipMin || optionData.selectValueMax != optionData.slipMax){
                    optionData.selectValue = "${optionData.selectValueMin}${optionData.slipUnit}-${optionData.selectValueMax}${optionData.slipUnit}"
                    optionData.selectTitle = optionData.selectValue
                    if (isDuplication(selectOption,optionData).not()){
                        selectOption.add(optionData)
                    }
                }
            }

        }
        return selectOption
    }

    private fun isDuplication(selectOption: MutableList<FilterOptionData>, optionData: FilterOptionData): Boolean {
            return selectOption.any { it.selectValue == optionData.selectValue }
    }
}