package com.alpha.lagouapk.ui.home.v2.filter.adapter

import android.content.Context
import android.content.res.ColorStateList
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import androidx.recyclerview.widget.RecyclerView
import com.alpha.lagouapk.R
import com.alpha.lagouapk.bean.config.FilterOptionData

/**
 * 导航筛选器左侧适配器
 */
class NavigateLeftFilterAdapter(
    val context: Context,
    private val textColor:ColorStateList?,
    private val onItemClickListener: ((FilterOptionData) -> Unit)?
) : RecyclerView.Adapter<NavigateLeftFilterAdapter.NavigateLeftFilterViewHolder>() {
    private var filterOptionData:List<FilterOptionData>? = null
    private var mSelectPosition:Int = 0

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): NavigateLeftFilterViewHolder {
        val holder = NavigateLeftFilterViewHolder(LayoutInflater.from(context).inflate(R.layout.item_province_default, parent, false))
        holder.itemView.setOnClickListener {
            filterOptionData?.get(holder.adapterPosition)?.let {
                onItemClick(holder.adapterPosition)
            }
        }
        return holder
    }

    private fun onItemClick(position:Int){
        if (position == mSelectPosition){
            return
        }
        notifyItemChanged(mSelectPosition)//原先
        mSelectPosition = position
        notifyItemChanged(mSelectPosition)
        filterOptionData?.get(position)?.let { onItemClickListener?.invoke(it) }
    }

    override fun getItemCount(): Int = this.filterOptionData?.size ?: 0

    override fun onBindViewHolder(holder: NavigateLeftFilterViewHolder, position: Int) {
        val tvContent = holder.itemView as TextView
        tvContent.text = filterOptionData?.get(position)?.selectTitle
        tvContent.isSelected = position == mSelectPosition
        tvContent.paint.isFakeBoldText = position == mSelectPosition
    }

    fun setFilterOptionData(filterOptionData:List<FilterOptionData>?,initSelectIndex:Int) {
        this.filterOptionData = filterOptionData
        this.mSelectPosition = initSelectIndex
        notifyDataSetChanged()
    }

    fun updateSelectPosition(selectIndex:Int){
        this.mSelectPosition = selectIndex
        notifyDataSetChanged()
    }

    inner class NavigateLeftFilterViewHolder(itemView: View) : RecyclerView.ViewHolder(itemView){
        init {
            textColor?.let {
                (itemView as TextView).setTextColor(textColor)
            }
        }
    }
}