package com.alpha.lagouapk.ui.interview.ai

import android.animation.Animator
import android.animation.AnimatorListenerAdapter
import android.animation.ObjectAnimator
import android.graphics.Rect
import android.media.MediaPlayer
import android.os.Bundle
import android.text.TextUtils
import android.view.MotionEvent
import android.view.View
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.alpha.lagouapk.Interface.GetPositionDetailDataHandler
import com.alpha.lagouapk.R
import com.alpha.lagouapk.bean.PositionDetailBean
import com.alpha.lagouapk.bean.aiinterview.AIInterviewChatRes
import com.alpha.lagouapk.bean.aiinterview.AIInterviewCreateRes
import com.alpha.lagouapk.bean.aiinterview.AiContentType
import com.alpha.lagouapk.bean.aiinterview.AiInterviewAsrRes
import com.alpha.lagouapk.bean.aiinterview.AiInterviewFloatEntryRes
import com.alpha.lagouapk.bean.aiinterview.AiInterviewSessionListRes
import com.alpha.lagouapk.bean.aiinterview.AiInterviewSessionListRes.AiSessionItem
import com.alpha.lagouapk.bean.aiinterview.AiInterviewSessionListRes.RoleType
import com.alpha.lagouapk.bean.aiinterview.AiInterviewTtsRes
import com.alpha.lagouapk.bean.aiinterview.AiRelateMsgItemType
import com.alpha.lagouapk.bean.aiinterview.IAiInterviewSessionState
import com.alpha.lagouapk.bean.aiinterview.IAiInterviewStatus
import com.alpha.lagouapk.bean.aiinterview.IAiSessionContentType
import com.alpha.lagouapk.manager.CBusinessMemoryCacheManager
import com.alpha.lagouapk.manager.CMKVUtils
import com.alpha.lagouapk.manager.net.AiNetManager
import com.alpha.lagouapk.observer.EventTags
import com.alpha.lagouapk.tools.IntentUtils
import com.alpha.lagouapk.ui.interview.ai.adapter.AIInterviewAdapter
import com.alpha.lagouapk.ui.interview.ai.widget.GuideItemAnim
import com.alpha.lagouapk.ui.position.PositionDetailController
import com.lagou.annotation.apt.RouterRule
import com.lagou.baselib.BaseActivity
import com.lagou.baselib.analytic.AnalyticHelper
import com.lagou.baselib.analytic.bean.json.ClickInfo
import com.lagou.baselib.analytic.bean.json.ShowLogInfo
import com.lagou.baselib.net.httpnetback.BaseCallback
import com.lagou.baselib.net.httpnetback.BaseEventCallback
import com.lagou.baselib.net.httpnetback.OnAdapterCommonListenerImpl
import com.lagou.baselib.util.BarUtil
import com.lagou.chat.chatlibrary.bean.AiInterviewSession
import com.lagou.chat.chatlibrary.db.SessionDBManager
import com.lagou.chat.chatlibrary.manager.MediaManager
import com.lagou.commonlibs.engine.ThreadUtil
import com.lagou.commonlibs.event.Event
import com.lagou.commonlibs.event.OkBus
import com.lagou.commonlibs.inter.OnSingleMethodListener
import com.lagou.commonlibs.utils.DeviceUtils
import com.lagou.commonlibs.utils.FileUtils
import com.lagou.commonlibs.utils.ListUtil
import com.lagou.commonlibs.utils.LogUtils
import com.lagou.commonlibs.utils.ResourceUtils
import com.lagou.commonlibs.utils.SafeTypeConvert
import com.lagou.commonlibs.utils.StringUtil
import com.lagou.commonlibs.utils.ToastUtil
import com.lagou.socketchannel.http.HttpCallback
import com.lagou.socketchannel.http.HttpRequest
import com.lagou.socketchannel.http.HttpResponse
import com.lagou.socketchannel.http.event.HttpEventHelper
import com.lagou.socketchannel.http.event.HttpEventResponse
import com.lagou.socketchannel.util.GsonUtil
import kotlinx.android.synthetic.main.activity_ai_interview.*
import java.io.File
import java.util.LinkedList
import java.util.Timer
import java.util.TimerTask

/**
 * Time：2024/6/13 10:42:56
 * Author：wangjunwei
 * Description：AI 面试IM详情页（侧重于情感安慰）
 *
 */
@RouterRule("lagou://lagou.com/ai/interview")
class AiInterviewActivity : BaseActivity() {

    private var rootViewVisibleHeight: Int = 0

    /**
     * 当前请求记录，便于关闭页面取消
     */
    private var httpRequest: HttpRequest<AIInterviewChatRes>? = null

    val DP_6_05 = ResourceUtils.getDimenSize(R.dimen.dp_6_05)
    val DP_10 = ResourceUtils.getDimenSize(R.dimen.dp_10)
    val DP_15 = ResourceUtils.getDimenSize(R.dimen.dp_15)
    val DP_60 = ResourceUtils.getDimenSize(R.dimen.dp_60)

    private var layoutManagerCompact: LinearLayoutManager? = null
    private var aiInterviewAdapter: AIInterviewAdapter? = null

    /**
     * 是否正在执行滑动中
     */
    private var isSmoothScrolling = false

    /**
     * 记录当前Ai返回的文本，目前使用追加模式，为了实现打字效果和语音播报需要组合成完整文本
     */
    private var currentAiSessionItem: AiSessionItem? = null

    /**
     * Ai 播报待转化文本的下标 每次转换的文本数量
     */
    private var convertVoiceCurrentIndex = 0

    /**
     * 是否是生成的第一条语音
     */
    private var isFirstVoice = true

    /**
     * 用于记录生成语音的文本对象
     */
    private var aiSessionItemGenerateVoice: AiSessionItem = AiSessionItem()

    /**
     * Ai生成的内容是分段给出的，所以生成的音频文件也是分片的，存储一次Ai生成文本对应的音频文件
     */
    private val voiceFilePathQueue = LinkedList<String>()

    /**
     * 语音是否正在生成中
     */
    private var isVoiceGenerating = false

    /**
     * 文本你是否正在生成中
     */
    private var isTextGenerating = false

    /**
     * Ai 面试历史会话信息
     */
    private var aiInterviewSessionList: List<AiInterviewSessionListRes?>? = null

    /**
     * Ai 面试最新的session 信息
     */
    private var newestAiSession: AiInterviewSessionListRes? = null

    /**
     * 打字效果处理器
     */
    private var displayTextFromNetDisplayOffset: DelayedTextDisplayOffset? = null

    /**
     * 用户输入的文本
     */
    private var displayTextFromUserDisplayOffset: DelayedTextDisplayOffset? = null

    /**
     * 是否有手动滑动意图，有的话中断自动滑动（smoothToNewestContent），等到生成结束的时候在滑动到最后
     */
    private var isHandSlide = false

    /**
     * 是否是打印输出中，如果是键盘弹出和收起不在执行smooth方法
     */
    private var isPrintInputIng = false

    /**
     * timer定时器管理类
     */
    private val timerList = mutableListOf<Timer?>()

    private lateinit var sendButtonAnimator: ObjectAnimator

    /**
     * 从Ai面试浮动入口进来，携带一部分数据：
     * 注意处理未开始面试的情况：点进来不建立聊天，端上需要构建一个SessionList列表
     * aiInterviewFloatEntryRes中包含开场白、职位id
     * 已经开始面试或者面试结束不用单独处理，直接走现有流程，目前不支持多个职位同时开启Ai面试
     */
    private var aiInterviewFloatEntryRes: AiInterviewFloatEntryRes? = null

    /**
     * 播放音频
     */
    private val onCompletionListener = object : MediaPlayer.OnCompletionListener {
        override fun onCompletion(mp: MediaPlayer) {
            if (!CMKVUtils.isAutoConvertVoice() || voiceFilePathQueue.isEmpty()) {
                //播放完成，播放流程就断了，后续语音在识别成功也不会播放，所以要把流程续上
                isFirstVoice = true
                return
            }
            val voiceFilePath = voiceFilePathQueue.poll()
            //LogUtils.i("aiGeneratingAutoPlay", "播放：2 voiceFilePath = $voiceFilePath")
            MediaManager.playSound(voiceFilePath, this, this@AiInterviewActivity)
        }
    }

    /**
     * 选择音色并开启自动播放
     */
    private val openSelectTimbreEvent = Event { msg ->
        if (msg.obj is Boolean){
            image_right_play.isSelected = msg.obj as Boolean
            CMKVUtils.setAutoConvertVoice(image_right_play.isSelected)
        }
    }

    /**
     * 1 语音页面交互同步到文字和语音混合界面 - 用户问题更新显示(暂时统一刷新会话列表)
     */
    private val fromVoiceUpdateUserInputEvent = Event {
        val answer = it.obj as? String
        if (!TextUtils.isEmpty(answer)) {
            fromVoiceUserInputToShow(answer)
        }
        currentAiSessionItem = null
    }

    /**
     * 把用户输入更新到页面上(暂时统一刷新会话列表)
     */
    private fun fromVoiceUserInputToShow(answer: String?) {

        //1.1 把用户输入的文本，更新到聊天窗口，用户自己输入文本，不做打字效果
        val aiSessionItem = AiSessionItem.createUserText(
            IAiInterviewStatus.SUCCESS,
            answer,
            newestAiSession?.sessionId ?: 0,
            newestAiSession?.positionId ?: 0,
            HttpEventHelper.getInstance().clientTag
        )
        aiInterviewAdapter?.updateInterviewContent(aiSessionItem)
        forceSmoothToNewestContent()
    }

    /**
     * 2 语音页面交互同步到文字和语音混合界面- Ai回答更新显示
     */
    private val fromVoiceUpdateAiAnswerEvent = Event {
        if (it.obj is AIInterviewChatRes) {
            val aiInterviewChatRes = it.obj as AIInterviewChatRes
            handleAiAnswer(aiInterviewChatRes, aiInterviewChatRes.userAnswer, false)
        }
    }

    /**
     * 3 Ai 语音聊天关闭之后，刷新Ai语音文字混合页面的会话列表
     */
    private val flushSessionListEvent = Event {
        fetchAiInterviewSessionList()
    }

    private fun playVoiceService() {
        if (!CMKVUtils.isAutoConvertVoice()) {
            voiceFilePathQueue.clear()
            MediaManager.pause()
            return
        }
        val voiceFilePath = voiceFilePathQueue.poll()
        //LogUtils.i("aiGeneratingAutoPlay", "播放：2 voiceFilePath = $voiceFilePath")
        MediaManager.playSound(voiceFilePath, onCompletionListener, this)
    }

    override fun getContentLayoutId(): Int {
        return R.layout.activity_ai_interview
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        //设置状态栏未全透明 + 当前布局能通到状态栏内
        BarUtil.strikeStatusBar(this, true)
    }

    override fun init() {
        sendButtonAnimator =
            ObjectAnimator.ofFloat(vv_send_button, "translationY", DP_60.toFloat(), 0f)
        sendButtonAnimator.duration = 120

        val realSpace = adapterStatusBarSpace()
        if (realSpace > 0) {
            frame_title.setPadding(0, realSpace, 0, 0)
        }
        initTitle()
        initRecycleView()
        initListener()
        BarUtil.keyboardEnable(this)

        aiInterviewFloatEntryRes = intent?.getParcelableExtra(KEY_AI_INTERVIEW_FLOAT_ENTRY)
        //1 获取Ai 面试会话列表
        fetchAiInterviewSessionList()
    }

    private fun initTitle() {
        title_left_iv.setOnClickListener {
            finish()
        }
        //1 打开音色选择页面
        image_right_timbre.setOnClickListener {
            AnalyticHelper.getInstance().sendClickInfo("t1ujm")

            IntentUtils.startAiTimberActivity(this)
        }
        //2 启动纯语音页面
        image_right_pure_voice.setOnClickListener {
            if (isTextGenerating || isVoiceGenerating) {
                return@setOnClickListener
            }
            AnalyticHelper.getInstance().sendClickInfo("r1ujj")

            //1 跳转到纯语音页面 - 已创建会话
            val sessionId = newestAiSession?.sessionId ?: 0
            if (sessionId > 0) {
                IntentUtils.startPureVoiceActivity(this, newestAiSession?.sessionId ?: 0)
                return@setOnClickListener
            }

            //2 未创建会话 - 从浮动入口一步步过来
            IntentUtils.startPureVoiceActivity(this, aiInterviewFloatEntryRes)
        }

        //3 开启&关闭自动语音播放
        image_right_play.setOnClickListener {
            val clickInfo = ClickInfo()
            clickInfo.address_id = "s1ujk"
            clickInfo.addClick_props("type", if (CMKVUtils.isAutoConvertVoice()) 0 else 1)
            AnalyticHelper.getInstance().sendClickInfo(clickInfo)

            //是否开启Ai生成内容自动播报
            image_right_play.isSelected = !CMKVUtils.isAutoConvertVoice()
            CMKVUtils.setAutoConvertVoice(image_right_play.isSelected)
            if (!CMKVUtils.isAutoConvertVoice()) {
                MediaManager.pause()
                voiceFilePathQueue.clear()
            } else {
                //MediaManager.resume()
            }
        }

        image_right_play.isSelected = CMKVUtils.isAutoConvertVoice()
    }

    private fun initListener() {
        //0 跳转到职位详情
        lin_chat_position.setOnClickListener {
            newestAiSession?.positionId?.toLong()?.apply {
                IntentUtils.startPositionDetailActivity(this@AiInterviewActivity, this)
            }
        }
        //1 适配器开始面试点击
        aiInterviewAdapter?.setStartInterviewListenerMainPage(object :
            OnSingleMethodListener<AiSessionItem> {
            override fun onCallBack(aiSessionItem: AiSessionItem?) {
                //1 无需新创建会话，直接和大模型交互
                if (aiSessionItem == null || !aiSessionItem.isNotCreateSession) {
                    sendInputToAiModel(null)
                    return
                }

                //2 需要新创建会话，成功之后再与大模型狡猾
                createNewAiSession(aiSessionItem)
            }
        })
        //2 适配器重新发送语音识别请求
        aiInterviewAdapter?.setConvertVoiceAsrListener(object :
            OnSingleMethodListener<AiSessionItem?> {
            override fun onCallBack(aiSessionItem: AiSessionItem?) {
                uploadVoiceToText(aiSessionItem?.originalVoicePath, aiSessionItem?.tag)
            }
        })
        //3 重新发送文本
        aiInterviewAdapter?.setChatTextListener(object : OnSingleMethodListener<AiSessionItem?> {
            override fun onCallBack(aiSessionItem: AiSessionItem?) {
                chatInterviewAdd(aiSessionItem?.userAnswerText, aiSessionItem?.tag)
            }
        })

        //4 发送按钮回调事件
        handleSendListener()

        //5 监听软键盘变化，自动滑动列表
        val rootView = this.findViewById<View>(android.R.id.content)
        rootView.viewTreeObserver.addOnGlobalLayoutListener {

            if (isPrintInputIng) {
                return@addOnGlobalLayoutListener
            }

            val r = Rect()
            rootView.getWindowVisibleDisplayFrame(r)
            val visibleHeight = r.height()
            if (rootViewVisibleHeight == 0) {
                rootViewVisibleHeight = visibleHeight
                return@addOnGlobalLayoutListener
            }
            if (rootViewVisibleHeight != visibleHeight) {
                isHandSlide = false
                val heightDifference: Int = rootViewVisibleHeight - visibleHeight
                val baseDifference = DeviceUtils.getScreenHeight() * 0.15
//                LogUtils.i(
//                    "addOnGlobalLayoutListener",
//                    "heightDifference = $heightDifference  baseDifference = $baseDifference"
//                )
                if (heightDifference > baseDifference) {
                    // 输入法弹起
                    forceSmoothToNewestContent()
//                    LogUtils.i("addOnGlobalLayoutListener", "输入法弹起~")
                } else if (heightDifference < -baseDifference) {
                    // 输入法收起
                    forceSmoothToNewestContent()
//                    LogUtils.i("addOnGlobalLayoutListener", "输入法收起~")
                }
                rootViewVisibleHeight = visibleHeight
            }
        }

        //6 录音过程中,停止播报
        vv_send_button.setRecordingListener(object : OnSingleMethodListener<Boolean> {
            override fun onCallBack(isRecording: Boolean) {
                if (isRecording) MediaManager.pause()
            }
        })

        recycle_ai_content.setOnTouchListener { v, event ->
            val action = event.action
            if (action == MotionEvent.ACTION_DOWN || action == MotionEvent.ACTION_MOVE) {
                // 手指按下或者滑动，人手触碰滑动开始
                isHandSlide = true
            }
            false
        }

        //7 注册音色选择和
        OkBus.getInstance()
            .register(EventTags.OPEN_SELECT_PLAY_VOICE, openSelectTimbreEvent)
            .register(EventTags.FROM_VOICE_UPDATE_USER_INPUT, fromVoiceUpdateUserInputEvent)
            .register(EventTags.FROM_VOICE_UPDATE_AI_ANSWER, fromVoiceUpdateAiAnswerEvent)
            .register(EventTags.AI_INTERVIEW_FLUSH_SESSION_LIST, flushSessionListEvent)
    }

    private fun initRecycleView() {
        aiInterviewAdapter = AIInterviewAdapter(this)
        layoutManagerCompact = LinearLayoutManager(this)
        val guideItemAnim = GuideItemAnim()
        guideItemAnim.setOnAnimEnd {
            sendButtonAnimator.addListener(object : AnimatorListenerAdapter() {
                override fun onAnimationStart(animation: Animator) {
                    super.onAnimationStart(animation)
                    vv_send_button.visibility = View.VISIBLE
                }
            })
            sendButtonAnimator.start()
        }

        recycle_ai_content.apply {
            layoutManager = layoutManagerCompact
            adapter = aiInterviewAdapter
            itemAnimator = guideItemAnim
            addItemDecoration(object : RecyclerView.ItemDecoration() {
                override fun getItemOffsets(
                    outRect: Rect,
                    view: View,
                    parent: RecyclerView,
                    state: RecyclerView.State
                ) {
                    super.getItemOffsets(outRect, view, parent, state)

                    val currentPosition = parent.getChildAdapterPosition(view)
                    // 水平方向广告类型，左右两侧不设置间距，其他类型设置间距
                    if (parent.adapter?.getItemViewType(currentPosition) == R.layout.item_ai_interview_horizontal_ad) {
                        outRect.set(0, DP_15, 0, DP_6_05)
                    } else {
                        outRect.set(DP_15, DP_15, DP_15, DP_6_05)
                    }
                }
            })
        }
    }

    private fun handleSendListener() {
        vv_send_button.setOnSendListener { msgType, contentOrPath ->
            if (TextUtils.isEmpty(contentOrPath)) {
                return@setOnSendListener
            }

            //1 文本类型
            if (msgType == AiContentType.TEXT) {
                //1.1 把用户输入的文本，更新到聊天窗口，用户自己输入文本，不做打字效果
                val aiSessionItem = AiSessionItem.createUserText(
                    IAiInterviewStatus.SUCCESS,
                    contentOrPath,
                    newestAiSession?.sessionId ?: 0,
                    newestAiSession?.positionId ?: 0,
                    HttpEventHelper.getInstance().clientTag
                )
                aiInterviewAdapter?.updateInterviewContent(aiSessionItem)
                forceSmoothToNewestContent()

                //1.2 发送用户输入文本给大模型
                sendInputToAiModel(contentOrPath)

                return@setOnSendListener
            }

            //2 语音类型路径为空或者语音文件不存在
            if (!(File(contentOrPath).exists())) {
                return@setOnSendListener
            }
            //2.1 先调用语音转文字接口，把用户语音转换为文本
            val voiceRequest = uploadVoiceToText(contentOrPath, null)
            //2.2 构造loading Ai AiSessionItem
            //2.3 构造一个AiSessionItem，初始状态为loading、语音识别成功之后替换为对应内容
            val aiSessionItemTemp = AiSessionItem()
            val tag = voiceRequest?.tag as? Int
            if (tag != null) {
                aiSessionItemTemp.tag = tag
            }
            aiSessionItemTemp.roleType = RoleType.USER_MESSAGE
            aiSessionItemTemp.isLoading = true
            aiSessionItemTemp.contentType = IAiSessionContentType.VOICE
            aiInterviewAdapter?.updateInterviewContent(aiSessionItemTemp)

            forceSmoothToNewestContent()
        }
    }

    /**
     * 新创建会话
     * 适用场景：从Ai面试浮动入口进来的，类比聊天，这里点击开始面试如果会话未创建，先创建会话，接着在发起Ai对话
     */
    private fun createNewAiSession(aiSessionItem: AiSessionItem) {
        AiNetManager.createAiInterviewSession(
            newestAiSession,
            object : OnSingleMethodListener<AIInterviewCreateRes> {
                override fun onCallBack(aiInterviewCreateRes: AIInterviewCreateRes?) {
                    if (isDestroyed || aiInterviewCreateRes == null) {
                        return
                    }

                    //fetchAiInterviewSessionList()
                    //0 实际是刚创建会话状态，强制设置为开始面试状态，因为紧接着和Ai交互的请求就发出
                    newestAiSession?.sessionId = aiInterviewCreateRes.sessionId
                    newestAiSession?.sessionState = IAiInterviewSessionState.START_INTERVIEW
                    aiSessionItem.sessionId = aiInterviewCreateRes.sessionId
                    aiSessionItem.sessionState = IAiInterviewSessionState.START_INTERVIEW

                    //1 更新职位卡片显示
                    aiInterviewAdapter?.updateInterviewContent(aiSessionItem)

                    //2 会话创建成功继续调用开始面试接口（与大模型开始交互）
                    sendInputToAiModel(null)

                    //3 更新JD页面Ai面试入口数据
                    OkBus.getInstance().onEvent(EventTags.UPDATE_AI_INTERVIEW_FLOAT_ENTRY)
                }
            })
    }

    /**
     * 发送用户输入文本
     * 1 先执行loading效果
     * 2 网络请求结果回来之后，在用文本替换
     */
    private fun sendInputToAiModel(contentOrPath: String?, isVoiceMessage: Boolean = false) {
        //1.1 发出网络请求
        //1.2 开启面试执行请求Ai 接口获取AI 提问
        val request = chatInterviewAdd(contentOrPath, null, isVoiceMessage) ?: return
        //1.3 构造一个AiSessionItem，初始状态为loading、AI模型返回内容之后替换为对应内容
        val aiSessionItemTemp = AiSessionItem()
        val tag = request.tag as? Int
        if (tag != null) {
            aiSessionItemTemp.tag = tag
        }
        aiSessionItemTemp.roleType = RoleType.SYSTEM_MESSAGE
        aiSessionItemTemp.isLoading = true
        aiInterviewAdapter?.updateInterviewContent(aiSessionItemTemp)

        forceSmoothToNewestContent()
    }


    /**
     * 和大模型交互，获取大模的回答
     * @param isPLayVoice 是否自动转语音播放，在纯语音页面，为了同步状态执行的与大模型交互的行为，不播放音频
     */
    fun chatInterviewAdd(
        answer: String?, originalTag: Int?,
        isVoiceMessage: Boolean = false,
        isPLayVoice: Boolean = true
    ): HttpRequest<AIInterviewChatRes>? {

        //新的Ai 生成内容请求发出，重置老的音频相关的变量
        convertVoiceCurrentIndex = 0
        voiceFilePathQueue.clear()
        aiSessionItemGenerateVoice = AiSessionItem()
        MediaManager.pause()
        isFirstVoice = true

        currentAiSessionItem = null
        displayTextFromNetDisplayOffset = null

        newestAiSession?.apply {
            isTextGenerating = true
            handleSendBtnStyle(false, true)
            httpRequest = AiNetManager.chatInterviewAdd(
                answer,
                originalTag,
                positionId,
                sessionId,
                isVoiceMessage,
                object : BaseEventCallback<AIInterviewChatRes?>() {
                    override fun onSuccess(
                        httpRequest: HttpRequest<*>,
                        httpResponse: HttpEventResponse<AIInterviewChatRes?>
                    ) {
                        onRequestCall(httpRequest, httpResponse, true)
                    }

                    override fun onClose(
                        httpRequest: HttpRequest<*>,
                        httpEventResponse: HttpEventResponse<AIInterviewChatRes?>
                    ) {
                        handleSendBtnStyle(true, true)

                        //onRequestCall(httpRequest, httpEventResponse, false)
                        isTextGenerating = false
                    }

                    override fun onFailure(
                        httpRequest: HttpRequest<*>,
                        httpEventResponse: HttpEventResponse<AIInterviewChatRes?>
                    ) {
                        super.onFailure(httpRequest, httpEventResponse)
                        isTextGenerating = false
                        handleSendBtnStyle(true, true)

                        onRequestCall(httpRequest, httpEventResponse, false)
                    }

                    private fun onRequestCall(
                        httpRequest: HttpRequest<*>,
                        httpResponse: HttpEventResponse<AIInterviewChatRes?>, isSuccess: Boolean
                    ) {
                        val aiInterviewChatRes = httpResponse.content ?: AIInterviewChatRes()
                        aiInterviewChatRes.apply {
                            if (httpRequest.tag is Int) tag = httpRequest.tag as Int
                        }
                        isTextGenerating = isSuccess && !aiInterviewChatRes.isLastCompleteData
                        handleAiAnswer(aiInterviewChatRes, answer)

                        httpResponse.isFinish
                    }
                }
            )

            return httpRequest
        }

        return null
    }

    private fun handleAiAnswer(
        aiInterviewChatRes: AIInterviewChatRes,
        answer: String?,
        isPLayVoice: Boolean = true
    ) {
        if (isDestroyed) {
            return
        }

        //0 发送录音按钮样式处理
        //当前流式过程结束，才把按钮置为可点击
        handleSendBtnStyle(aiInterviewChatRes.isCanSendMsg, !aiInterviewChatRes.isInitState)

        //1 目前流用的是追加模式，所以在给打字器之前，必须组合成完整的文本
        if (currentAiSessionItem == null) {
            currentAiSessionItem = AiSessionItem.convert(aiInterviewChatRes)
        } else {
            currentAiSessionItem?.apply {
                status = aiInterviewChatRes.status
                tag = aiInterviewChatRes.tag
                if (aiInterviewChatRes.isSuccess) {
                    content += aiInterviewChatRes.content
                } else {
                    content = aiInterviewChatRes.content
                }
                sessionState = aiInterviewChatRes.sessionState
                messageId = aiInterviewChatRes.messageId
                itemRelateMsgList = aiInterviewChatRes.itemRelateMsgList
                isLoading = false
            }
        }

        //2 如果status状态不是0 和 108001 -1
        if ((aiInterviewChatRes.isExceptionStatus || aiInterviewChatRes.isSessionEnd)) {
            if (!TextUtils.isEmpty(aiInterviewChatRes.statusMessage)) ToastUtil.show(this@AiInterviewActivity, aiInterviewChatRes.statusMessage)
            currentAiSessionItem?.let {
                aiInterviewAdapter?.updateInterviewContent(it)
            }
            return
        }

        //3 成功获取结果，打字更新显示
        currentAiSessionItem?.let { it ->
            it.userAnswerText = answer
            //LogUtils.i("onRequestCall","1 内容 ${currentAiSessionItem?.content}")
            if (displayTextFromNetDisplayOffset == null) {
                displayTextFromNetDisplayOffset = DelayedTextDisplayOffset(it)
                displayTextFromNetDisplayOffset?.startDelayedTextDisplay()
            } else {
                //有新的流式文本更新
                displayTextFromNetDisplayOffset?.updateGoalContent(it, aiInterviewChatRes)
            }

            //3.1 分片转换为语音，进行自动播报
            if (isPLayVoice) aiGeneratingAutoPlay(it)

            //4 最后一条会话的最后一个包，给用户提示，写面试评价
            displayTextFromNetDisplayOffset?.setOutputEndListener {
                it?.apply {
                    if (isLastCompleteData && !TextUtils.isEmpty(evaluate)) {
                        val aiSessionItem = AiSessionItem()
                        aiSessionItem.tag = HttpEventHelper.getInstance().clientTag
                        aiSessionItem.content = aiInterviewChatRes.evaluate
                        aiSessionItem.roleType = RoleType.SYSTEM_MESSAGE
                        aiSessionItem.sessionState = aiInterviewChatRes.sessionState
                        aiSessionItem.status = aiInterviewChatRes.status
                        aiInterviewAdapter?.updateInterviewContent(aiSessionItem)

                        smoothToNewestContent()
                        convertVoiceCurrentIndex = 0
                        if (isPLayVoice) aiGeneratingAutoPlay(aiSessionItem)
                    }
                }
            }
        }

    }

    /**
     * 发送按钮的样式和点击
     */
    private fun handleSendBtnStyle(isCanSend: Boolean, isVisible: Boolean) {
        if (isDestroyed) {
            return
        }
        vv_send_button.setVoiceEnable(!isTextGenerating && isCanSend)
        //底部按钮的是否可见处理
        vv_send_button.visibility = if (isVisible) View.VISIBLE else View.GONE
    }

    /**
     * Ai生成的文本自动转化为音频文件，并启动播放
     */
    private fun aiGeneratingAutoPlay(aiSessionItem: AiSessionItem) {
        aiSessionItemGenerateVoice = aiSessionItem

        //1 关闭播放，取消语音合成
        if (!CMKVUtils.isAutoConvertVoice()) {
            voiceFilePathQueue.clear()
            return
        }
        //2 语音生成中，只有语音生成中才有效（防止涉生成语音不完整）
        if (isVoiceGenerating) {
            return
        }
        //3 说明语音合成已完成
        if (TextUtils.isEmpty(aiSessionItemGenerateVoice.content) || convertVoiceCurrentIndex >= aiSessionItemGenerateVoice.content.length) {
            return
        }

        //当前文本
        val sourceContent = aiSessionItemGenerateVoice.content
        //指定下标开始的，首个标点符号位置
        val firstSymbolIndex =
            StringUtil.findPunctuationStartIndex(sourceContent, convertVoiceCurrentIndex)
        //4 不是最后一个包，如果内容不到一个标点符号（说明不完整），则等待，降低语音转换接口的调用，保障合成语音的语气语境
        if (aiSessionItemGenerateVoice.isSuccess && firstSymbolIndex == -1) {
            return
        }
        //5 截取文本分段合成语音
        //5.1 如果是正常返回语音，分段合成；如果是语音返回结束或者出现异常，则直接合成剩余文字
        val step = if (aiSessionItemGenerateVoice.isSuccess) {
            if (convertVoiceCurrentIndex + firstSymbolIndex < sourceContent.length) firstSymbolIndex + 1
            else sourceContent.length - convertVoiceCurrentIndex
        } else {
            sourceContent.length - convertVoiceCurrentIndex
        }

        val waitConvertContent =
            sourceContent.substring(convertVoiceCurrentIndex, convertVoiceCurrentIndex + step)

        LogUtils.i(
            "aiGeneratingAutoPlay", "当前下标：$convertVoiceCurrentIndex  转换文字的个数：$step " +
                    "待转换内容 waitConvertContent = $waitConvertContent  总的内容：aiSessionItem.content = " + aiSessionItem.content
        )
        convertVoiceCurrentIndex += step
        isVoiceGenerating = true
        AiNetManager.convertTextVoice(
            text = waitConvertContent,
            voiceType = CMKVUtils.getSelectTimbre()?.value,
            baseCallback = object : BaseCallback<AiInterviewTtsRes?>() {
                override fun onSuccess(
                    request: HttpRequest<*>,
                    response: HttpResponse<AiInterviewTtsRes?>
                ) {
                    isVoiceGenerating = false
                    //取消语音合成
                    if (!CMKVUtils.isAutoConvertVoice()) {
                        return
                    }
//                    LogUtils.i(
//                        "aiGeneratingAutoPlay",
//                        "生成：2 voicePath = ${aiInterviewTtsRes?.voicePath} ；；； waitConvertContent = $waitConvertContent"
//                    )
                    response.content?.voicePath?.apply {
                        voiceFilePathQueue.add(this)
                        //1 生成的第一条的语音之后，触发播放操作
                        if (isFirstVoice) {
                            isFirstVoice = false
                            playVoiceService()
                        }
                    }

                    //2 一次Ai语音结束，判断生成的语音是否覆盖所有的Ai文本，如果没有生成结束，继续调用aiGeneratingAutoPlay生成语音
                    //  注意递归调用
                    if (convertVoiceCurrentIndex < aiSessionItemGenerateVoice.content.length) {
                        aiGeneratingAutoPlay(aiSessionItemGenerateVoice)
                    }
                }

                override fun onFailure(
                    request: HttpRequest<*>, response: HttpResponse<AiInterviewTtsRes?>
                ) {
                    super.onFailure(request, response)
                    isVoiceGenerating = false
                }
            })
    }

    /**
     * 语音转文本：用户输入语音，调用asr接口转换为文字，然后提交大模型
     */
    private fun uploadVoiceToText(
        contentOrPath: String?,
        originalTag: Int?
    ): HttpRequest<AiInterviewAsrRes>? {

        if (contentOrPath.isNullOrEmpty()) return null

        handleSendBtnStyle(false, true)

        return AiNetManager.uploadVoiceToText(
            contentOrPath,
            originalTag,
            object : HttpCallback<AiInterviewAsrRes?> {
                override fun onSuccess(
                    request: HttpRequest<*>,
                    httpResponse: HttpResponse<AiInterviewAsrRes?>
                ) {

                    if (TextUtils.isEmpty(httpResponse.content?.asrText)) {
                        ToastUtil.show(this@AiInterviewActivity, "未识别到文字")
                    }

                    //1 把用户输入的文本，更新到聊天窗口，用户自己输入文本，不做打字效果
                    val aiSessionItem = AiSessionItem.createUserText(
                        IAiInterviewStatus.SUCCESS,
                        httpResponse.content?.asrText,
                        newestAiSession?.sessionId ?: 0,
                        newestAiSession?.positionId ?: 0,
                        request.tag as Int
                    )
                    aiSessionItem.contentType = IAiSessionContentType.TEXT
                    //if (displayTextFromUserDisplayOffset == null) {
                    //    displayTextFromUserDisplayOffset = DelayedTextDisplayOffset(aiSessionItem)
                    //    displayTextFromUserDisplayOffset?.startDelayedTextDisplay()
                    //    return
                    //}
                    //有新的流式文本更新
                    //displayTextFromUserDisplayOffset?.updateGoalContent(aiSessionItem)
                    aiInterviewAdapter?.updateInterviewContent(aiSessionItem)

                    //2 发送到AI大模型，获取AI生成文本（未识别到任何文字，就不要发给大模型了）
                    if (TextUtils.isEmpty(httpResponse.content?.asrText)) {
                        handleSendBtnStyle(true, true)
                        return
                    }
                    sendInputToAiModel(httpResponse.content?.asrText, true)
                }

                override fun onFailure(
                    request: HttpRequest<*>,
                    httpResponse: HttpResponse<AiInterviewAsrRes?>
                ) {
//                    LogUtils.i("uploadVoiceToText", "上传语音失败 response = " + httpResponse)
                    if (isDestroyed) {
                        return
                    }

                    handleSendBtnStyle(true, true)

                    if (!TextUtils.isEmpty(httpResponse.message)) {
                        ToastUtil.show(this@AiInterviewActivity, httpResponse.message)
                    }
                    val aiSessionItem = AiSessionItem.createUserText(
                        httpResponse.state,
                        httpResponse.content?.asrText ?: "",
                        newestAiSession?.sessionId ?: 0,
                        newestAiSession?.positionId ?: 0,
                        request.tag as Int
                    )
                    aiSessionItem.contentType = IAiSessionContentType.VOICE
                    aiSessionItem.originalVoicePath = contentOrPath
                    aiInterviewAdapter?.updateInterviewContent(aiSessionItem)
                }
            })
    }

    /**
     * 1 获取Ai 面试的会话列表
     */
    private fun fetchAiInterviewSessionList() {

        handleSendBtnStyle(false, true)
        AiNetManager.fetchAiInterviewSessionList(object :
            OnAdapterCommonListenerImpl<List<AiInterviewSessionListRes?>?, HttpResponse<List<AiInterviewSessionListRes?>?>>(
                this
            ) {
            override fun onSuccess(aiInterviewSessionList: List<AiInterviewSessionListRes?>?) {
                super.onSuccess(aiInterviewSessionList)
                val realAiInterviewSessionList = mutableListOf<AiInterviewSessionListRes?>()
                //1 接口返回的会话列表添加到新列表中
                aiInterviewSessionList?.let {
                    realAiInterviewSessionList.addAll(it)
                }

                //2 判断当前会话中是否有正在进行中的，如果有则不自动构造会话（从Ai面试浮动入口进来，客户端自己构造的职位卡片和开始面试入口）
                val tempAiInterviewSession = realAiInterviewSessionList.firstOrNull()
                //2.1 只有面试处于结束的情况下，才能往session列表中增加自己构造的职位卡片和开始面试入口
                //2.2 ---827修改，可以随意创建会话，没有结束也可以创建（但要确保两个职位的id不一样，才会把本地构造的欢迎语、职位卡片添加到列表中）
                //3 如果是从Ai浮动入口进入，需要用上一页带出来的数据覆盖构造
                if (tempAiInterviewSession == null || tempAiInterviewSession.positionId != aiInterviewFloatEntryRes?.positionId) {
                    val aiInterviewChatRes =
                        AiInterviewSessionListRes.createSessionListRes(aiInterviewFloatEntryRes)
                    if (aiInterviewChatRes != null) {
                        realAiInterviewSessionList.add(0, aiInterviewChatRes)
                    }
                }

                this@AiInterviewActivity.aiInterviewSessionList =
                    realAiInterviewSessionList.reversed()
                this@AiInterviewActivity.newestAiSession =
                    this@AiInterviewActivity.aiInterviewSessionList?.last()

                //4 底部按钮是否显示、点击
                newestAiSession?.let {
                    recycle_ai_content.post {
                        handleSendBtnStyle(
                            !it.isFinishState,
                            !it.isInitState && !it.isNotCreateSession
                        )
                    }
                }
                //5 返回的 response.content List<AiInterviewSessionListRes>，多个会话组合，为了方便在列表中显示
                //  需要对各个list展开，并标记好每个会话起始位置，把sessionId positionId sessionState 在具体的会话详情中也放一份
                //  方便处理和判断
                val allSessionList: List<AiSessionItem> =
                    AiNetManager.handleNetSessionList(realAiInterviewSessionList.reversed())
                //6 填充显示页面
                aiInterviewAdapter?.setData(true, allSessionList)
                recycle_ai_content.postDelayed({
                    //加载结束滑动到最新的一条（底部）
                    aiInterviewAdapter?.updateOccupyType()
                    isHandSlide = false
                    slideToNewestContent(false)
                }, 100)

                //3 利用拿到的职位id，获取职位名称等信息，用于显示
                fetchPositionDetail()
            }

            override fun isCareResponse(): Boolean {
                return false
            }
        })
    }

    /**
     * 获取职位详情
     */
    fun fetchPositionDetail() {
        newestAiSession?.positionId?.toLong()?.let {
            PositionDetailController.getInstance().requestPositionDetailDataUseCache(it, object :
                GetPositionDetailDataHandler {
                override fun requestDataSuccess(bean: PositionDetailBean?) {
                    if (bean == null) {
                        return
                    }
                    CBusinessMemoryCacheManager.cachePositionDetail(
                        SafeTypeConvert.toInt(bean.positionId),
                        bean
                    )
                    tv_chat_position_content.text = bean.positionName
                }

                override fun onError(status: Int) {
                }
            })
        }
    }


    /**
     * 通过操作下标偏移的方式，实现打字机效果，更适合流式逐步给出内容
     * 不确定：接口请求流式出文案，速度不定
     * 匀速：本地控制显示匀速出
     * 注意：流式给出第一条文案就要及时显示到列表，后续有更新及时更新本地匀速出的数据源
     * 保障：timer 定时器不关闭，除非本地匀速出文案结束（currentIndex >= aigcReleaseRes.content.length）&接口出文案也结束的情况下（finishReleaseRes不为空）
     */
    inner class DelayedTextDisplayOffset(aiSessionItem: AiSessionItem) {

        private var aiInterviewChatRes: AIInterviewChatRes? = null
        private var sessionItem: AiSessionItem
        private var finishSessionItem: AiSessionItem? = null
        private var timer: Timer? = null

        /**
         * 打字效果的当前下标
         */
        private var currentIndex: Int = 0

        /***
         * 当前偏移量，注意判断不要越界
         */
        private var offsetCount = 0

        /**
         * 输出结束回调
         */
        private var outputEndListener: ((AIInterviewChatRes?) -> Unit)? = null

        init {
            sessionItem = aiSessionItem
        }

        /**
         * 流式过程中不停地更新文本，避免重复开启任务流，判断只有当当前执行流程结束时才重新激活
         */
        fun updateGoalContent(aiSessionItem: AiSessionItem, aiInterviewChatRes: AIInterviewChatRes) {
            if (aiSessionItem.isSuccess) {
                sessionItem = aiSessionItem
            } else if (aiSessionItem.isUserChat) {
                //用户输入文本处理
                sessionItem = aiSessionItem
                finishSessionItem = aiSessionItem
            } else {
                //正常结束先缓存起来等到打字效果偏移量走完了在更新
                finishSessionItem = aiSessionItem
                if (aiSessionItem.isLastCompleteData){
                    this.aiInterviewChatRes = aiInterviewChatRes
                }
            }
        }

        /**
         * 设置输出结束回调
         */
        fun setOutputEndListener(outputEndListener: ((AIInterviewChatRes?) -> Unit)) {
            this.outputEndListener = outputEndListener
        }

        fun startDelayedTextDisplay() {
            isHandSlide = false
            displayTextWithDelay()
        }

        /***
         * 是否是异常数据，是的话丢弃当次timer任务，等待下次timer任务执行
         */
        private fun isExceptionData(): Boolean {
            return TextUtils.isEmpty(sessionItem.content)
        }

        private fun displayTextWithDelay() {
            timer = Timer()
            timerList.add(timer)
            timer?.scheduleAtFixedRate(object : TimerTask() {
                override fun run() {
                    if (isExceptionData()) {
                        return
                    }
                    try {
                        //fix Attempt to invoke virtual method 'int java.lang.String.length()' on a null object reference
//                        LogUtils.i("onRequestCall","2 内容 ${sessionItem.content} currentIndex = " + currentIndex + " isLastContent " + sessionItem.isLastCompleteData)
                        if (currentIndex < sessionItem.content.length) {
                            isPrintInputIng = true

                            // 在此处处理显示文本的逻辑，例如将 text 设置到 TextView 中显示
                            offsetCount = StringUtil.generateRandomNumber(1, 3)
                            if (currentIndex + offsetCount > sessionItem.content.length) {
                                offsetCount = sessionItem.content.length - currentIndex
                            }
                            val resultTxt =
                                sessionItem.content.substring(
                                    currentIndex,
                                    currentIndex + offsetCount
                                )

//                            LogUtils.i("onRequestCall","3 内容 ${sessionItem.content} resultTxt = " + resultTxt + "")
                            val aiSessionItem =
                                AiSessionItem.createOffsetText(resultTxt, sessionItem)

                            recycle_ai_content.post {
                                aiInterviewAdapter?.updateAppendInterviewContent(aiSessionItem)
                                currentIndex += offsetCount
                                smoothToNewestContent()
                            }
                            return
                        }

                        // 最后一个包，处理显示广告位的情况
                        if (currentIndex >= sessionItem.content.length) {
                            finishSessionItem?.apply {
                                sessionItem.status = status
                                sessionItem.sessionId = sessionId
                                sessionItem.sessionState = sessionState
                                sessionItem.messageId = messageId
                                sessionItem.content = content
                                sessionItem.itemRelateMsgList = itemRelateMsgList
                                finishSessionItem = null
                                recycle_ai_content.post {
                                    aiInterviewAdapter?.updateInterviewContent(sessionItem)
                                    forceSmoothToNewestContent()
                                    completePackageDataHandleAd(sessionItem)

                                    //LogUtils.i("finish_session", "++++ sessionItem = $sessionItem")
                                    //输出结束回调
                                    recycle_ai_content.postDelayed({
                                        outputEndListener?.invoke(aiInterviewChatRes)
                                        //LogUtils.i("finish_session", "++++ aiInterviewChatRes = $aiInterviewChatRes")
                                    }, 300)

                                    val iterator = timerList.iterator()
                                    while (iterator.hasNext()) {
                                        val timer = iterator.next()
                                        timer?.cancel()
                                        iterator.remove()
                                    }
                                    isPrintInputIng = false
                                    //timer?.cancel()
                                    //timerList.remove(timer)
                                }
                            }
                        }
                    } catch (e: Exception) {
                        e.printStackTrace()
                    }
                }
            }, 0, 100)
        }

        /***
         * 看下是否有广告，有的话在打字效果结束后显示广告
         */
        private fun completePackageDataHandleAd(sessionItem: AiSessionItem) {
            //1 判断广告列表是否为空，如果不为空则单独放到一个新的AiSession中
            if (!ListUtil.isEmpty(sessionItem.itemRelateMsgList)) {
                //2 构造广告aiSessionItem
                val adsSessionItem = AiSessionItem()
                adsSessionItem.itemRelateMsgList = sessionItem.itemRelateMsgList
                adsSessionItem.msgType =
                    sessionItem.itemRelateMsgList[0]?.msgType ?: AiRelateMsgItemType.COMMON
                adsSessionItem.tag = HttpEventHelper.getInstance().clientTag

                //3 原始的消息session广告列表置空&add
                sessionItem.itemRelateMsgList = null

                //4 新的广告aiSession add到List中
                //LogUtils.i("completePackageDataHandleAd", "Ai 输出完整的一个会话之后，展示广告，itemRelateMsgList ${adsSessionItem.itemRelateMsgList}")
                recycle_ai_content.postDelayed({
                    aiInterviewAdapter?.updateInterviewContentAds(adsSessionItem)
                    forceSmoothToNewestContent()
                }, 150)
            }
        }
    }


    fun forceSmoothToNewestContent() {
        isHandSlide = false
        slideToNewestContent(true)
    }

    private fun smoothToNewestContent() {
        slideToNewestContent(true)
    }

    /***
     * 列表滑动到底部，方便用户看到最新的输出
     */
    fun slideToNewestContent(isSmoothSlide: Boolean) {
        //如果是有手动滑动的意图，则中断当前生成内容会话的自动滑动，直到当前内容生成结束才重置isHandSlide状态
        if (isHandSlide) {
            return
        }

        if (isSmoothScrolling) {
            return
        }

        isSmoothScrolling = true
        //控制200ms内只能有一次调用smoothScrollToPosition
        recycle_ai_content.postDelayed({

            var lastItemIndex = aiInterviewAdapter?.itemCount ?: 0
            if (lastItemIndex > 0) {
                lastItemIndex -= 1
            }
//            LogUtils.i("slideToNewestContent", "lastItemIndex = $lastItemIndex  itemCount = ${aiInterviewAdapter?.itemCount}")
            //val offset: Int = recycle_ai_content.computeVerticalScrollRange() - recycle_ai_content.computeVerticalScrollExtent()
            //layoutManagerCompact?.smoothScrollToPosition(recycle_ai_content, null, lastItemIndex)
            //recycle_ai_content?.scrollBy(0, offset)
            if (isSmoothSlide) {
                recycle_ai_content.smoothScrollToPosition(lastItemIndex)
            } else {
                recycle_ai_content.scrollToPosition(lastItemIndex)
            }

            isSmoothScrolling = false
        }, 200)
    }

    override fun onStop() {
        super.onStop()
        MediaManager.pause()
        //更新会话中的最后一条消息
        ThreadUtil.io {
            val content = aiInterviewAdapter?.getLastMsg()?.content
            //排除是职位id的情况
            if (TextUtils.isEmpty(content) || SafeTypeConvert.toInt(content) != -1) {
                return@io
            }
            val lastMsgTime = aiInterviewAdapter?.getLastMsg()?.createTime ?: 0
            SessionDBManager.getInstance()
                .updateLastMessage(AiInterviewSession.SESSION_ID, content, lastMsgTime)
        }
    }

    override fun onRestart() {
        super.onRestart()
        MediaManager.resume()
    }

    override fun isEnableCommonBarSetting(): Boolean {
        return false
    }

    override fun onDestroy() {
        super.onDestroy()
        MediaManager.release()
        voiceFilePathQueue.clear()
        ThreadUtil.io {
            //清除指定目录的文件
            FileUtils.deleteFile(AiNetManager.getTtsOutDir(this))
        }

        OkBus.getInstance()
            .unRegister(openSelectTimbreEvent)
            .unRegister(fromVoiceUpdateUserInputEvent)
            .unRegister(fromVoiceUpdateAiAnswerEvent)
            .unRegister(flushSessionListEvent)
        try {
            timerList.forEach { it?.cancel() }
        } catch (e: Exception) {
            e.printStackTrace()
        } finally {
            HttpEventHelper.getInstance().cancel(httpRequest)
        }
    }

    override fun getPageId(): String {
        return "m1uiq"
    }

    override fun getShowLogInfo(): ShowLogInfo {
        val showLogInfo = ShowLogInfo()
        showLogInfo.setPage_Id(pageId)
        showLogInfo.setAddress_Id("n1uir")
        return showLogInfo
    }

    companion object {
        /***
         * 从浮层Ai面试入口进入，携带数据
         */
        const val KEY_AI_INTERVIEW_FLOAT_ENTRY = "AI_INTERVIEW_FLOAT_ENTRY"
    }


    var txt =
        "{\"content\":\"由于您在整个面试过程中的回答过于简略，无法充分展示您在 Java 开发方面的技术能力、经验和沟通能力等，本次面试得分 30 分。打分规则主要基于您对面试问题的回答完整度、清晰度以及所展现的专业知识深度和广度。本次面试结束 \",\"evaluate\":\"非常感谢您的参与，请用几句话分享一下对这次面试体验的感受，您的意见对我们非常宝贵。\",\"itemRelateMsgList\":[{\"code\":6,\"imgUrl\":\"https://lagou-zhaopin-fe.lagou.com/activities/aichat/ai_qzsj_icon_88.png\",\"linkUrl\":\"lagou://lagou.com/h5/open?url=https%3A%2F%2Factivity.lagou.com%2Fapp%2Ftoc%2Ftutoring.html%3Futm_source%3Dapp_aigxa_qzsj&hideTitle=1\",\"msgType\":2,\"tip\":\"专家陪伴求职\",\"title\":\"求职私教\",\"weappLinkUrl\":\"https://activity.lagou.com/app/toc/tutoring.html?utm_source=weapp_ai_chat&hideTitle=1\"},{\"code\":9,\"imgUrl\":\"https://lagou-zhaopin-fe.lagou.com/activities/aichat/ai_vip_icon_88.png\",\"linkUrl\":\"lagou://lagou.com/h5/open?url=https%3A%2F%2Factivity.lagou.com%2Fapp%2Ftoc%2Fmember.html%3Futm_source%3Dapp_gxa_vip&hideTitle=1\",\"msgType\":2,\"tip\":\"提升求职效率\",\"title\":\"拉勾VIP\",\"weappLinkUrl\":\"https://activity.lagou.com/app/toc/member.html#/?utm_source=weapp_ai_chat\"}],\"messageId\":\"112987494756452166\",\"sessionId\":\"285\",\"status\":-1,\"stautsMessage\":\"执行完毕\"}"

    private fun test(){
        val aiInterviewChatResFirst = GsonUtil.fromJson(txt, AIInterviewChatRes::class.java)

        aiInterviewChatResFirst.content = "您好，"
        aiInterviewChatResFirst.tag = 200000

        handleAiAnswer(aiInterviewChatResFirst, "我不知道", true)

        recycle_ai_content.postDelayed({
            val aiInterviewChatRes = GsonUtil.fromJson(txt, AIInterviewChatRes::class.java)
            aiInterviewChatRes.tag = 200000
            handleAiAnswer(aiInterviewChatRes, "我不知道", true)
        }, 1000)
    }
}