package com.alpha.lagouapk.ui.interview.ai

import android.media.MediaPlayer
import android.text.TextUtils
import android.view.View
import com.alpha.lagouapk.Interface.GetPositionDetailDataHandler
import com.alpha.lagouapk.R
import com.alpha.lagouapk.bean.PositionDetailBean
import com.alpha.lagouapk.bean.aiinterview.AIInterviewChatRes
import com.alpha.lagouapk.bean.aiinterview.AIInterviewCreateRes
import com.alpha.lagouapk.bean.aiinterview.AiInterviewAsrRes
import com.alpha.lagouapk.bean.aiinterview.AiInterviewFloatEntryRes
import com.alpha.lagouapk.bean.aiinterview.AiInterviewSessionListRes
import com.alpha.lagouapk.bean.aiinterview.AiInterviewTtsRes
import com.alpha.lagouapk.bean.aiinterview.IAiInterviewSessionState
import com.alpha.lagouapk.manager.CBusinessMemoryCacheManager
import com.alpha.lagouapk.manager.CMKVUtils
import com.alpha.lagouapk.manager.net.AiNetManager
import com.alpha.lagouapk.observer.EventTags
import com.alpha.lagouapk.tools.IntentUtils
import com.alpha.lagouapk.ui.interview.ai.AiInterviewVoiceActivity.Companion.ROUTER_URL
import com.alpha.lagouapk.ui.interview.ai.widget.PureVoiceView
import com.alpha.lagouapk.ui.position.PositionDetailActivity
import com.alpha.lagouapk.ui.position.PositionDetailController
import com.lagou.annotation.apt.RouterRule
import com.lagou.baselib.BaseActivity
import com.lagou.baselib.analytic.bean.json.ShowLogInfo
import com.lagou.baselib.net.httpnetback.BaseCallback
import com.lagou.baselib.net.httpnetback.BaseEventCallback
import com.lagou.baselib.net.httpnetback.OnAdapterCommonListenerImpl
import com.lagou.chat.chatlibrary.manager.MediaManager
import com.lagou.commonlibs.bean.WebViewExtentBean
import com.lagou.commonlibs.engine.ThreadUtil
import com.lagou.commonlibs.event.OkBus
import com.lagou.commonlibs.inter.OnSingleMethodListener
import com.lagou.commonlibs.permisssion.RuntimePermissionUtil
import com.lagou.commonlibs.utils.FileUtils
import com.lagou.commonlibs.utils.LogUtils
import com.lagou.commonlibs.utils.SafeTypeConvert
import com.lagou.commonlibs.utils.StringUtil
import com.lagou.commonlibs.utils.ToastUtil
import com.lagou.commonlibs.web.router.RouteUtils
import com.lagou.commonlibs.web.router.Router
import com.lagou.socketchannel.http.HttpCallback
import com.lagou.socketchannel.http.HttpRequest
import com.lagou.socketchannel.http.HttpResponse
import com.lagou.socketchannel.http.event.HttpEventResponse
import kotlinx.android.synthetic.main.activity_ai_interview_voice.ai_answer_animation
import kotlinx.android.synthetic.main.activity_ai_interview_voice.constraint_root
import kotlinx.android.synthetic.main.activity_ai_interview_voice.image_ai_baby
import kotlinx.android.synthetic.main.activity_ai_interview_voice.image_ai_back_mask
import kotlinx.android.synthetic.main.activity_ai_interview_voice.lin_position
import kotlinx.android.synthetic.main.activity_ai_interview_voice.pure_voice_view
import kotlinx.android.synthetic.main.activity_ai_interview_voice.tv_position_content
import kotlinx.android.synthetic.main.activity_ai_interview_voice.tv_previous_question
import pl.droidsonroids.gif.GifDrawable
import java.io.File
import java.util.concurrent.LinkedBlockingQueue

/**
 * Time：2024/8/1 16:37:23
 * Author：wangjunwei
 * Description：
 * Ai语音面试主页面
 */
@RouterRule(ROUTER_URL)
class AiInterviewVoiceActivity : BaseActivity(), OnSingleMethodListener<Float> {

    /**
     * 指定sessionId的会话详情
     */
    private var aiInterviewSession: AiInterviewSessionListRes? = null

    /**
     * 会话已经创建
     */
    private var sessionId = 0


    /**
     * 从Ai面试浮动入口进来，携带一部分数据：
     * 注意处理未开始面试的情况：点进来不建立聊天，端上需要构建一个SessionList列表
     * aiInterviewFloatEntryRes中包含开场白、职位id
     * 已经开始面试或者面试结束不用单独处理，直接走现有流程，目前不支持多个职位同时开启Ai面试
     */
    private var aiInterviewFloatEntryRes: AiInterviewFloatEntryRes? = null

    /**
     * 是否来源于push
     */
    private var isFromPush = false

    /**
     * Ai 播报待转化文本的下标 每次转换的文本数量
     */
    private var convertVoiceCurrentIndex = 0

    /**
     * Ai生成的内容是分段给出的，所以生成的音频文件也是分片的，存储一次Ai生成文本对应的音频文件
     */
    private val voiceFilePathQueue = LinkedBlockingQueue<String>()

    /**
     * 用于记录生成语音的文本对象
     */
    private var aiSessionItemGenerateVoice: AiInterviewSessionListRes.AiSessionItem =
        AiInterviewSessionListRes.AiSessionItem()

    /**
     * 是否是生成的第一条语音
     */
    private var isFirstVoice = true

    /**
     * 记录当前Ai返回的文本，目前使用追加模式，为了实现打字效果和语音播报需要组合成完整文本
     */
    private var currentAiSessionItem: AiInterviewSessionListRes.AiSessionItem? = null

    /**
     * 文本你是否正在生成中
     */
    private var isTextGenerating = false

    /**
     * 语音是否正在生成中
     */
    private var isVoiceGenerating = false

    /**
     * 生成语音是否结束
     */
    @Volatile
    private var isGenerateVoiceEnd = true

    /**
     * 最新的会话状态
     */
    private var newSessionState: Int = 1000


    companion object {
        const val KEY_SESSION_ID = "sessionId"
        const val ROUTER_URL = "lagou://lagou.com/ai/interview/voice?$KEY_SESSION_ID={sessionId}"

        /***
         * 从浮层Ai面试入口进入，携带数据
         */
        const val KEY_AI_INTERVIEW_FLOAT_ENTRY = "AI_INTERVIEW_FLOAT_ENTRY"
    }

    override fun getContentLayoutId(): Int {
        return R.layout.activity_ai_interview_voice
    }

    override fun init() {
        val routeUrl = RouteUtils.getRouteUrl(intent)
        if (!TextUtils.isEmpty(routeUrl) && routeUrl.contains(KEY_SESSION_ID)) {
            sessionId = RouteUtils.getQueryParameter<Int>(routeUrl, KEY_SESSION_ID, Int::class.java)

        } else if (intent.hasExtra(KEY_SESSION_ID)) {
            sessionId = intent.getIntExtra(KEY_SESSION_ID, 0)
        } else if (intent.hasExtra(KEY_AI_INTERVIEW_FLOAT_ENTRY)) {
            aiInterviewFloatEntryRes = intent?.getParcelableExtra(KEY_AI_INTERVIEW_FLOAT_ENTRY)
        }
        if (sessionId <= 0 && aiInterviewFloatEntryRes == null) {
            finish()
            return
        }

        //0 是否来自push
        isFromPush = (intent.extras?.getSerializable(Router.INTENT_KEY_EXTENT_DATA)
                as? WebViewExtentBean)?.source == PositionDetailActivity.SOURCE.JD_PUSH
        //1 Ai 机器人设置
        val animationDrawable = GifDrawable(
            resources, R.drawable.xiaowa_min
        )
        image_ai_baby.setImageDrawable(animationDrawable)
        animationDrawable.loopCount = 0

        //2 获取指定sessionId的会话列表
        fetchAiInterviewSessionList()

        initListener()

        //3 自动开启自动播报语音（进入纯语音面试页面）
        OkBus.getInstance().onEvent(EventTags.OPEN_SELECT_PLAY_VOICE, true)
    }

    private fun initListener() {
        /**
         * 0 进入JD详情页
         */
        lin_position.setOnClickListener {
            aiInterviewSession?.positionId?.let {
                if (it <= 0) {
                    return@setOnClickListener
                }
                IntentUtils.startPositionDetailActivity(this@AiInterviewVoiceActivity, it.toLong())
            }
        }
        /**
         * 1 录音交互自定义View - 设置发送消息回调
         */
        pure_voice_view.setOnSendListener { msgType, contentOrPath ->
            if (TextUtils.isEmpty(contentOrPath)) {
                return@setOnSendListener
            }

            //1 语音类型路径为空或者语音文件不存在
            if (!(File(contentOrPath).exists())) {
                return@setOnSendListener
            }

            //1.1 先调用语音转文字接口，把用户语音转换为文本
            pure_voice_view.changeState(PureVoiceView.AIVoiceState.SENDING)
            uploadVoiceToText(contentOrPath)
        }

        //2 录音交互自定义View - 设置创建会话或者开始面试回调
        pure_voice_view.setStartInterviewListener {
            //如果还没有创建会话，就先创建会话
            if (aiInterviewSession?.isNotCreateSession == true) {
                createNewAiSession()
                return@setStartInterviewListener
            }
            if (aiInterviewSession?.isInitState == true) {
                //如果还没有开始面试，就发起开始面试
                chatInterviewAdd(null, isVoiceType = false)
                return@setStartInterviewListener
            }
        }

        //3 录音过程中,停止播报
        pure_voice_view.setRecordingListener(object : OnSingleMethodListener<Boolean> {
            override fun onCallBack(isRecording: Boolean) {
                if (isRecording) MediaManager.pause()
            }
        })

        //4 点击任意区域，如果是暂停就恢复
        constraint_root.setOnClickListener {
            pure_voice_view.resumeRecord()
            image_ai_back_mask.visibility = View.GONE
        }

        //5 设置恢复和暂停回调
        pure_voice_view.setPauseResumeListener {
            image_ai_back_mask.visibility = if (it) View.GONE else View.VISIBLE
        }
    }

    /**
     * 1 指定sessionId获取会话列表
     */
    private fun fetchAiInterviewSessionList() {
        pure_voice_view.showLoading()

        AiNetManager.fetchAiInterviewSessionList(object :
            OnAdapterCommonListenerImpl<List<AiInterviewSessionListRes?>?,
                    HttpResponse<List<AiInterviewSessionListRes?>?>>(this) {
            override fun onSuccess(aiInterviewSessionList: List<AiInterviewSessionListRes?>?) {
                super.onSuccess(aiInterviewSessionList)
                val realAiInterviewSessionList = mutableListOf<AiInterviewSessionListRes?>()
                //1 接口返回的会话列表添加到新列表中
                aiInterviewSessionList?.let {
                    realAiInterviewSessionList.addAll(it)
                }

                //2 判断当前会话中是否有正在进行中的，如果有则不自动构造会话（从Ai面试浮动入口进来，客户端自己构造的职位卡片和开始面试入口）
                val tempAiInterviewSession = realAiInterviewSessionList.firstOrNull()
                //2.1 只有面试处于结束或者面试未开始的情况下，才能往session列表中增加自己构造的职位卡片和开始面试入口
                //2.2 ---827修改，可以随意创建会话，没有结束也可以创建（但要确保两个职位的id不一样，才会把本地构造的欢迎语、职位卡片添加到列表中）
                //if (tempAiInterviewSession == null || tempAiInterviewSession.isFinishState) {
                //3 如果是从Ai浮动入口进入，需要用上一页带出来的数据覆盖构造
                if (tempAiInterviewSession == null || tempAiInterviewSession.positionId != aiInterviewFloatEntryRes?.positionId) {
                    val aiInterviewChatRes =
                        AiInterviewSessionListRes.createSessionListRes(aiInterviewFloatEntryRes)
                    if (aiInterviewChatRes != null) {
                        realAiInterviewSessionList.add(0, aiInterviewChatRes)
                    }
                }

                this@AiInterviewVoiceActivity.aiInterviewSession =
                    realAiInterviewSessionList.firstOrNull()

                aiInterviewSession?.let { interviewSession ->
                    newSessionState = interviewSession.sessionState

                    //1 获取职位信息，用于显示
                    fetchPositionDetail(interviewSession.positionId.toLong())

                    //2 跑马灯效果显示最近一次Ai提问
                    val showText =
                        if (interviewSession.isNotCreateSession) aiInterviewFloatEntryRes?.prologue
                        else {
                            //1 如果是已创建会话，还未开始面试，则取第一句
                            if (interviewSession.isInitState){
                                interviewSession.sessionItemList.firstOrNull { it.isSystemType }?.content
                            }else {
                                interviewSession.sessionItemList.lastOrNull { it.isModelType }?.content
                            }
                        }
                    if (TextUtils.isEmpty(showText) || interviewSession.isFinishState) {
                        tv_previous_question.visibility = View.INVISIBLE
                    } else {
                        tv_previous_question.visibility = View.VISIBLE
                        tv_previous_question.isSelected = true
                        tv_previous_question.setHorizontallyScrolling(true)
                        tv_previous_question.text = showText
                    }

                    //3 核心交互部分样式设置：录音、播放、暂停等
                    fillPageStyle(interviewSession)
                }
            }

            override fun isCareResponse(): Boolean {
                return false
            }
        })
    }

    /***
     * 获取会话状态成功之后，填充页面样式
     */
    private fun fillPageStyle(interviewSession: AiInterviewSessionListRes) {
        pure_voice_view.setData(aiInterviewSession)

        //1 会话已经结束
        if (interviewSession.isFinishState) {
            pure_voice_view.changeState(PureVoiceView.AIVoiceState.SESSION_END)
            return
        }
        //2 当前未授予音频录制权限或者来自push-右侧显示接听按钮
        if (!RuntimePermissionUtil.checkRecordPermission(this) || isFromPush) {
            pure_voice_view.changeState(PureVoiceView.AIVoiceState.WAIT_LISTEN)
            //if (interviewSession.isNotCreateSession || interviewSession.isInitState) {
            //    //如果还没有开始面试，就发起开始面试
            //    chatInterviewAdd(null, isVoiceType = false)
            // }
            return
        }

        //3 已经授予录音权限&&也不是来自push
        //3.1 面试未开始状态
        pure_voice_view.changeState(PureVoiceView.AIVoiceState.LOADING)
        //如果还没有创建会话，就先创建会话
        if (interviewSession.isNotCreateSession) {
            createNewAiSession()
            return
        }
        if (interviewSession.isInitState) {
            //如果还没有开始面试，就发起开始面试
            chatInterviewAdd(null, isVoiceType = false)
            return
        }

        //3.2 面试中状态-开启录音，接收用户语音输入
        pure_voice_view.realStartRecord()
    }

    /**
     * 语音转文本：用户输入语音，调用asr接口转换为文字，然后提交大模型
     */
    private fun uploadVoiceToText(contentOrPath: String?): HttpRequest<AiInterviewAsrRes>? {

        if (contentOrPath.isNullOrEmpty()) return null


        return AiNetManager.uploadVoiceToText(
            contentOrPath,
            null,
            object : HttpCallback<AiInterviewAsrRes?> {
                override fun onSuccess(
                    request: HttpRequest<*>,
                    httpResponse: HttpResponse<AiInterviewAsrRes?>
                ) {
                    if (TextUtils.isEmpty(httpResponse.content?.asrText)) {
                        ToastUtil.show(this@AiInterviewVoiceActivity, "未识别到文字")
                        pure_voice_view.postDelayed({
                            pure_voice_view.realStartRecord()
                        }, 800)
                        return
                    }
                    chatInterviewAdd(httpResponse.content?.asrText)
                }

                override fun onFailure(
                    request: HttpRequest<*>,
                    httpResponse: HttpResponse<AiInterviewAsrRes?>
                ) {
//                    LogUtils.i("uploadVoiceToText", "上传语音失败 response = " + httpResponse)
                    if (isDestroyed) {
                        return
                    }
                    if (!TextUtils.isEmpty(httpResponse.message)) {
                        ToastUtil.show(this@AiInterviewVoiceActivity, httpResponse.message)
                        pure_voice_view.postDelayed({
                            pure_voice_view.realStartRecord()
                        }, 800)
                    }
                }
            })
    }

    /**
     * 新创建会话
     * 适用场景：从Ai面试浮动入口进来的，类比聊天，这里点击开始面试如果会话未创建，先创建会话，接着在发起Ai对话
     */
    private fun createNewAiSession() {
        AiNetManager.createAiInterviewSession(
            aiInterviewSession,
            object : OnSingleMethodListener<AIInterviewCreateRes> {
                override fun onCallBack(aiInterviewCreateRes: AIInterviewCreateRes?) {
                    if (isDestroyed || aiInterviewCreateRes == null) {
                        return
                    }

                    //fetchAiInterviewSessionList()
                    //0 实际是刚创建会话状态，强制设置为开始面试状态，因为紧接着和Ai交互的请求就发出
                    aiInterviewSession?.sessionId = aiInterviewCreateRes.sessionId
                    aiInterviewSession?.sessionState = IAiInterviewSessionState.START_INTERVIEW

                    //2 会话创建成功继续调用开始面试接口（与大模型开始交互）
                    chatInterviewAdd(null, isVoiceType = false)

                    //3 更新JD页面Ai面试入口数据
                    OkBus.getInstance().onEvent(EventTags.UPDATE_AI_INTERVIEW_FLOAT_ENTRY)
                }
            })
    }


    /**
     * 和大模型交互，获取大模的回答
     * @param answer 用户的回答
     * @param isVoiceType 是否是语音类型
     */
    private fun chatInterviewAdd(answer: String?, isVoiceType: Boolean = true) {

        /**
         * 用户问题同步到AI 文字和语音页面
         */
        //OkBus.getInstance().onEvent(EventTags.FROM_VOICE_UPDATE_USER_INPUT, answer)

        MediaManager.pause()
        showLoading()
        //新的Ai 生成内容请求发出，重置老的音频相关的变量
        convertVoiceCurrentIndex = 0
        voiceFilePathQueue.clear()
        aiSessionItemGenerateVoice = AiInterviewSessionListRes.AiSessionItem()
        isFirstVoice = true
        currentAiSessionItem = null
        isTextGenerating = true

        aiInterviewSession?.apply {
            AiNetManager.chatInterviewAdd(
                answer = answer,
                originalTag = null,
                positionId = positionId,
                sessionId = sessionId,
                voiceMessage = isVoiceType,
                baseEventCallback = object : BaseEventCallback<AIInterviewChatRes?>() {
                    override fun onSuccess(
                        httpRequest: HttpRequest<*>,
                        httpResponse: HttpEventResponse<AIInterviewChatRes?>
                    ) {
                        onRequestCall(httpRequest, httpResponse)
                    }

                    override fun onClose(
                        httpRequest: HttpRequest<*>,
                        httpEventResponse: HttpEventResponse<AIInterviewChatRes?>
                    ) {
                        // onRequestCall(httpRequest, httpEventResponse)
                        isTextGenerating = false
                    }

                    override fun onFailure(
                        httpRequest: HttpRequest<*>,
                        httpEventResponse: HttpEventResponse<AIInterviewChatRes?>
                    ) {
                        super.onFailure(httpRequest, httpEventResponse)
                        //onRequestCall(httpRequest, httpEventResponse)
                        if (isDestroyed) {
                            return
                        }
                        pure_voice_view.postDelayed({
                            pure_voice_view.realStartRecord()
                        }, 500)
                    }

                    private fun onRequestCall(
                        httpRequest: HttpRequest<*>,
                        httpResponse: HttpEventResponse<AIInterviewChatRes?>
                    ) {
                        if (isDestroyed) {
                            return
                        }
                        hideLoading()
                        //一旦A响应成功，隐藏最近一条Ai的会话
                        tv_previous_question.visibility = View.INVISIBLE

                        val aiInterviewChatRes = httpResponse.content ?: AIInterviewChatRes()
                        aiInterviewChatRes.apply {
                            if (httpRequest.tag is Int) tag = httpRequest.tag as Int
                        }

                        //结果同步到文字和语音混合面试页面++++++
                        //aiInterviewChatRes.userAnswer = answer
                        //OkBus.getInstance()
                        //    .onEvent(EventTags.FROM_VOICE_UPDATE_AI_ANSWER, aiInterviewChatRes)

                        //更新录音&播放交互组件回话的状态，防止意外情况：比如面试未开始无线触发自动发起面试行为
                        aiInterviewSession?.sessionState = aiInterviewChatRes.convertStatusToSessionState()
                        pure_voice_view.setData(aiInterviewSession)

                        //会话结束，更新语音页面状态
                        if (aiInterviewChatRes.isFinishState) pure_voice_view.changeState(
                            PureVoiceView.AIVoiceState.SESSION_END
                        )

                        //最新的会话状态
                        newSessionState = aiInterviewChatRes.sessionState
                        //0 发送录音按钮样式处理
                        //当前流式过程结束，才把按钮置为可点击
                        isTextGenerating = !aiInterviewChatRes.isLastCompleteData

                        //1 目前流用的是追加模式，所以在给打字器之前，必须组合成完整的文本
                        if (currentAiSessionItem == null) {
                            currentAiSessionItem =
                                AiInterviewSessionListRes.AiSessionItem.convert(aiInterviewChatRes)
                        } else {
                            currentAiSessionItem?.apply {
                                status = aiInterviewChatRes.status
                                tag = aiInterviewChatRes.tag
                                if (aiInterviewChatRes.isSuccess) {
                                    content += aiInterviewChatRes.content
                                } else {
                                    //把面试评价追加上去
                                    content =
                                        aiInterviewChatRes.content + (aiInterviewChatRes.evaluate
                                            ?: "")
                                }
                                sessionState = aiInterviewChatRes.sessionState
                                messageId = aiInterviewChatRes.messageId
                                itemRelateMsgList = aiInterviewChatRes.itemRelateMsgList
                                isLoading = false
                            }
                        }

                        //2 如果status状态不是0 -1
                        if ((aiInterviewChatRes.isExceptionStatus || aiInterviewChatRes.isSessionEnd) && !TextUtils.isEmpty(
                                aiInterviewChatRes.statusMessage
                            )
                        ) {
                            ToastUtil.show(
                                this@AiInterviewVoiceActivity,
                                aiInterviewChatRes.statusMessage
                            )
                            //aiGeneratingAutoPlay(aiInterviewChatRes.statusMessage, null)
                            return
                        }

                        //3 成功获取结果，打字更新显示
                        currentAiSessionItem?.let {
                            it.userAnswerText = answer
                            //3.1 分片转换为语音，进行自动播报
                            aiGeneratingAutoPlay(it)
                        }
                    }
                }
            )
        }
    }

    /**
     * 异常信息，生成音频&播报
     */
    private fun aiGeneratingAutoPlay(text: String?, callback: (() -> Unit)?) {

        if (isActivityDestroyed || TextUtils.isEmpty(text)) {
            return
        }
        if (isVoiceGenerating) {
            return
        }
        isVoiceGenerating = true
        AiNetManager.convertTextVoice(
            text = text,
            voiceType = CMKVUtils.getSelectTimbre()?.value,
            baseCallback = object : BaseCallback<AiInterviewTtsRes?>() {

                override fun onSuccess(
                    request: HttpRequest<*>,
                    response: HttpResponse<AiInterviewTtsRes?>
                ) {
                    isVoiceGenerating = false
                    //取消语音合成
                    if (isActivityDestroyed) {
                        return
                    }
                    response.content?.voicePath?.apply {
                        voiceFilePathQueue.offer(this)
                        //异常信息也生成语音播放&执行动画
                        ai_answer_animation.startAnimation()
                        pure_voice_view.changeState(PureVoiceView.AIVoiceState.AI_SPEAK)
                        playVoiceService()

                        //执行语音播报回调
                        callback?.invoke()
                    }
                }

                override fun onFailure(
                    request: HttpRequest<*>, response: HttpResponse<AiInterviewTtsRes?>
                ) {
                    super.onFailure(request, response)
                    isVoiceGenerating = false
                }
            })
    }

    /**
     * Ai生成的文本自动转化为音频文件，并启动播放
     */
    private fun aiGeneratingAutoPlay(aiSessionItem: AiInterviewSessionListRes.AiSessionItem) {
        aiSessionItemGenerateVoice = aiSessionItem

        //1 关闭播放，取消语音合成
        if (isActivityDestroyed) {
            voiceFilePathQueue.clear()
            return
        }
        //2 语音生成中，只有语音生成中才有效（防止涉生成语音不完整）
        if (isVoiceGenerating) {
            return
        }
        //3 说明语音合成已完成
        if (TextUtils.isEmpty(aiSessionItemGenerateVoice.content) || convertVoiceCurrentIndex >= aiSessionItemGenerateVoice.content.length) {
            return
        }

        //当前文本
        val sourceContent = aiSessionItemGenerateVoice.content
        //指定下标开始的，首个标点符号位置
        val firstSymbolIndex =
            StringUtil.findPunctuationStartIndex(sourceContent, convertVoiceCurrentIndex)
        //4 不是最后一个包，如果内容不到一个标点符号（说明不完整），则等待，降低语音转换接口的调用，保障合成语音的语气语境
        if (aiSessionItemGenerateVoice.isSuccess && firstSymbolIndex == -1) {
            return
        }
        //5 截取文本分段合成语音
        //5.1 如果是正常返回语音，分段合成；如果是语音返回结束或者出现异常，则直接合成剩余文字
        val step = if (aiSessionItemGenerateVoice.isSuccess) {
            if (convertVoiceCurrentIndex + firstSymbolIndex < sourceContent.length) firstSymbolIndex + 1
            else sourceContent.length - convertVoiceCurrentIndex
        } else {
            sourceContent.length - convertVoiceCurrentIndex
        }

        val waitConvertContent =
            sourceContent.substring(convertVoiceCurrentIndex, convertVoiceCurrentIndex + step)

        LogUtils.i(
            "aiGeneratingAutoPlay", "当前下标：$convertVoiceCurrentIndex  转换文字的个数：$step " +
                    "待转换内容 waitConvertContent = $waitConvertContent  总的内容：aiSessionItem.content = " + aiSessionItem.content
        )
        isGenerateVoiceEnd = false
        convertVoiceCurrentIndex += step
        isVoiceGenerating = true
        AiNetManager.convertTextVoice(
            text = waitConvertContent,
            voiceType = CMKVUtils.getSelectTimbre()?.value,
            baseCallback = object : BaseCallback<AiInterviewTtsRes?>() {

                override fun onSuccess(
                    request: HttpRequest<*>,
                    response: HttpResponse<AiInterviewTtsRes?>
                ) {
                    isVoiceGenerating = false
                    //取消语音合成
                    if (isActivityDestroyed) {
                        return
                    }
//                    LogUtils.i(
//                        "aiGeneratingAutoPlay",
//                        "生成：2 voicePath = ${aiInterviewTtsRes?.voicePath} ；；； waitConvertContent = $waitConvertContent"
//                    )
                    response.content?.voicePath?.apply {
                        voiceFilePathQueue.offer(this)
                        //1 生成的第一条的语音之后，触发播放操作
                        if (isFirstVoice) {
                            isFirstVoice = false
                            //开始播放第一个音频时，执行动画
                            ai_answer_animation.startAnimation()
                            pure_voice_view.changeState(PureVoiceView.AIVoiceState.AI_SPEAK)

                            playVoiceService()
                        }
                    }

                    //2 一次Ai语音结束，判断生成的语音是否覆盖所有的Ai文本，如果没有生成结束，继续调用aiGeneratingAutoPlay生成语音
                    //  注意递归调用
                    if (convertVoiceCurrentIndex < aiSessionItemGenerateVoice.content.length) {
                        aiGeneratingAutoPlay(aiSessionItemGenerateVoice)
                    } else {
                        isGenerateVoiceEnd = true
                    }
                }

                override fun onFailure(
                    request: HttpRequest<*>, response: HttpResponse<AiInterviewTtsRes?>
                ) {
                    super.onFailure(request, response)
                    isVoiceGenerating = false
                    isGenerateVoiceEnd = true
                }
            })
    }

    /**
     * 播放音频
     */
    private val onCompletionListener = object : MediaPlayer.OnCompletionListener {
        override fun onCompletion(mp: MediaPlayer) {
            ThreadUtil.io {
                //1 如果生成语音结束，则判断音频文件列表是否为空，为空说明已经完成所有音频播放，结束播放环节，进入继续录音环节
                if (isGenerateVoiceEnd && voiceFilePathQueue.isEmpty()) {
                    ThreadUtil.ui {
                        //播放完成，播放流程就断了，后续语音在识别成功也不会播放，所以要把流程续上
                        isFirstVoice = true
                        ai_answer_animation.startReverseAnimation()

                        //判断会话是否结束，未结束继续开启录音模式
                        if (newSessionState != IAiInterviewSessionState.END_INTERVIEW) {
                            pure_voice_view.realStartRecord()
                        }
                    }
                    return@io
                }

                val voiceFilePath = voiceFilePathQueue.take()
                //LogUtils.i("aiGeneratingAutoPlay", "播放：2 voiceFilePath = $voiceFilePath")
                ThreadUtil.ui {
                    MediaManager.playSound(
                        voiceFilePath, this,
                        this@AiInterviewVoiceActivity, this@AiInterviewVoiceActivity
                    )
                }
            }
        }
    }

    private fun playVoiceService() {
        if (isActivityDestroyed) {
            voiceFilePathQueue.clear()
            MediaManager.pause()
            return
        }
        val voiceFilePath = voiceFilePathQueue.poll()
        //LogUtils.i("aiGeneratingAutoPlay", "播放：2 voiceFilePath = $voiceFilePath")
        MediaManager.playSound(voiceFilePath, onCompletionListener, this, this)
    }

    /**
     * 获取职位详情
     */
    fun fetchPositionDetail(positionId: Long) {

        PositionDetailController.getInstance()
            .requestPositionDetailDataUseCache(positionId, object :
                GetPositionDetailDataHandler {
                override fun requestDataSuccess(bean: PositionDetailBean?) {
                    if (bean == null) {
                        return
                    }
                    CBusinessMemoryCacheManager.cachePositionDetail(
                        SafeTypeConvert.toInt(bean.positionId),
                        bean
                    )
                    tv_position_content.text = bean.positionName
                }

                override fun onError(status: Int) {
                }
            })
    }

    override fun showLoading() {
        //super.showLoading()
        pure_voice_view.showLoading()
    }

    override fun hideLoading() {
        //super.hideLoading()
        pure_voice_view.hideLoading()
    }

    override fun onStop() {
        super.onStop()
        pure_voice_view.pauseRecord()
    }

    override fun onRestart() {
        super.onRestart()
        pure_voice_view.resumeRecord()
    }

    override fun getStatusBarColor(): Int {
        return R.color.color_031221
    }

    /**
     * 播放音频振幅率监听，更新播放音频动效效果
     */
    override fun onCallBack(amplitudeRate: Float) {
        //Ai回答语音实时振幅回调
        ai_answer_animation.amplitude = amplitudeRate
    }

    override fun onBackPressed() {
        super.onBackPressed()
        //页面关闭，刷新下Ai面试页面的列表
        OkBus.getInstance().onEvent(EventTags.AI_INTERVIEW_FLUSH_SESSION_LIST)
    }

    override fun onDestroy() {
        super.onDestroy()
        MediaManager.release()
        voiceFilePathQueue.clear()

        ThreadUtil.io {
            //清除指定目录的文件
            FileUtils.deleteFile(AiNetManager.getTtsOutDir(this))
        }
    }

    override fun getPageId(): String {
        return "m1uiq"
    }

    override fun getShowLogInfo(): ShowLogInfo {
        val showLogInfo = super.getShowLogInfo()
        showLogInfo.setAddress_Id("w1ujq")
        return showLogInfo
    }
}