package com.alpha.lagouapk.ui.interview.ai

import androidx.fragment.app.Fragment
import androidx.lifecycle.lifecycleScope
import androidx.viewpager.widget.ViewPager
import com.alpha.lagouapk.R
import com.alpha.lagouapk.bean.aiinterview.TimbreCategory
import com.alpha.lagouapk.manager.CMKVUtils
import com.alpha.lagouapk.manager.net.AiNetManager
import com.alpha.lagouapk.observer.EventTags
import com.lagou.baselib.BaseActivity
import com.lagou.baselib.analytic.AnalyticHelper
import com.lagou.baselib.analytic.bean.json.ClickInfo
import com.lagou.chat.chatlibrary.manager.MediaManager
import com.lagou.commonlibs.event.Event
import com.lagou.commonlibs.event.OkBus
import kotlinx.android.synthetic.main.activity_ai_timbre_select.slid_tab_custom
import kotlinx.android.synthetic.main.activity_ai_timbre_select.title_bar
import kotlinx.android.synthetic.main.activity_ai_timbre_select.view_pager_timbre
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch

/**
 * Time：2024/7/29 10:27:54
 * Author：wangjunwei
 * Description：Ai 音色选择页
 *
 */
class AiTimbreSelectActivity : BaseActivity() {

    private val titles = arrayOf(TimbreCategory.recommend.displayName,
        TimbreCategory.male.displayName, TimbreCategory.female.displayName,
        TimbreCategory.accents.displayName)
    /**
     * 当前的fragment
     */
    private var currentFragment: AiTimbreSelectFragment? = null

    /**
     * 选择音色并开启自动播放
     */
    private val openSelectTimbreEvent = Event { msg ->
        if (msg.obj is Boolean) {
            title_bar.imgRightTwo.isSelected = msg.obj as Boolean
            CMKVUtils.setAutoConvertVoice(title_bar.imgRightTwo.isSelected)
        }
    }

    override fun getContentLayoutId(): Int {
        return R.layout.activity_ai_timbre_select
    }

    override fun init() {
        initTitle()
        initListener()
        initFragment()
    }
    private fun initTitle() {
        title_bar.imgRightTwo.isSelected = CMKVUtils.isAutoConvertVoice()
    }

    private fun initListener() {
        title_bar.setLeftImgListener {
            finish()
        }
        title_bar.setRightImgListener {
            val clickInfo = ClickInfo()
            clickInfo.address_id = "s1ujk"
            clickInfo.addClick_props("type", if (CMKVUtils.isAutoConvertVoice()) 0 else 1)
            AnalyticHelper.getInstance().sendClickInfo(clickInfo)

            // 开启和关闭语音播报
            title_bar.imgRightTwo.isSelected = !title_bar.imgRightTwo.isSelected
            OkBus.getInstance().onEvent(EventTags.OPEN_SELECT_PLAY_VOICE, title_bar.imgRightTwo.isSelected)
        }

        OkBus.getInstance().register(EventTags.OPEN_SELECT_PLAY_VOICE, openSelectTimbreEvent)
    }

    private fun initFragment() {
        lifecycleScope.launch(Dispatchers.Main){
            //通过协程获取
            val aiTimbreRes = AiNetManager.getTimbreListData()

            val fragments = ArrayList<Fragment>()

            titles.forEachIndexed { index, key ->
                when(key){
                    TimbreCategory.recommend.displayName -> fragments.add(AiTimbreSelectFragment.newInstance(TimbreCategory.recommend,
                        aiTimbreRes?.getRecommendTimbreList()))
                    TimbreCategory.male.displayName -> fragments.add(AiTimbreSelectFragment.newInstance(TimbreCategory.male,
                        aiTimbreRes?.getMaleTimbreList()))
                    TimbreCategory.female.displayName -> fragments.add(AiTimbreSelectFragment.newInstance(TimbreCategory.female,
                        aiTimbreRes?.getFemaleTimbreList()))
                    TimbreCategory.accents.displayName -> fragments.add(AiTimbreSelectFragment.newInstance(TimbreCategory.accents,
                        aiTimbreRes?.getAccentTimbreList()))
                }

            }
            currentFragment = fragments[0] as AiTimbreSelectFragment

            //1 初始化viewpager相关的数据
            view_pager_timbre.offscreenPageLimit = fragments.size
            slid_tab_custom.setViewPager(view_pager_timbre, titles, this@AiTimbreSelectActivity, fragments)

            view_pager_timbre.addOnPageChangeListener(object : ViewPager.SimpleOnPageChangeListener() {
                override fun onPageSelected(position: Int) {
                    super.onPageSelected(position)
                    currentFragment = fragments[position] as AiTimbreSelectFragment
                }
            })
        }
    }

    override fun onStop() {
        super.onStop()
        MediaManager.release()
    }

    override fun onDestroy() {
        super.onDestroy()
        OkBus.getInstance().unRegister(openSelectTimbreEvent)
    }
}
