package com.alpha.lagouapk.ui.interview.ai

import android.content.Context
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.TextView
import androidx.recyclerview.widget.DividerItemDecoration
import androidx.recyclerview.widget.RecyclerView
import com.alpha.lagouapk.R
import com.alpha.lagouapk.bean.aiinterview.AiTimbreSelectRes
import com.alpha.lagouapk.bean.aiinterview.TimbreCategory
import com.alpha.lagouapk.manager.CMKVUtils
import com.alpha.lagouapk.manager.net.AiNetManager
import com.alpha.lagouapk.observer.EventTags
import com.common.adapter.viewholder.BaseVHAdapter
import com.common.adapter.viewholder.BaseViewHolder
import com.common.inter.OnItemClickListener
import com.common.utils.GlideHelper
import com.lagou.baselib.BaseKotlinFragment
import com.lagou.baselib.download.SimpleDownloadListener
import com.lagou.chat.chatlibrary.manager.MediaManager
import com.lagou.commonlibs.event.OkBus
import com.lagou.commonlibs.imageload.ImageLoadUtils
import com.lagou.commonlibs.imageload.glide.impl.GlideOptionHelper
import com.lagou.commonlibs.utils.ResourceUtils
import kotlinx.android.synthetic.main.fragment_ai_timbre_select.recycle_view_timbre
import java.io.File

/**
 * Time：2024/7/29 11:27:54
 * Author：wangjunwei
 * Description：Ai 音色选择-fragment
 */
class AiTimbreSelectFragment : BaseKotlinFragment() {

    private var timbreType: String? = null
    private var timbreList: ArrayList<AiTimbreSelectRes.VoiceTimbreType.VoiceTimbre?>? = null
    private var aiTimbreSelectAdapter : AiTimbreSelectAdapter? = null

    /**
     * 选中的音色
     */
    private var selectTimbre: AiTimbreSelectRes.VoiceTimbreType.VoiceTimbre? = null

    companion object {

        private const val KEY_TIMBRE_TYPE = "timbre_type"
        private const val KEY_TIMBRE_LIST = "timbre_list"

        /**
         * 根据音色类型创建不同的Fragment
         */
        fun newInstance(timbreCategory: TimbreCategory,
                        timbreList: List<AiTimbreSelectRes.VoiceTimbreType.VoiceTimbre?>?): AiTimbreSelectFragment {
            val bundle = Bundle()
            bundle.putString(KEY_TIMBRE_TYPE, timbreCategory.name)
            bundle.putParcelableArrayList(KEY_TIMBRE_LIST, timbreList as ArrayList<AiTimbreSelectRes.VoiceTimbreType.VoiceTimbre?>?)
            val fragment = AiTimbreSelectFragment()
            fragment.arguments = bundle
            return fragment
        }
    }

    override fun getLayoutId(): Int {
        return R.layout.fragment_ai_timbre_select
    }

    override fun init() {
        timbreList = arguments?.getParcelableArrayList(KEY_TIMBRE_LIST)
        timbreType = arguments?.getString(KEY_TIMBRE_TYPE)
    }

    override fun fillUi() {
        initRecyclerView()
        initListener()
    }

    private fun initListener() {
        aiTimbreSelectAdapter?.setItemClickListener(object : OnItemClickListener<AiTimbreSelectRes.VoiceTimbreType.VoiceTimbre?> {
            override fun onItemClickListener(
                itemView: View,
                position: Int,
                bean: AiTimbreSelectRes.VoiceTimbreType.VoiceTimbre?
            ) {
                selectTimbre = bean
                OkBus.getInstance().onEvent(EventTags.OPEN_SELECT_PLAY_VOICE, true)
                CMKVUtils.setSelectTimbre(bean)
            }
        })
    }

    private fun initRecyclerView() {
        val dividerDecoration = DividerItemDecoration(mContext, DividerItemDecoration.VERTICAL)
        dividerDecoration.setDrawable(ResourceUtils.getDrawable(R.drawable.recycler_divider))
        aiTimbreSelectAdapter = AiTimbreSelectAdapter(mContext)
        aiTimbreSelectAdapter?.setData(true,
            timbreList as List<AiTimbreSelectRes.VoiceTimbreType.VoiceTimbre?>?
        )

        recycle_view_timbre.apply {
            addItemDecoration(dividerDecoration)
            adapter = aiTimbreSelectAdapter
        }
    }

    /**
     * 获取选中的音色
     */
    fun getSelectTimbre(): AiTimbreSelectRes.VoiceTimbreType.VoiceTimbre? {
        return selectTimbre
    }


    inner class AiTimbreSelectAdapter(context: Context) :
        BaseVHAdapter<AiTimbreSelectRes.VoiceTimbreType.VoiceTimbre?>(context) {

        private var itemListener: OnItemClickListener<AiTimbreSelectRes.VoiceTimbreType.VoiceTimbre?>? = null

        fun setItemClickListener(itemListener: OnItemClickListener<AiTimbreSelectRes.VoiceTimbreType.VoiceTimbre?>) {
            this.itemListener = itemListener
        }

        override fun getItemViewType(position: Int): Int {
            return R.layout.item_timbre_select
        }

        override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): AiTimbreViewHolder {
            val itemView = LayoutInflater.from(mContext).inflate(viewType, parent, false)
            return AiTimbreViewHolder(itemView)
        }

        override fun onBindViewHolder(holder: RecyclerView.ViewHolder, position: Int) {
            super.onBindViewHolder(holder, position)
            val voiceTimbre = getItem(position) as? AiTimbreSelectRes.VoiceTimbreType.VoiceTimbre

            (holder as AiTimbreViewHolder).onBindViewHolder(
                holder.itemView,
                voiceTimbre,
                position
            )
        }

        inner class AiTimbreViewHolder(itemView: View) :
            BaseViewHolder<AiTimbreSelectRes.VoiceTimbreType.VoiceTimbre?>(itemView) {

            private val imageTimbreHead = itemView.findViewById<ImageView>(R.id.image_timbre_head)
            private val tvName = itemView.findViewById<TextView>(R.id.tv_name)
            private val imageSelectSymbol =
                itemView.findViewById<ImageView>(R.id.image_select_symbol)

            override fun onBindViewHolder(
                itemView: View,
                voiceTimbre: AiTimbreSelectRes.VoiceTimbreType.VoiceTimbre?,
                position: Int
            ) {
                //1 可选择音色对应的头像
                ImageLoadUtils.with(mContext)
                    .load<String>(voiceTimbre?.icon)
                    .error(R.drawable.icon_login_head)
                    .placeholder(R.drawable.icon_login_head)
                    .displayOption(GlideOptionHelper.getHeaderOption(mContext))
                    .signature(GlideHelper.cacheSignWithIn1Day())
                    .into(imageTimbreHead)
                //2 可选择音色对应的名字
                tvName.text = voiceTimbre?.name
                //3 可选择音色对应的选中状态
                imageSelectSymbol.visibility =
                    if (voiceTimbre?.isSelected == true) View.VISIBLE else View.GONE

                itemView.setOnClickListener {

                    //回调给调用方 记录选中的音色
                    itemListener?.onItemClickListener(itemView, position, voiceTimbre)

                    if (voiceTimbre == null) {
                        return@setOnClickListener
                    }

                    //1 判断当前音色是否被选中，如果被选中，则重新播放一遍语音，其它不做处理
                    if (voiceTimbre.isSelected) {
                        //播放当前语音
                        handlePlayVoice(voiceTimbre)
                        return@setOnClickListener
                    }

                    //2 如果选择的是未被选中的语音，则将所有被选中的语音都设置为不被选中状态
                    //已经播放的音色需要暂停
                    MediaManager.release()

                    mBeans.forEachIndexed { index, voiceTimbre ->
                        //将指定下标的音色标记为选中状态，其他的都设置为不选中状态
                        voiceTimbre?.let {
                            if (position == index) {
                                it.isSelected = true
                                notifyItemChanged(index)
                                //播放选中音色
                                handlePlayVoice(it)
                            } else {
                                if (it.isSelected) {
                                    it.isSelected = false
                                    notifyItemChanged(index)
                                }
                            }
                        }
                    }

                }
            }

            /**
             * 播放语音处理
             */
            private fun handlePlayVoice(voiceTimbre: AiTimbreSelectRes.VoiceTimbreType.VoiceTimbre) {
                AiNetManager.downloadTimbreFile(voiceTimbre.voiceUrl, object :
                    SimpleDownloadListener() {
                    override fun onFinish(file: File) {
                        if (!file.exists()) {
                            return
                        }
                        // 播放选中音色
                        MediaManager.playSound(file.absolutePath, null, baseActivity)
                    }
                })
            }
        }
    }

}
