package com.alpha.lagouapk.ui.interview.ai.adapter

import android.content.Context
import android.text.TextUtils
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.FrameLayout
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.TextView
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.LinearSnapHelper
import androidx.recyclerview.widget.RecyclerView
import com.alpha.lagouapk.R
import com.alpha.lagouapk.bean.aiinterview.AIRelateMsgItem
import com.alpha.lagouapk.bean.aiinterview.AiInterviewSessionListRes.AiSessionItem
import com.alpha.lagouapk.bean.aiinterview.IAiInterviewSessionState
import com.alpha.lagouapk.tools.IntentUtils
import com.alpha.lagouapk.tools.SPUtils
import com.alpha.lagouapk.ui.interview.ai.widget.AiInterviewPositionItem
import com.common.adapter.viewholder.BaseVHAdapter
import com.common.adapter.viewholder.BaseViewHolder
import com.common.utils.GlideHelper.Companion.cacheSignWithIn1Day
import com.lagou.chat.chatlibrary.view.CircleImageView
import com.lagou.chat.chatlibrary.view.EmojiconTextView
import com.lagou.commonlibs.imageload.ImageLoadUtils
import com.lagou.commonlibs.imageload.glide.impl.GlideOptionHelper
import com.lagou.commonlibs.inter.OnSingleMethodListener
import com.lagou.commonlibs.utils.ListUtil
import com.lagou.commonlibs.utils.LogUtils
import com.lagou.commonlibs.utils.ResourceUtils

/**
 * Time：2024/6/14 14:09:45
 * Author：wangjunwei
 * Description：AI 面试适配器
 */
class AIInterviewAdapter(context: Context) : BaseVHAdapter<AiSessionItem>(context) {

    private var startInterviewListener: OnSingleMethodListener<AiSessionItem>? = null

    /**
     * 把语音转换为文本的回调
     */
    private var convertVoiceAsrListener: OnSingleMethodListener<AiSessionItem?>? = null

    /**
     * 和大模型交互，获取大模的回答回调（用于失败重新发送）
     */
    private var chatTextListener: OnSingleMethodListener<AiSessionItem?>? = null

    /**
     * 是否是首次请求职位卡片（首次请求职位卡片&&请求的是最后一个职位卡片，请求完成之后滑动到最后）
     */
    var isFirstSessionPositionItem = true

    override fun setData(isFirst: Boolean, mBeans: List<AiSessionItem>) {
        super.setData(isFirst, mBeans)
    }

    /**
     * 开始面试动作回调到主页面
     */
    fun setStartInterviewListenerMainPage(startInterviewListener: OnSingleMethodListener<AiSessionItem>) {
        this.startInterviewListener = startInterviewListener
    }

    /**
     * 语音转换为文本重试回调
     */
    fun setConvertVoiceAsrListener(convertVoiceAsrListener: OnSingleMethodListener<AiSessionItem?>) {
        this.convertVoiceAsrListener = convertVoiceAsrListener
    }

    /**
     * 和大模型交互，获取大模的回答回调（用于失败重新发送）
     */
    fun setChatTextListener(chatTextListener: OnSingleMethodListener<AiSessionItem?>) {
        this.chatTextListener = chatTextListener
    }

    /**
     * 更新识别文本显示
     */
    fun updateInterviewContent(sessionItem: AiSessionItem) {

        if (mBeans == null) {
            mBeans = mutableListOf()
        }
        val index = mBeans.indexOf(sessionItem)

        //1 不包含在mBeans 直接新增
        if (index == -1) {
            //预留出站位符号位置
            val beforeSessionItem = if (mBeans.size > 0) mBeans[mBeans.size - 1] else null
            if (beforeSessionItem == null || !beforeSessionItem.isOccupyType) {
                mBeans.add(sessionItem)
                notifyItemInserted(mBeans.size - 1)
            } else {
                mBeans.add(mBeans.size - 1, sessionItem)
                notifyItemInserted(mBeans.size - 1)
                notifyItemRangeChanged(mBeans.size - 2, 2)
            }
            return
        }

        //内容为空，且不是loading状态，直接remove
        if (TextUtils.isEmpty(sessionItem.content) && !sessionItem.isLoading) {
            mBeans.removeAt(index)
            notifyItemRemoved(index)
            if (index < mBeans.size - 1) notifyItemRangeChanged(index, mBeans.size - index)
            LogUtils.i("AiInterviewAdapter", "index = $index  mBeans.size = ${mBeans.size}")
            return
        }

        //2 包含aiSessionItem，根据
        val aiSessionItem = mBeans[index]
        aiSessionItem.status = sessionItem.status
        aiSessionItem.sessionId = sessionItem.sessionId
        aiSessionItem.content = sessionItem.content
        aiSessionItem.sessionState = sessionItem.sessionState
        aiSessionItem.messageId = sessionItem.messageId
        aiSessionItem.roleType = sessionItem.roleType
        aiSessionItem.isLoading = sessionItem.isLoading
        aiSessionItem.originalVoicePath = sessionItem.originalVoicePath
        aiSessionItem.isLastMessage = (index == mBeans.size - 2)
        notifyItemChanged(index, 0)
    }

    /**
     * 更新广告显示
     */
    fun updateInterviewContentAds(adsItem: AiSessionItem) {
        if (mBeans == null) {
            mBeans = mutableListOf()
        }
        val index = mBeans.indexOf(adsItem)
        //1 不包含在mBeans 直接新增
        if (index == -1) {
            //预留出站位符号位置
            val beforeSessionItem = if (mBeans.size > 0) mBeans[mBeans.size - 1] else null
            if (beforeSessionItem == null || !beforeSessionItem.isOccupyType) {
                mBeans.add(adsItem)
                notifyItemInserted(mBeans.size - 1)
            } else {
                mBeans.add(mBeans.size - 1, adsItem)
                notifyItemInserted(mBeans.size - 1)
                notifyItemRangeChanged(mBeans.size - 2, 2)
            }
            return
        }
        //2 包含aiSessionItem，根据
        val aiSessionItem = mBeans[index]
        aiSessionItem.itemRelateMsgList = adsItem.itemRelateMsgList
        aiSessionItem.msgType = adsItem.msgType
        aiSessionItem.isLastMessage = (index == mBeans.size - 2)
        notifyItemChanged(index, 0)
    }

    /**
     * 追加的方式更新文本
     */
    fun updateAppendInterviewContent(sessionItem: AiSessionItem) {

        if (mBeans == null) {
            mBeans = mutableListOf()
        }
        val index = mBeans.indexOf(sessionItem)
        //1 不包含在mBeans 直接新增
        if (index == -1) {
            //预留出站位符号位置
            val beforeSessionItem = if (mBeans.size > 0) mBeans[mBeans.size - 1] else null
            if (beforeSessionItem == null || !beforeSessionItem.isOccupyType) {
                mBeans.add(sessionItem)
                notifyItemInserted(mBeans.size - 1)
            } else {
                mBeans.add(mBeans.size - 1, sessionItem)
                notifyItemInserted(mBeans.size - 1)
                notifyItemRangeChanged(mBeans.size - 2, 2)
            }
            return
        }

        //2 包含aiSessionItem，根据
        val aiSessionItem = mBeans[index]
        aiSessionItem.appendOffsetText(sessionItem)
        notifyItemChanged(index, 0)
    }

    /**
     * 更新站位类型
     */
    fun updateOccupyType() {
        val aiSessionItem = AiSessionItem()
        aiSessionItem.tag = -100000
        aiSessionItem.isOccupyType = true
        aiSessionItem.content = "占位，解决特定情况下，页面自动滚动跳动问题"
        updateInterviewContent(aiSessionItem)
    }

    fun getLastMsg(): AiSessionItem? {
        if (ListUtil.isEmpty(mBeans)) {
            return null
        }

        //d倒序遍历，找到最近一个不为空而且content有值的，且不是占位sessionItem
        for (item in mBeans.asReversed()) {
            if (item != null && !item.isOccupyType && !TextUtils.isEmpty(item.content)) {
                return item
            }
        }
        return null
    }


    /**
     * 是否是第一次请求最后一个职位卡片
     */
    fun isFirstLoadLastItem(goalIndex: Int): Boolean {
        if (!isFirstSessionPositionItem) {
            return false
        }
        var lastPositionItemItemIndex = 0
        //判断是否是最后一个职位卡片
        run breaking@{
            mBeans.reversed().forEachIndexed { index, aiSessionItem ->
                if (aiSessionItem.isPositionItemType) {
                    lastPositionItemItemIndex = index
                    return@breaking
                }
            }
        }

        return isFirstSessionPositionItem && (goalIndex == lastPositionItemItemIndex)
    }

    override fun getItemViewType(position: Int): Int {
        LogUtils.i("AiInterviewAdapter", "getItemViewType position = $position")

        if (position < 0){
            return R.layout.item_ai_interview_system
        }

        val aiSessionItem = mBeans[position]
        //1 系统消息类型
        if (aiSessionItem.isPureSystemOtherPosition) {
            return R.layout.item_ai_interview_system
        }

        //2 职位卡片消息
        if (aiSessionItem.isPositionItemType) {
            return R.layout.item_ai_intiview_position_out
        }

        //3 用户类型消息 - 位置-右边位置
        if (aiSessionItem.isUserType) {
            return R.layout.item_ai_interview_user
        }

        //4 水平方向广告
        if (aiSessionItem.isHorizontalAds) {
            return R.layout.item_ai_interview_horizontal_ad
        }

        //5 垂直方向广告
        if (aiSessionItem.isVerticalAds) {
            return R.layout.item_ai_interview_vertical_ad
        }

        //6 占位
        if (aiSessionItem.isOccupyType) {
            return R.layout.layout_space
        }

        return R.layout.item_ai_interview_system
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): RecyclerView.ViewHolder {

        val rootView = LayoutInflater.from(mContext).inflate(viewType, parent, false)

        //1 系统消息Holder创建
        if (viewType == R.layout.item_ai_interview_system) {
            return AiInterviewSystemHolder(rootView)
        }

        //2 职位卡片消息
        if (viewType == R.layout.item_ai_intiview_position_out) {
            return AiInterviewPositionHolder(rootView, this)
        }

        //3 用户类型消息
        if (viewType == R.layout.item_ai_interview_user) {
            return AiInterviewUserHolder(rootView)
        }

        //4 水平方向广告
        if (viewType == R.layout.item_ai_interview_horizontal_ad) {
            return AiInterviewHorizontalAdHolder(rootView)
        }

        //5 垂直方向广告
        if (viewType == R.layout.item_ai_interview_vertical_ad) {
            return AiInterviewVerticalAdHolder(rootView)
        }

        //6 站位类型
        if (viewType == R.layout.layout_space) {
            return AiOccupyHolder(rootView)
        }

        return super.onCreateViewHolder(parent, viewType)
    }

    override fun onBindViewHolder(holder: RecyclerView.ViewHolder, position: Int) {
        if (position < 0) {
            return
        }

        if (holder is AiInterviewUserHolder) {
            holder.onBindViewHolder(
                holder.itemView,
                getItem(position) as AiSessionItem,
                position
            )
            return
        }
        if (holder is AiInterviewSystemHolder) {
            holder.onBindViewHolder(
                holder.itemView,
                getItem(position) as AiSessionItem,
                position
            )
            return
        }
        if (holder is AiInterviewPositionHolder) {
            holder.onBindViewHolder(
                holder.itemView,
                getItem(position) as AiSessionItem,
                position
            )
            return
        }

        if (holder is AiInterviewHorizontalAdHolder){
            holder.onBindViewHolder(
                holder.itemView,
                (getItem(position) as AiSessionItem).itemRelateMsgList,
                position
            )
            return
        }

        if (holder is AiInterviewVerticalAdHolder){
            holder.onBindViewHolder(
                holder.itemView,
                (getItem(position) as AiSessionItem).itemRelateMsgList,
                position
            )
            return
        }

        if (holder is AiOccupyHolder) {
            holder.onBindViewHolder(
                holder.itemView,
                getItem(position) as AiSessionItem,
                position
            )
            return
        }

        super.onBindViewHolder(holder, position)
    }

    inner class AiInterviewPositionHolder(rootView: View?, aiInterviewAdapter: AIInterviewAdapter) :
        BaseViewHolder<AiSessionItem>(rootView) {

        var aiInterviewPositionItem: AiInterviewPositionItem
        private var aiInterviewAdapter: AIInterviewAdapter

        init {
            aiInterviewPositionItem = itemView as AiInterviewPositionItem
            this.aiInterviewAdapter = aiInterviewAdapter
        }

        override fun onBindViewHolder(
            view: View,
            aiSessionItem: AiSessionItem,
            position: Int
        ) {
            aiInterviewPositionItem.aiInterviewAdapter = this@AIInterviewAdapter
            aiInterviewPositionItem.update(aiSessionItem, position)
            //点击开始面试按钮
            aiInterviewPositionItem.setCallback {
                val originalAiSessionItem = aiSessionItem.clone()
                //1 隐藏开始AI 面试按钮
                aiSessionItem.sessionState = IAiInterviewSessionState.START_INTERVIEW
                aiInterviewAdapter.notifyItemChanged(position)

                //2 执行接口回调，生成AI内容占位item（先执行loading，等AI模型返回内容时，替换为对应内容）
                startInterviewListener?.onCallBack(originalAiSessionItem)
            }
        }
    }

    inner class AiInterviewUserHolder(rootView: View) : BaseAiInterviewHolder(rootView) {

        override fun onBindViewHolder(
            view: View,
            aiSessionItem: AiSessionItem,
            position: Int
        ) {
            super.onBindViewHolder(view, aiSessionItem, position)
        }
    }

    inner class AiInterviewSystemHolder(rootView: View) : BaseAiInterviewHolder(rootView) {

        override fun onBindViewHolder(
            view: View,
            aiSessionItem: AiSessionItem,
            position: Int
        ) {
            super.onBindViewHolder(view, aiSessionItem, position)
        }
    }

    inner open class BaseAiInterviewHolder(rootView: View) :
        BaseViewHolder<AiSessionItem>(rootView) {

        private val tvTime = rootView.findViewById<TextView>(R.id.chat_item_date)
        private val imageSendFail = rootView.findViewById<ImageView>(R.id.send_fail)
        private val imageAvatar = rootView.findViewById<CircleImageView>(R.id.chat_item_avatar)
        private val frameItemMsg = rootView.findViewById<FrameLayout>(R.id.chat_item_msg)
        private val imageLoading = rootView.findViewById<ImageView>(R.id.image_ai_loading)
        private val tvMsgTxt = rootView.findViewById<EmojiconTextView>(R.id.chat_item_msg_text)

        override fun onBindViewHolder(
            view: View,
            aiSessionItem: AiSessionItem,
            position: Int
        ) {

            view.visibility = View.VISIBLE
            if (aiSessionItem.createTime > 0) {
                tvTime.visibility = View.VISIBLE
                tvTime.text = aiSessionItem.showTimeTxt
            } else {
                tvTime.visibility = View.GONE
            }

            //0 处理消息状态
            handleMessageState(aiSessionItem, position)

            val frameItemLayoutParams = frameItemMsg.layoutParams
            //1 判断是否是用户消息
            if (aiSessionItem.isUserType) {
                ImageLoadUtils.with(mContext)
                    .load<String>(SPUtils.getUserPhoto())
                    .error(R.drawable.icon_login_head)
                    .placeholder(R.drawable.icon_login_head)
                    .displayOption(GlideOptionHelper.getHeaderOption(mContext))
                    .signature(cacheSignWithIn1Day())
                    .into(imageAvatar)
            }

            imageLoading.visibility = View.GONE
            //2 loading 中
            if (aiSessionItem.isLoading) {
                imageLoading.visibility = View.VISIBLE
                tvMsgTxt.visibility = View.GONE
                //气泡容器填充满（AI 生成内容还未出来之前）
                frameItemLayoutParams.width = LinearLayout.LayoutParams.MATCH_PARENT
                frameItemLayoutParams.height = LinearLayout.LayoutParams.WRAP_CONTENT
                frameItemMsg.layoutParams = frameItemLayoutParams
                ImageLoadUtils
                    .with(mContext)
                    .load(if (aiSessionItem.isUserType) R.drawable.ai_loading_while else R.drawable.ai_loading)
                    .asGif()
                    .into(imageLoading)
                return
            }

            //3 文本展示
            frameItemLayoutParams.width = LinearLayout.LayoutParams.WRAP_CONTENT
            frameItemLayoutParams.height = LinearLayout.LayoutParams.WRAP_CONTENT
            frameItemMsg.layoutParams = frameItemLayoutParams
            tvMsgTxt.visibility = View.VISIBLE
            tvMsgTxt.text = aiSessionItem.content
        }


        /**
         * 处理消息的状态
         */
        private fun handleMessageState(aiSessionItem: AiSessionItem, position: Int) {
            if (imageSendFail == null) {
                return
            }

            //loading 过程中，不显示消息状态
            if (aiSessionItem.isLoading) {
                imageSendFail.visibility = View.GONE
                return
            }

            if (aiSessionItem.isFail) {
                imageSendFail.visibility = View.VISIBLE
                imageSendFail.setImageResource(
                    if (aiSessionItem.isLastMessage)
                        R.drawable.pi_ai_im_shuaxin_40 else R.drawable.ic_chat_send_fail
                )

                if (aiSessionItem.isLastMessage) {
                    imageSendFail.setOnClickListener {
                        //1 重新发送语音消息
                        aiSessionItem.isLoading = true
                        notifyItemChanged(position)
                        if (aiSessionItem.isVoiceType) {
                            convertVoiceAsrListener?.onCallBack(aiSessionItem)
                            return@setOnClickListener
                        }

                        //2 重新发送文本
                        chatTextListener?.onCallBack(aiSessionItem)
                    }
                } else {
                    imageSendFail.setOnClickListener(null)
                }
                return
            }

            imageSendFail.visibility = View.GONE
            imageSendFail.setOnClickListener(null)
        }
    }

    /**
     * 横向广告位 外部holder
     */
    inner class AiInterviewHorizontalAdHolder(itemView: View) :
        BaseViewHolder<List<AIRelateMsgItem>>(itemView) {

        private val recyclerView: RecyclerView
        private val itemHorizontalAdapter: ItemHorizontalAdapter

        init {
            recyclerView = itemView as RecyclerView
            val layoutManager = LinearLayoutManager(mContext, LinearLayoutManager.HORIZONTAL, false)
            recyclerView.layoutManager = layoutManager

            itemHorizontalAdapter = ItemHorizontalAdapter(itemView.context)
            recyclerView.adapter = itemHorizontalAdapter

            //recyclerView.isNestedScrollingEnabled = false
            recyclerView.setHasFixedSize(true)
            recyclerView.setLayerType(View.LAYER_TYPE_HARDWARE, null)
            val snapHelper = LinearSnapHelper()
            snapHelper.attachToRecyclerView(recyclerView)

        }

        override fun onBindViewHolder(
            view: View,
            relateMsgItemList: List<AIRelateMsgItem>,
            position: Int
        ) {
            itemHorizontalAdapter.setData(true, relateMsgItemList)
        }

        /**
         * 横向广告位 适配器
         */
        inner class ItemHorizontalAdapter(context: Context?) :
            BaseVHAdapter<AIRelateMsgItem>(context) {

            override fun onCreateViewHolder(
                parent: ViewGroup,
                viewType: Int
            ): RecyclerView.ViewHolder {
                val itemView =
                    LayoutInflater.from(parent.context)
                        .inflate(R.layout.item_ai_interview_horizontal_ad_sub, parent, false)
                return AdViewHolder(itemView)
            }

            override fun getItemCount(): Int {
                return mBeans?.size ?: 0
            }

            override fun onBindViewHolder(holder: RecyclerView.ViewHolder, position: Int) {
                if (holder is AdViewHolder) {
                    holder.bindViewHolder(mBeans?.get(position), position)
                }
            }

            inner class AdViewHolder(itemView: View) : RecyclerView.ViewHolder(itemView) {
                private val imageBusinessIcon =
                    itemView.findViewById<ImageView>(R.id.image_business_icon)
                private val tvTitle = itemView.findViewById<TextView>(R.id.tv_title)
                private val tvDes = itemView.findViewById<TextView>(R.id.tv_des)

                fun bindViewHolder(aiRelateMsgItem: AIRelateMsgItem?, position: Int) {
                    tvTitle.text = aiRelateMsgItem?.title
                    tvDes.text = aiRelateMsgItem?.tip
                    ImageLoadUtils.with(mContext)
                        .load<String>(aiRelateMsgItem?.imgUrl)
                        //.displayOption(GlideOptionHelper.getHeaderOption(mContext))
                        //.signature(cacheSignWithIn1Day())
                        .into(imageBusinessIcon)
                    itemView.setOnClickListener {
                        IntentUtils.startLagouWebViewActivity(mContext, aiRelateMsgItem?.linkUrl)
                    }
                }
            }
        }
    }

    /**
     * 垂直广告位，外部holder
     */
    class AiInterviewVerticalAdHolder(itemView: View) :
        BaseViewHolder<List<AIRelateMsgItem>>(itemView) {

        private val verticalContainer: LinearLayout

        init {
            verticalContainer = itemView as LinearLayout
        }

        override fun onBindViewHolder(
            view: View,
            relateMsgItemList: List<AIRelateMsgItem>,
            position: Int
        ) {
            verticalContainer.removeAllViews()
            relateMsgItemList.forEachIndexed { index, aiRelateMsgItem ->
                val view = LayoutInflater.from(mContext).inflate(R.layout.item_ai_interview_vertical_ad_sub, null, false)
                val textView = view.findViewById<TextView>(R.id.tv_content)
                textView.text = aiRelateMsgItem.tip
                val params = LinearLayout.LayoutParams(LinearLayout.LayoutParams.WRAP_CONTENT,
                    LinearLayout.LayoutParams.WRAP_CONTENT)
                if (index != 0){
                    params.topMargin = ResourceUtils.getDimenSize(R.dimen.dp_15)
                }
                verticalContainer.addView(view, params)
                view.setOnClickListener {
                    IntentUtils.startLagouWebViewActivity(mContext, aiRelateMsgItem.linkUrl)
                }
            }
        }
    }

    inner class AiOccupyHolder(rootView: View) : BaseViewHolder<AiSessionItem>(rootView) {
        override fun onBindViewHolder(view: View, obj: AiSessionItem, position: Int) {
        }
    }
}


