package com.alpha.lagouapk.ui.interview.ai.widget

import android.animation.Animator
import android.animation.AnimatorListenerAdapter
import android.animation.ObjectAnimator
import android.view.View
import android.widget.LinearLayout
import androidx.recyclerview.widget.DefaultItemAnimator
import androidx.recyclerview.widget.RecyclerView
import com.alpha.lagouapk.R
import com.alpha.lagouapk.ui.interview.ai.adapter.AIInterviewAdapter
import com.lagou.commonlibs.utils.ResourceUtils


class GuideItemAnim: DefaultItemAnimator() {
    private var mPositionCardViewHolder: AIInterviewAdapter.AiInterviewPositionHolder? = null
    private val mInitTopMargin = ResourceUtils.getDimenSize(R.dimen.dp_14)
    private val mMaxTopMargin = ResourceUtils.getDimenSize(R.dimen.dp_65)
    private val mLoadingMaxTopMargin = ResourceUtils.getDimenSize(R.dimen.dp_65)
    private var mCurAnimateValue:Float = 0f
    private var mGrow = true
    private var mFirstAdd = true
    private var onAnimEnd:(()->Unit)? = null
    companion object{
        const val MOVE_ITEM_DURATION = 400L
        const val LOADING_MOVE_DURATION = 250L
    }
    override fun animateRemove(holder: RecyclerView.ViewHolder?): Boolean {
        val itemViewHeight = holder?.itemView?.height?.toFloat() ?: 0f
        val animator: ObjectAnimator = ObjectAnimator.ofFloat(holder?.itemView, "translationY", 0f,itemViewHeight)
        animator.duration = MOVE_ITEM_DURATION
        animator.addUpdateListener {
            updateMove(it.animatedValue as Float,itemViewHeight)
        }
        animator.addListener(object : AnimatorListenerAdapter() {
            override fun onAnimationEnd(animation: Animator) {
                dispatchRemoveFinished(holder)
            }

            override fun onAnimationStart(animation: Animator) {
                super.onAnimationStart(animation)
                dispatchRemoveStarting(holder)
            }
        })
        animator.start()
        return true
    }
    private fun updateMove(animatedValue: Float,height:Float) {
        val startInterview = mPositionCardViewHolder?.aiInterviewPositionItem?.tvStartInterview
        startInterview?.let {
            val params:LinearLayout.LayoutParams = startInterview.layoutParams as LinearLayout.LayoutParams
            var topMargin = 0
            if (mGrow){
                topMargin = mInitTopMargin + (animatedValue * 0.7).toInt()
            }else{
                topMargin = mMaxTopMargin - ((mMaxTopMargin - mInitTopMargin) / (height - mCurAnimateValue) * (animatedValue - mCurAnimateValue)).toInt()
            }
            if (topMargin >= mMaxTopMargin && mGrow) {
                mGrow = false
                mCurAnimateValue = animatedValue
            }
            params.topMargin = topMargin
            it.layoutParams = params

            startInterview.alpha = 1 - animatedValue / height
            if (animatedValue == height){
                startInterview.visibility = View.GONE
            }
        }
    }


    override fun onRemoveStarting(viewHolder: RecyclerView.ViewHolder?) {
        super.onRemoveStarting(viewHolder)
        viewHolder?.itemView?.visibility = View.GONE
        mGrow = true
        mCurAnimateValue = 0f
    }


    override fun onMoveStarting(holder: RecyclerView.ViewHolder?) {
        super.onMoveStarting(holder)
       if (holder is AIInterviewAdapter.AiInterviewPositionHolder){
           this.mPositionCardViewHolder = holder
       }
    }

    override fun onAddStarting(holder: RecyclerView.ViewHolder?) {
        super.onAddStarting(holder)
        if (mFirstAdd && holder is AIInterviewAdapter.AiInterviewSystemHolder){
            mFirstAdd = false
            updateLoadingMove(holder)
        }
    }

    /**
     * loading 动效
     */
    private fun updateLoadingMove(holder: AIInterviewAdapter.AiInterviewSystemHolder) {
        val animator: ObjectAnimator = ObjectAnimator.ofFloat(holder.itemView, "translationY", mLoadingMaxTopMargin.toFloat(),0f)
        animator.duration = LOADING_MOVE_DURATION
        animator.addUpdateListener {
            val alpha = 1 - (it.animatedValue as Float) / mLoadingMaxTopMargin
            holder.itemView.alpha = alpha

        }
        animator.addListener(object : AnimatorListenerAdapter() {
            override fun onAnimationEnd(animation: Animator) {
                holder.itemView.alpha = 1f
                onAnimEnd?.invoke()
            }

            override fun onAnimationStart(animation: Animator) {
                super.onAnimationStart(animation)
                holder.itemView.alpha = 0f
            }
        })
        animator.start()
    }


    override fun getMoveDuration(): Long {
        return MOVE_ITEM_DURATION
    }

    fun setOnAnimEnd(onAnimEnd:(()->Unit)?){
        this.onAnimEnd = onAnimEnd
    }
}