package com.alpha.lagouapk.ui.interview.ai.widget

import android.animation.Animator
import android.animation.AnimatorListenerAdapter
import android.animation.AnimatorSet
import android.animation.ObjectAnimator
import android.content.Context
import android.graphics.Canvas
import android.graphics.Color
import android.graphics.LinearGradient
import android.graphics.Paint
import android.graphics.RectF
import android.graphics.Shader
import android.util.AttributeSet
import android.view.View
import com.alpha.lagouapk.R
import com.lagou.commonlibs.utils.ResourceUtils
import kotlin.random.Random

/**
 * Time：2024/8/2 14:31:04
 * Author：wangjunwei
 * Description：纯播放语音面试页面，录音交互自定义View
 *
 */
class PurePlayVoiceWaveView @JvmOverloads constructor(
    context: Context,
    attrs: AttributeSet? = null,
    style: Int = 0
) : View(context, attrs, style) {

    private val mWaveItemWidth = ResourceUtils.getDimenSize(R.dimen.dp_7)//语音条的宽度
    private val mWaveItemMinHeight = ResourceUtils.getDimenSize(R.dimen.dp_7)//语音条的最小高度
    private val mWaveItemMaxHeight = ResourceUtils.getDimenSize(R.dimen.dp_22)//语音条最大高度
    private val mWaveItemGap = ResourceUtils.getDimenSize(R.dimen.dp_7)//语音条的间隔
    private val mWaveItemCorner = ResourceUtils.getDimenSize(R.dimen.dp_4)//语音条的圆角
    private val mWaveItemCount = 3 //语音条的数量

    private val mWavePaint: Paint = Paint(Paint.ANTI_ALIAS_FLAG)//绘制语音波纹画笔
    private val waveStartColor = Color.parseColor("#19BCFF")
    private val waveEndColor = Color.parseColor("#0063FF")

    /**
     * 绘制背景-圆角矩形
     */
    private val backgroundRoundHeight = ResourceUtils.getDimenSize(R.dimen.dp_44)
    private val backgroundRoundCorner = ResourceUtils.getDimenSize(R.dimen.dp_30)
    private var backgroundPaint: Paint = Paint(Paint.ANTI_ALIAS_FLAG)

    /**
     * 录音交互View最大高最小高
     */
    private val dp_44 = ResourceUtils.getDimenSize(R.dimen.dp_44)

    private val scaleSetAnimation: AnimatorSet
    private val reverseScaleSetAnimation: AnimatorSet

    /**
     * 语音波纹浮动系数，从0-1
     */
     var amplitude: Float = 0f
        set(value) {
            if (value == field){
                return
            }

            if (value > 1){
                field = 1f
            }else {
                field = value
            }
            invalidate()
        }

    init {
        mWavePaint.strokeWidth = mWaveItemWidth.toFloat()
        mWavePaint.style = Paint.Style.FILL

        backgroundPaint.color = Color.parseColor("#FFFFFF")
        backgroundPaint.style = Paint.Style.FILL

        //设置放缩起点，为左上角
        pivotX = 0f
        pivotY = ResourceUtils.getDimenSize(R.dimen.dp_44).toFloat()
        val scaleXAnimation = ObjectAnimator.ofFloat(this, "scaleX", 0f, 1f)
        val scaleYAnimation = ObjectAnimator.ofFloat(this, "scaleY", 0f, 1f)
        scaleSetAnimation = AnimatorSet()
        scaleSetAnimation.playTogether(scaleXAnimation, scaleYAnimation)
        scaleSetAnimation.duration = 800
        scaleSetAnimation.addListener(object : AnimatorListenerAdapter() {
            override fun onAnimationStart(animation: Animator?) {
                super.onAnimationStart(animation)
                visibility = VISIBLE
            }
            override fun onAnimationEnd(animation: Animator?) {
                this@PurePlayVoiceWaveView.scaleX = 1.0f
                this@PurePlayVoiceWaveView.scaleY = 1.0f
            }
        })

        val reverseScaleXAnimation = ObjectAnimator.ofFloat(this, "scaleX", 1f, 0f)
        val reverseScaleYAnimation = ObjectAnimator.ofFloat(this, "scaleY", 1f, 0f)
        reverseScaleSetAnimation = AnimatorSet()
        reverseScaleSetAnimation.playTogether(reverseScaleXAnimation, reverseScaleYAnimation)
        reverseScaleSetAnimation.duration = 800
        reverseScaleSetAnimation.addListener(object : AnimatorListenerAdapter() {
            override fun onAnimationEnd(animation: Animator?) {
                this@PurePlayVoiceWaveView.scaleX = 0f
                this@PurePlayVoiceWaveView.scaleY = 0f
                visibility = GONE
            }
        })
    }

    /**
     * 左下角开始慢慢变大，直到达到整个View100%大小停止
     */
    fun startAnimation() {
        if (reverseScaleSetAnimation.isRunning) {
            reverseScaleSetAnimation.end()
        }
        if (scaleSetAnimation.isRunning) {
            return
        }
        scaleSetAnimation.start()
    }

    /**
     * 右上角开始慢慢变小，直到达到整个View0%大小停止
     */
    fun startReverseAnimation() {
        if (scaleSetAnimation.isRunning) {
            scaleSetAnimation.end()
        }
        if (reverseScaleSetAnimation.isRunning) {
            return
        }
        reverseScaleSetAnimation.start()
    }


    private fun getViewWidth(): Int {
        return mWaveItemWidth * mWaveItemCount + (mWaveItemCount - 1) * mWaveItemGap
    }

    override fun onMeasure(widthMeasureSpec: Int, heightMeasureSpec: Int) {
        super.onMeasure(widthMeasureSpec, heightMeasureSpec)
        val heightSpecSize = MeasureSpec.getSize(heightMeasureSpec)
        val heightSpecMode = MeasureSpec.getMode(heightMeasureSpec)

        val widthSpecSize = MeasureSpec.getSize(widthMeasureSpec)
        val realHeight = dp_44
        if (heightSpecMode == MeasureSpec.AT_MOST || heightSpecMode == MeasureSpec.UNSPECIFIED) {
            setMeasuredDimension(widthSpecSize, realHeight)
        } else {
            setMeasuredDimension(widthSpecSize, heightSpecSize.coerceAtLeast(realHeight))
        }
    }

    override fun onDraw(canvas: Canvas) {
        super.onDraw(canvas)
        drawBackgroundRound(canvas)
        drawWave(canvas)
    }

    /**
     * 绘制圆角矩形背景
     */
    private fun drawBackgroundRound(canvas: Canvas) {
        canvas.drawRoundRect(
            RectF(
                0f,
                (height - backgroundRoundHeight).toFloat(),
                width.toFloat(),
                height.toFloat()
            ), backgroundRoundCorner.toFloat(), backgroundRoundCorner.toFloat(), backgroundPaint
        )
    }

    /**
     * 绘制语音波纹
     */
    private fun drawWave(canvas: Canvas) {

        val gradient = LinearGradient(
            0f,
            0f,
            0f,
            height.toFloat(),
            waveStartColor,
            waveEndColor,
            Shader.TileMode.CLAMP
        )
        mWavePaint.shader = gradient

        val centerY = height / 2
        val offsetX = width / 2f - getViewWidth() / 2f
        for (i in 0.until(mWaveItemCount)) {
            //wave 高度变幻最大值  最大高减去最小高
            val indicatorHeight = mWaveItemMinHeight + Random.nextInt(
                0,
                mWaveItemMaxHeight - mWaveItemMinHeight
            ) * amplitude
            val startX = i * (mWaveItemWidth + mWaveItemGap).toFloat() + offsetX
            canvas.drawRoundRect(
                RectF(
                    startX,
                    centerY - indicatorHeight / 2f,
                    startX + mWaveItemWidth,
                    centerY + indicatorHeight / 2f
                ), mWaveItemCorner.toFloat(), mWaveItemCorner.toFloat(), mWavePaint
            )
        }
    }
}