package com.alpha.lagouapk.ui.interview.ai.widget

import android.animation.ObjectAnimator
import android.app.Activity
import android.content.Context
import android.os.Handler
import android.os.Looper
import android.text.TextUtils
import android.util.AttributeSet
import android.view.animation.CycleInterpolator
import android.widget.FrameLayout
import android.widget.ImageView
import android.widget.RelativeLayout
import com.alpha.lagouapk.R
import com.alpha.lagouapk.bean.aiinterview.AiContentType
import com.alpha.lagouapk.bean.aiinterview.AiInterviewSessionListRes
import com.alpha.lagouapk.observer.EventTags
import com.alpha.lagouapk.manager.AiManager
import com.alpha.lagouapk.manager.net.AiNetManager
import com.lagou.baselib.analytic.AnalyticHelper
import com.lagou.baselib.privacy.KEY_RECORD_CHAT
import com.lagou.baselib.privacy.checkPrivacy
import com.lagou.chat.chatlibrary.manager.MediaManager
import com.lagou.chat.chatlibrary.manager.RecordManager
import com.lagou.commonlibs.engine.ThreadUtil
import com.lagou.commonlibs.event.OkBus
import com.lagou.commonlibs.inter.OnSingleMethodListener
import com.lagou.commonlibs.privacy.AdapterPrivacyRightListener
import com.lagou.commonlibs.privacy.PrivacyPermissionKey
import com.lagou.commonlibs.utils.ResourceUtils

/**
 * Time：2024/8/5 14:09:50
 * Author：wangjunwei
 * Description：
 * 纯语音聊天界面，交互自定义View
 */
class PureVoiceView @JvmOverloads constructor(
    context: Context,
    attrs: AttributeSet? = null,
    style: Int = 0
) : FrameLayout(context, attrs, style) {
    private val pureVoiceWaveView: PureVoiceWaveView
    private val relLoading: RelativeLayout
    private val relOperateArea: RelativeLayout
    private val imageClose: ImageView
    private var imagePause: ImageView
    private var imageResume: ImageView
    private var imageOpen: ImageView

    private var mOnSendListener: ((AiContentType, String?) -> Unit)? = null
    private var recordingListener: OnSingleMethodListener<Boolean>? = null

    /**
     * 开始面试回调
     */
    private var startInterviewListener: ((Boolean) -> Unit)? = null

    /**
     * 暂停和恢复回调
     */
    private var pauseResumeListener: ((Boolean)->Unit)? = null

    private val recordManager: RecordManager = RecordManager.getInstance()
    private var startRecordTime: Long = 0
    private val updateWaveHandler: Handler = Handler(Looper.getMainLooper())

    /**
     * 抖动动画配置-接听Ai电话
     */
    private lateinit var imageShakeAnimation: ObjectAnimator
    private val dp_30 = ResourceUtils.getDimenSize(R.dimen.dp_30)

    /**init
     * 默认正在loading中
     */
    private var currentState: AIVoiceState = AIVoiceState.LOADING

    /**
     *  初始进来页面，最新会话的数据
     */
    private var initAiInterviewSession: AiInterviewSessionListRes? = null


    /**
     * Ai 语音面试各个状态
     */
    enum class AIVoiceState {
        LOADING, //loading 中
        WAIT_LISTEN, //待接听
        RECORDING, //录音中
        RECORD_PAUSE, //录音暂停
        SENDING, //发送中
        AI_SPEAK, //AI 说话中
        AI_SPEAK_PAUSE, //AI说话暂停
        SESSION_END //会话结束
    }

    companion object {
        //单次语音最大时长 单位ms
        val maxTotalDuration = AiManager.getAiChatConfigInfo().totalDuration * 1000

        //剩余多长时间开始倒计时单位ms
        val residueDuration = AiManager.getAiChatConfigInfo().residueDuration * 1000

        //没有说话超过s，则自动发送语音
        val asrTriggerInterval = AiManager.getAiChatConfigInfo().asrTriggerInterval * 1000
    }

    init {
        inflate(context, R.layout.layout_pure_voice_view, this)

        relOperateArea = findViewById(R.id.rel_operate_area)
        pureVoiceWaveView = findViewById(R.id.pure_voice_wave_view)
        relLoading = findViewById(R.id.rel_loading)
        imageClose = findViewById(R.id.image_close)
        imagePause = findViewById(R.id.image_pause)
        imageResume = findViewById(R.id.image_resume)
        imageOpen = findViewById(R.id.image_open)
        setListener()
    }

    /**
     * 1 会话结束状态的样式
     */
    private fun fillSessionFinishStyle() {
        currentState = AIVoiceState.SESSION_END

        //1 音频录制组件处理
        pureVoiceWaveView.changeState(PureVoiceWaveView.State.SESSION_FINISH)
        pureVoiceWaveView.visibility = VISIBLE
        relLoading.visibility = GONE

        imagePause.visibility = GONE
        imageResume.visibility = GONE
        imageOpen.visibility = GONE
        handleCloseStyle()
    }

    /**
     * 2 当前未授予音频录制权限或者来自push-右侧显示接听按钮
     */
    private fun fillUnRightOrPushStyle() {
        currentState = AIVoiceState.WAIT_LISTEN
        pureVoiceWaveView.changeState(PureVoiceWaveView.State.WAIT_LISTEN)

        pureVoiceWaveView.visibility = INVISIBLE
        relLoading.visibility = GONE

        imagePause.visibility = GONE
        imageResume.visibility = GONE
        imageOpen.visibility = VISIBLE

        //2.1 对于开启AI面试按钮，配置上下跳动的动画
        imageShakeAnimation = ObjectAnimator.ofFloat(imageOpen, "translationY", 0f, dp_30.toFloat())
        imageShakeAnimation.duration = 1000
        imageShakeAnimation.interpolator = CycleInterpolator(3F)
        // 执行上下抖动动画
        if (!imageShakeAnimation.isRunning) {
            imageShakeAnimation.start()
        }
        //2.2 结束按钮样式处理
        handleCloseStyle()
    }

    fun setData(aiInterviewSession: AiInterviewSessionListRes?){
        this.initAiInterviewSession = aiInterviewSession
    }

    /**
     * 3 loading中
     */
    private fun fillLoadingStyle() {
        currentState = AIVoiceState.LOADING
        pureVoiceWaveView.changeState(PureVoiceWaveView.State.SENDING)

        pureVoiceWaveView.visibility = INVISIBLE
        relLoading.visibility = VISIBLE

        imagePause.visibility = GONE
        imageResume.visibility = GONE
        imageOpen.visibility = GONE
        handleCloseStyle()
    }

    /**
     * 4 已经授予录音权限&&也不是来自push-接收录音中
     */
    private fun fillRecordStyle() {
        currentState = AIVoiceState.RECORDING
        pureVoiceWaveView.changeState(PureVoiceWaveView.State.UN_SPECK)

        pureVoiceWaveView.visibility = VISIBLE
        relLoading.visibility = GONE

        imagePause.visibility = VISIBLE
        imageResume.visibility = GONE

        imageOpen.visibility = GONE
        imageOpen.isSelected = false
        handleCloseStyle()
    }

    /**
     * 4.2 录音暂停中
     */
    private fun fillRecordPauseStyle() {
        currentState = AIVoiceState.RECORD_PAUSE
        pureVoiceWaveView.changeState(PureVoiceWaveView.State.RECORD_PAUSE)

        imagePause.visibility = GONE
        imageResume.visibility = VISIBLE

        imageOpen.visibility = GONE
        imageOpen.isSelected = false
        handleCloseStyle()
    }

    /**
     * 5 AI 正在说话中
     */
    private fun fillAiSpeckStyle() {
        currentState = AIVoiceState.AI_SPEAK

        pureVoiceWaveView.changeState(PureVoiceWaveView.State.AI_SPECK)

        relLoading.visibility = GONE
        pureVoiceWaveView.visibility = INVISIBLE

        imagePause.visibility = VISIBLE
        imageResume.visibility = GONE
        handleCloseStyle()
    }

    /**
     * 5.2 AI 说话暂停中
     */
    private fun fillAiSpeckPauseStyle() {
        currentState = AIVoiceState.AI_SPEAK_PAUSE
        pureVoiceWaveView.changeState(PureVoiceWaveView.State.AI_SPECK_PAUSE)

        relLoading.visibility = GONE
        pureVoiceWaveView.visibility = INVISIBLE

        imagePause.visibility = GONE
        imageResume.visibility = VISIBLE
        handleCloseStyle()
    }

    /**
     * 变更状态
     */
    fun changeState(aiVoiceState: AIVoiceState) {
        currentState = aiVoiceState
        when (currentState) {
            AIVoiceState.LOADING -> fillLoadingStyle()
            AIVoiceState.WAIT_LISTEN -> fillUnRightOrPushStyle()
            AIVoiceState.RECORDING -> fillRecordStyle()
            AIVoiceState.RECORD_PAUSE -> fillRecordPauseStyle()
            AIVoiceState.SENDING -> fillLoadingStyle()
            AIVoiceState.AI_SPEAK -> fillAiSpeckStyle()
            AIVoiceState.AI_SPEAK_PAUSE -> fillAiSpeckPauseStyle()
            AIVoiceState.SESSION_END -> fillSessionFinishStyle()
        }
    }

    private fun setListener() {
        imageClose.setOnClickListener {
            AnalyticHelper.getInstance().sendClickInfo("z1ujt")

            //关闭当前页面
            (context as Activity).finish()
            //页面关闭，刷新下Ai面试页面的列表
            OkBus.getInstance().onEvent(EventTags.AI_INTERVIEW_FLUSH_SESSION_LIST)
        }

        imagePause.setOnClickListener {
            AnalyticHelper.getInstance().sendClickInfo("x1ujr")

            // 暂停：录音或者是Ai机器人说话
            if (currentState == AIVoiceState.RECORDING){
                stopRecord(false)
                changeState(AIVoiceState.RECORD_PAUSE)

                pauseResumeListener?.invoke(false)

                return@setOnClickListener
            }

            if (currentState == AIVoiceState.AI_SPEAK){
                MediaManager.pause()
                changeState(AIVoiceState.AI_SPEAK_PAUSE)

                pauseResumeListener?.invoke(false)

                return@setOnClickListener
            }
        }
        imageResume.setOnClickListener {
            AnalyticHelper.getInstance().sendClickInfo("y1ujs")

            // 恢复：录音或者是Ai机器人说话
            if (currentState == AIVoiceState.RECORD_PAUSE){
                realStartRecord()
                changeState(AIVoiceState.RECORDING)

                pauseResumeListener?.invoke(true)

                return@setOnClickListener
            }
            if (currentState == AIVoiceState.AI_SPEAK_PAUSE){
                MediaManager.resume()
                changeState(AIVoiceState.AI_SPEAK)

                pauseResumeListener?.invoke(true)

                return@setOnClickListener
            }
        }
        imageOpen.setOnClickListener {

            //1 会话处于未创建或者未开始面试状态-发起开始面试（统一转移到权限授予之后，或者判断权限授予之后）
//            if(initAiInterviewSession?.isNotCreateSession == true
//                || initAiInterviewSession?.isInitState == true) {
//                //如果还没有开始面试，就发起开始面试
//                initAiInterviewSession = null
//
//                startInterviewListener?.invoke(true)
//                return@setOnClickListener
//            }

            //2 处理从push 或者AI面试未授权语音权限的，已经开始面试&授予录音权限
            realStartRecord()
        }
    }

    /**
     * 开启录音-包含权限请求
     */
    fun realStartRecord() {
        checkPrivacy(
            (context as Activity), listOf(KEY_RECORD_CHAT),
            object : AdapterPrivacyRightListener() {
                override fun onAgreeRight() {
                    //1 会话处于未创建或者未开始面试状态-发起开始面试
                    if(initAiInterviewSession?.isNotCreateSession == true
                        || initAiInterviewSession?.isInitState == true) {
                        //如果还没有开始面试，就发起开始面试
                        //initAiInterviewSession = null

                        startInterviewListener?.invoke(true)
                        return
                    }

                    recordManager.prepareRecord(
                        object : RecordManager.RecordStateChangeListener {
                            override fun wellPrepared() {
                                changeState(AIVoiceState.RECORDING)
                                startRecord()
                                recordingListener?.onCallBack(true)
                            }

                            override fun onUserSpeak() {
                                //处于倒计时，不切换到正在听的状态
                                if (pureVoiceWaveView.getCurrentState() == PureVoiceWaveView.State.COUNT_DOWN_SPECK){
                                    return
                                }
                                pureVoiceWaveView.changeState(PureVoiceWaveView.State.ON_SPECK)
                            }

                            override fun failPrepared() {
                                recordManager.cancel()
                            }

                            override fun maxDurationStopRecord() {
                                ThreadUtil.ui {
                                    //切换成发送中
                                    changeState(AIVoiceState.SENDING)
                                    stopRecord(false)

                                    //等待文件写完成，否则肯能导致 java.net.ProtocolException: expected 6903 bytes but received
                                    ThreadUtil.uiDelay({
                                        sendVoice()
                                    }, 100)
                                }
                            }

                            override fun recordTimeStatistics(recordTimeDuration: Long) {
                                //剩余可录制时长，小于等于 residueDuration展示可录音的倒计时文案
                                val remainCanRecordDuration = maxTotalDuration - recordTimeDuration
                                if (remainCanRecordDuration in 0 until residueDuration + 1) {
                                    pureVoiceWaveView.countDownText =
                                        ("你还可以再说${remainCanRecordDuration / 1000}s")
                                } else {
                                    //pureVoiceWaveView.countDownText = "" // 正在听
                                }
                            }

                            override fun exceedSomeSeconds() {
                                ThreadUtil.ui {
                                    stopRecord(false)
                                    //切换成发送中
                                    changeState(AIVoiceState.SENDING)
                                    //等待文件写完成，否则肯能导致 java.net.ProtocolException: expected 6903 bytes but received
                                    ThreadUtil.uiDelay({
                                        sendVoice()
                                    }, 100)
                                }
                            }

                            override fun asrTriggerInterval() = asrTriggerInterval
                        },
                        maxTotalDuration,
                        AiNetManager.getRecordOutDir(context)
                    )

                    changeState(AIVoiceState.RECORDING)
                }

                override fun onDenied() {
                    super.onDenied()
                }

                override fun onParameterError() {
                    super.onParameterError()
                }
            },
            PrivacyPermissionKey.RECORD_AUDIO
        )
    }

    /**
     * 开始录音
     */
    private fun startRecord() {
        startRecordTime = System.currentTimeMillis()
        updateWaveHandler.post(updateWaveform)
    }

    /**
     * 结束录音
     * @param isCancelSend 是否取消发送
     */
    fun stopRecord(isCancelSend: Boolean) {
        updateWaveHandler.removeCallbacks(updateWaveform)
        if (isCancelSend) {
            recordManager.cancel()
        } else {
            recordManager.release()
        }
    }

    /**
     * 恢复和暂停录音、语音播报 - 页面退到后台和重新恢复
     */
    fun resumeRecord(){
        imageResume.callOnClick()
    }

    fun pauseRecord(){
        imagePause.callOnClick()
    }

    private val updateWaveform: Runnable = object : Runnable {
        override fun run() {
            pureVoiceWaveView.amplitude = recordManager.getVoiceRate(4)
            updateWaveHandler.postDelayed(this, 100)
        }
    }

    /**
     * 消息发送回调
     */
    fun setOnSendListener(callback: ((AiContentType, contentOrPath: String?) -> Unit)?) {
        this.mOnSendListener = callback
    }

    /**
     * 设置开始面试回调
     */
    fun setStartInterviewListener(startInterviewListener: ((Boolean) -> Unit)?){
        this.startInterviewListener = startInterviewListener
    }

    /**
     * 设置恢复和暂停回调
     * true 恢复 false 暂停
     */
    fun setPauseResumeListener(pauseResumeListener: ((Boolean) -> Unit)?){
        this.pauseResumeListener = pauseResumeListener
    }

    /**
     * 录音过程
     */
    fun setRecordingListener(recordingListener: OnSingleMethodListener<Boolean>) {
        this.recordingListener = recordingListener
    }

    /**
     * 发送录音
     */
    private fun sendVoice() {
        //val wavPath = getWavFormat(mRecordManager.curFilePath)
        if (TextUtils.isEmpty(recordManager.curFilePath)) {
            return
        }
        mOnSendListener?.invoke(AiContentType.VOICE, recordManager.curFilePath)
    }

    /**
     * 处理关闭按钮的样式
     */
    private fun handleCloseStyle() {
        //1 对于关闭按钮，根据最右边按钮显示和隐藏，设置按钮位置
        val imageCloseParams = imageClose.layoutParams as RelativeLayout.LayoutParams
        if (imagePause.visibility == GONE
            && imageResume.visibility == GONE
            && imageOpen.visibility == GONE) {
            imageCloseParams.addRule(RelativeLayout.CENTER_HORIZONTAL, RelativeLayout.TRUE)
        } else {
            imageCloseParams.removeRule(RelativeLayout.CENTER_HORIZONTAL)
        }
        imageClose.layoutParams = imageCloseParams
    }

    fun showLoading() {
        changeState(AIVoiceState.LOADING)
    }

    fun hideLoading() {
        relLoading.visibility = GONE
        pureVoiceWaveView.visibility = VISIBLE
    }
}