package com.alpha.lagouapk.ui.interview.ai.widget

import android.annotation.SuppressLint
import android.app.Activity
import android.content.Context
import android.os.Handler
import android.os.Looper
import android.text.InputFilter
import android.text.TextUtils
import android.util.AttributeSet
import android.view.View
import android.widget.FrameLayout
import android.widget.ImageView
import com.alpha.lagouapk.R
import com.alpha.lagouapk.bean.aiinterview.AiContentType
import com.alpha.lagouapk.manager.AiManager
import com.alpha.lagouapk.manager.net.AiNetManager
import com.common.afterTextChanged
import com.lagou.baselib.privacy.KEY_RECORD_CHAT
import com.lagou.baselib.privacy.checkPrivacy
import com.lagou.chat.chatlibrary.manager.RecordManager
import com.lagou.chat.chatlibrary.view.EmojiconEditText
import com.lagou.commonlibs.engine.ThreadUtil
import com.lagou.commonlibs.inter.OnSingleMethodListener
import com.lagou.commonlibs.privacy.AdapterPrivacyRightListener
import com.lagou.commonlibs.privacy.PrivacyPermissionKey
import com.lagou.commonlibs.utils.InputMethodUtil
import com.lagou.commonlibs.utils.LogUtils
import com.lagou.commonlibs.view.filter.EmojiFilter


@SuppressLint("ClickableViewAccessibility")
class VoiceView @JvmOverloads constructor(
    context: Context,
    attrs: AttributeSet? = null,
    style: Int = 0
) : FrameLayout(context, attrs, style) {
    private var mVoiceWaveView: VoiceWaveView
    private var mIvAction: ImageView
    private var mEditInput: EmojiconEditText
    private var mOnSendListener: ((AiContentType, String?) -> Unit)? = null
    private var recordingListener: OnSingleMethodListener<Boolean>? = null

    private val mRecordManager: RecordManager = RecordManager.getInstance()
    private var mStartRecordTime: Long = 0
    private val mUpdateWaveHandler: Handler = Handler(Looper.getMainLooper())

    companion object {
        //最短时间2s
        const val MIN_RECORD_TIME = 1000
        //单次语音最大时长 单位ms
        val maxTotalDuration = AiManager.getAiChatConfigInfo().totalDuration * 1000
        //剩余多长时间开始倒计时单位ms
        val residueDuration = AiManager.getAiChatConfigInfo().residueDuration * 1000
    }

    enum class IconState {
        NORMAL,//默认状态
        INPUT_EMPTY,//输入空
        INPUT_FILL//输入有内容
    }

    private var mIconState: IconState = IconState.NORMAL

    init {
        inflate(context, R.layout.voice_view, this)
        mVoiceWaveView = findViewById(R.id.voice_wave_view)
        mIvAction = findViewById(R.id.iv_ai_action)
        mEditInput = findViewById(R.id.edit_input_ai_content)
        mEditInput.filters = arrayOf<InputFilter>(EmojiFilter())
        setListener()
    }

    private fun setListener() {
        mIvAction.setOnClickListener { toggleButton() }
        mEditInput.setOnFocusChangeListener { _, hasFocus ->
            if (hasFocus.not()) {
                InputMethodUtil.hideSystemKeyBoard(context, mEditInput)
            }
        }
        mEditInput.afterTextChanged {
            val isEmpty = it?.toString()?.trim().isNullOrEmpty()
            mIconState = if (isEmpty) IconState.INPUT_EMPTY else IconState.INPUT_FILL
            mIvAction.setImageResource(if (isEmpty) R.drawable.im_yuyin_icon_58 else R.drawable.im_send_icon_58)
        }
        mVoiceWaveView.setOnActionDownCallback {
            mIvAction.visibility = View.GONE
            checkPrivacy(
                (context as Activity), listOf(KEY_RECORD_CHAT),
                object : AdapterPrivacyRightListener() {
                    override fun onAgreeRight() {
                        mVoiceWaveView.shouldHandleTouchEvents(true)
                        mRecordManager.prepareRecord(
                            object : RecordManager.SimpleRecordStateChangeListener() {
                                override fun wellPrepared() {
                                    startRecord()
                                    recordingListener?.onCallBack(true)
                                }

                                override fun failPrepared() {
                                    mRecordManager.cancel()
                                }

                                override fun maxDurationStopRecord() {
                                    ThreadUtil.ui {
                                        mVoiceWaveView.shouldHandleTouchEvents(false)//防止手指抬起时再次发送
                                        stopRecord(false)
                                        mIvAction.visibility = View.VISIBLE
                                        setVoiceEnable(false)

                                        //等待文件写完成，否则肯能导致 java.net.ProtocolException: expected 6903 bytes but received
                                        ThreadUtil.uiDelay({
                                            sendVoice()
                                        }, 100)
                                    }
                                }

                                override fun recordTimeStatistics(recordTimeDuration: Long) {
                                    //4 剩余可录制时长，小于等于 residueDuration展示可录音的倒计时文案
                                    val remainCanRecordDuration = maxTotalDuration - recordTimeDuration
                                    if (remainCanRecordDuration in 1000 until residueDuration) {
                                        mVoiceWaveView.countDownText = ("${remainCanRecordDuration / 1000}后停止录音")
                                    } else {
                                        mVoiceWaveView.countDownText = ""
                                    }
                                }
                            },
                            maxTotalDuration,
                            AiNetManager.getRecordOutDir(context)
                        )
                    }

                    override fun onDenied() {
                        super.onDenied()
                        mVoiceWaveView.shouldHandleTouchEvents(false)
                    }

                    override fun onParameterError() {
                        super.onParameterError()
                        mVoiceWaveView.shouldHandleTouchEvents(false)
                    }
                },
                PrivacyPermissionKey.RECORD_AUDIO
            )
        }
        mVoiceWaveView.setOnConfirmCallback { state, isSend ->
            //手指抬起时结束录音回调
            mIvAction.visibility = View.VISIBLE
            if (isSend.not()) return@setOnConfirmCallback
            var isCancel = false
            if (state == VoiceWaveView.State.RELEASE_SEND && System.currentTimeMillis() - mStartRecordTime >= MIN_RECORD_TIME) {
                isCancel = false
                //等待文件写完成，否则肯能导致 java.net.ProtocolException: expected 6903 bytes but received
                ThreadUtil.uiDelay({
                    sendVoice()
                }, 100)
            } else {
                isCancel = true
            }
            stopRecord(isCancel)
        }
    }

    private fun sendVoice(){
        //val wavPath = getWavFormat(mRecordManager.curFilePath)
        if (TextUtils.isEmpty(mRecordManager.curFilePath)){
            LogUtils.i("VoiceView", "原始音频文件：mRecordManager.filePath = " + mRecordManager.curFilePath)
            return
        }

        mOnSendListener?.invoke(AiContentType.VOICE, mRecordManager.curFilePath)
        //PcmToWavUtil.getInstance().pcmToWav(mRecordManager.curFilePath, wavPath)
        //ThreadUtil.ui {
        //    mOnSendListener?.invoke(MsgType.VOICE, wavPath)
        //}
    }

    /**
     * 开始录音
     */
    private fun startRecord() {
        mStartRecordTime = System.currentTimeMillis()
        mUpdateWaveHandler.post(updateWaveform)
    }

    /**
     * 结束录音
     * @param isCancelSend 是否取消发送
     */
    private fun stopRecord(isCancelSend: Boolean) {
        mUpdateWaveHandler.removeCallbacks(updateWaveform)
        if (isCancelSend) {
            mRecordManager.cancel()
        } else {
            mRecordManager.release()
        }
    }

    private fun toggleButton() {
        when (mIconState) {
            IconState.NORMAL -> { //切换到输入框
                mIvAction.setImageResource(R.drawable.im_yuyin_icon_58)
                mIconState = IconState.INPUT_EMPTY
                mEditInput.visibility = View.VISIBLE
                mVoiceWaveView.visibility = View.GONE
                mEditInput.requestFocus()
                mEditInput.setText("")
                if (InputMethodUtil.isShowKeyboard(context, mEditInput).not()) {
                    InputMethodUtil.showKeyboard(context, mEditInput)
                }
            }

            IconState.INPUT_FILL -> { //发送
                mIvAction.setImageResource(R.drawable.im_yuyin_icon_58)
                mIconState = IconState.INPUT_EMPTY
                mOnSendListener?.invoke(AiContentType.TEXT, mEditInput.text.toString().trim())
                mEditInput.setText("")
            }

            IconState.INPUT_EMPTY -> { //切换到默认
                mIvAction.setImageResource(R.drawable.im_shuru_icon_58)
                mIconState = IconState.NORMAL
                mEditInput.clearFocus()
                mEditInput.visibility = View.GONE
                mVoiceWaveView.postDelayed({ mVoiceWaveView.visibility = View.VISIBLE }, 50)
            }
        }
    }

    /**
     * 消息发送回调
     */
    fun setOnSendListener(callback: ((AiContentType, contentOrPath: String?) -> Unit)?) {
        this.mOnSendListener = callback
    }

    /**
     * 录音过程
     */
    fun setRecordingListener(recordingListener: OnSingleMethodListener<Boolean>) {
        this.recordingListener = recordingListener
    }

    private val updateWaveform: Runnable = object : Runnable {
        override fun run() {
            mVoiceWaveView.setAmplitude(mRecordManager.getVoiceRate(4))
            mUpdateWaveHandler.postDelayed(this, 100)
        }
    }

    fun setVoiceEnable(isEnable: Boolean) {
        mVoiceWaveView.setVoiceWaveEnable(isEnable)
        mIvAction.isEnabled = isEnable
        LogUtils.i("setVoiceEnable", "isEnable = $isEnable")
        mIvAction.alpha = if (isEnable) 1f else 0.3f
        mEditInput.isEnabled = isEnable
    }


}