package com.alpha.lagouapk.ui.interview.ai.widget

import android.content.Context
import android.graphics.Canvas
import android.graphics.Color
import android.graphics.LinearGradient
import android.graphics.Paint
import android.graphics.RectF
import android.graphics.Shader
import android.os.Vibrator
import android.text.TextUtils
import android.util.AttributeSet
import android.view.MotionEvent
import android.view.View
import com.alpha.lagouapk.R
import com.lagou.commonlibs.utils.ResourceUtils
import kotlin.random.Random

class VoiceWaveView @JvmOverloads constructor(
    context: Context,
    attrs: AttributeSet? = null,
    defStyleAttr: Int = 0
) : View(context, attrs, defStyleAttr) {
    private val mRecordButtonHeight = ResourceUtils.getDimenSize(R.dimen.dp_60)
    private val mRecordButtonCorner: Int = ResourceUtils.getDimenSize(R.dimen.dp_10)//按钮的圆角
    private val mRecordButtonSendStartColor = Color.parseColor("#189CF3")//松手发送 背景起始颜色
    private val mRecordButtonSendEndColor = Color.parseColor("#0A63E2")//松手发送 背景结束颜色
    private val mRecordButtonCancelStartColor = Color.parseColor("#F37018")//松手取消 背景起始颜色
    private val mRecordButtonCancelEndColor = Color.parseColor("#CA0303")//松手取消 背景结束颜色
    private val mRecordButtonNormalStartColor = Color.parseColor("#FFFFFF")//默认 背景起始颜色
    private val mRecordButtonNormalEndColor = Color.parseColor("#FFFFFF")//默认 背景结束颜色
    private var mRecordButtonPaint: Paint = Paint(Paint.ANTI_ALIAS_FLAG)

    private val mWaveItemWidth = ResourceUtils.getDimenSize(R.dimen.dp_3)//语音条的宽度
    private val mWaveItemMinHeight = ResourceUtils.getDimenSize(R.dimen.dp_7)//语音条的最小高度
    private val mWaveItemGap = ResourceUtils.getDimenSize(R.dimen.dp_3)//语音条的间隔
    private val mWaveItemCorner = ResourceUtils.getDimenSize(R.dimen.dp_3)//语音条的圆角
    private val mWaveItemCount = 30 //语音条的数量
    private var amplitude = 0f // 语音波纹浮动系数，从0-1
    private var mWavePaint: Paint = Paint(Paint.ANTI_ALIAS_FLAG)//绘制语音波纹画笔

    private val dp_24 = resources.getDimension(R.dimen.dp_24)
    private var mState: State = State.NORMAL

    private val mTopTipPaint = Paint(Paint.ANTI_ALIAS_FLAG)
    private val mSpeakPaint = Paint(Paint.ANTI_ALIAS_FLAG)//按住说话画笔

    private var onConfirmCallback: ((State, Boolean) -> Unit)? = null
    private var onActionDownCallback: (() -> Unit)? = null
    private var vibrator: Vibrator? = null
    private var shouldHandleTouchEvents = false//是否需要处理touch事件

    /**
     * 倒计时文本
     * 设置倒计时文案，不为空则RELEASE_SEND状态下优先展示倒计时文案（不展示 ”松手发送，上移取消“）
     */
    var countDownText: String? = ""
        set(value) {
            field = value
            invalidate()
        }

    enum class State {
        DISABLE,//禁用状态
        NORMAL,//默认状态
        RELEASE_SEND,//松手发送
        RELEASE_CANCEL,//松手取消
    }

    init {
        mWavePaint.color = Color.parseColor("#FFFFFF")
        mWavePaint.strokeWidth = mWaveItemWidth.toFloat()
        mWavePaint.style = Paint.Style.FILL
        mTopTipPaint.color = Color.parseColor("#999999")
        mTopTipPaint.textSize = ResourceUtils.getDimenSize(R.dimen.sp_12).toFloat()
        mSpeakPaint.color = Color.parseColor("#333333")
        mSpeakPaint.textSize = ResourceUtils.getDimenSize(R.dimen.sp_18).toFloat()
        mSpeakPaint.isFakeBoldText = true
        vibrator = context.getSystemService(Context.VIBRATOR_SERVICE) as Vibrator;
    }

    private fun getViewWidth(): Int {
        return mWaveItemWidth * mWaveItemCount + (mWaveItemCount - 1) * mWaveItemGap
    }

    override fun onMeasure(widthMeasureSpec: Int, heightMeasureSpec: Int) {
        super.onMeasure(widthMeasureSpec, heightMeasureSpec)
        val heightSpecSize = MeasureSpec.getSize(heightMeasureSpec)
        val heightSpecMode = MeasureSpec.getMode(heightMeasureSpec)
        val widthSpecSize = MeasureSpec.getSize(widthMeasureSpec)
        val realHeight =
            if (mState == State.NORMAL || mState == State.DISABLE) mRecordButtonHeight else ResourceUtils.getDimenSize(
                R.dimen.dp_90
            )
        if (heightSpecMode == MeasureSpec.AT_MOST || heightSpecMode == MeasureSpec.UNSPECIFIED) {
            setMeasuredDimension(widthSpecSize, realHeight)
        } else {
            setMeasuredDimension(widthSpecSize, heightSpecSize.coerceAtLeast(realHeight))
        }
    }

    override fun onDraw(canvas: Canvas?) {
        super.onDraw(canvas)
        canvas?.let {
            drawTopTip(it)
            drawRecordButton(it)
            drawWave(it)
        }
    }

    /**
     * 绘制顶部提示
     */
    private fun drawTopTip(canvas: Canvas) {
        if (mState == State.NORMAL || mState == State.DISABLE) {
            return
        }

        mTopTipPaint.color = Color.parseColor("#999999")
        val text = if (mState == State.RELEASE_SEND) {
            if (TextUtils.isEmpty(countDownText)) {
                "松手发送，上移取消"
            } else {
                mTopTipPaint.color = Color.parseColor("#0C6BE4")
                countDownText!!
            }
        } else "松手取消"

        val textWidth = mTopTipPaint.measureText(text)
        canvas.drawText(
            text,
            (canvas.width - textWidth) / 2,
            -mTopTipPaint.fontMetrics.ascent,
            mTopTipPaint
        )
    }

    /**
     * 绘制录音button
     */
    private fun drawRecordButton(canvas: Canvas) {
        when (mState) {
            State.NORMAL, State.DISABLE -> {
                val gradient = LinearGradient(
                    0f,
                    0f,
                    0f,
                    height.toFloat(),
                    mRecordButtonNormalStartColor,
                    mRecordButtonNormalEndColor,
                    Shader.TileMode.CLAMP
                )
                mRecordButtonPaint.shader = gradient
                canvas.drawRoundRect(
                    RectF(0f, 0f, width.toFloat(), height.toFloat()),
                    mRecordButtonCorner.toFloat(),
                    mRecordButtonCorner.toFloat(),
                    mRecordButtonPaint
                )
                mSpeakPaint.color =
                    if (mState == State.NORMAL) Color.parseColor("#333333") else Color.parseColor("#CCCCCC")
                val speakText = "按住说话"
                val textWidth = mSpeakPaint.measureText(speakText)
                val fontMetrics = mSpeakPaint.fontMetrics
                val centerX = width / 2 - textWidth / 2
                val centerY = height / 2 - ((fontMetrics.ascent + fontMetrics.descent) / 2)
                canvas.drawText(speakText, centerX, centerY, mSpeakPaint)
            }

            State.RELEASE_SEND -> {
                val gradient = LinearGradient(
                    0f,
                    0f,
                    0f,
                    height.toFloat(),
                    mRecordButtonSendStartColor,
                    mRecordButtonSendEndColor,
                    Shader.TileMode.CLAMP
                )
                mRecordButtonPaint.shader = gradient
                canvas.drawRoundRect(
                    RectF(
                        0f,
                        (height - mRecordButtonHeight).toFloat(),
                        width.toFloat(),
                        height.toFloat()
                    ),
                    mRecordButtonCorner.toFloat(),
                    mRecordButtonCorner.toFloat(),
                    mRecordButtonPaint
                )
            }

            State.RELEASE_CANCEL -> {
                val gradient = LinearGradient(
                    0f,
                    0f,
                    0f,
                    height.toFloat(),
                    mRecordButtonCancelStartColor,
                    mRecordButtonCancelEndColor,
                    Shader.TileMode.CLAMP
                )
                mRecordButtonPaint.shader = gradient
                canvas.drawRoundRect(
                    RectF(
                        0f,
                        (height - mRecordButtonHeight).toFloat(),
                        width.toFloat(),
                        height.toFloat()
                    ),
                    mRecordButtonCorner.toFloat(),
                    mRecordButtonCorner.toFloat(),
                    mRecordButtonPaint
                )
            }
        }
    }

    /**
     * 绘制语音波纹
     */
    private fun drawWave(canvas: Canvas) {
        if (mState == State.NORMAL || mState == State.DISABLE) {
            return
        }
        val centerY = height - mRecordButtonHeight / 2f
        val offsetX = width / 2f - getViewWidth() / 2f
        for (i in 0.until(mWaveItemCount)) {
            when (i) {
                0, mWaveItemCount - 1 -> {
                    mWavePaint.color = Color.parseColor("#4dFFFFFF")
                }

                1, mWaveItemCount - 2 -> {
                    mWavePaint.color = Color.parseColor("#66FFFFFF")
                }

                2, mWaveItemCount - 3 -> {
                    mWavePaint.color = Color.parseColor("#99FFFFFF")
                }

                3, mWaveItemCount - 4 -> {
                    mWavePaint.color = Color.parseColor("#CCFFFFFF")
                }

                else -> mWavePaint.color = Color.parseColor("#FFFFFFFF")
            }
            val indicatorHeight = mWaveItemMinHeight + Random.nextInt(0, dp_24.toInt()) * amplitude
            val startX = i * (mWaveItemWidth + mWaveItemGap).toFloat() + offsetX
            canvas.drawRoundRect(
                RectF(
                    startX,
                    centerY - indicatorHeight / 2f,
                    startX + mWaveItemWidth,
                    centerY + indicatorHeight / 2f
                ), mWaveItemCorner.toFloat(), mWaveItemCorner.toFloat(), mWavePaint
            )
        }
    }

    /**
     * 设置振幅0-1
     */
    fun setAmplitude(amplitude: Float) {
        this.amplitude = amplitude
        invalidate()
    }

    private val location = IntArray(2)
    override fun onTouchEvent(event: MotionEvent?): Boolean {
        when (event?.action) {
            MotionEvent.ACTION_DOWN -> {
                if (mState != State.DISABLE) {
                    changeState(State.RELEASE_SEND)
                    onActionDownCallback?.invoke()
                    return true
                }
            }

            MotionEvent.ACTION_MOVE -> {
                if (shouldHandleTouchEvents) {
                    getLocationOnScreen(location)
                    val top = location[1]
                    if (event.rawY < top) {
                        //松手取消
                        changeState(State.RELEASE_CANCEL)
                    } else {
                        //松手发送
                        changeState(State.RELEASE_SEND)
                    }
                }

            }

            MotionEvent.ACTION_UP -> {
                onConfirmCallback?.invoke(mState, shouldHandleTouchEvents)
                if (mState != State.DISABLE) {
                    changeState(State.NORMAL)//从 disable 到 normal 需要调用setVoiceWaveEnable()更改
                }
            }
        }
        return super.onTouchEvent(event)
    }

    /**
     * 出发振动
     */
    private fun triggerVibrate() {
        vibrator?.vibrate(100)
    }

    private fun changeState(state: State) {
        if (mState == state) {
            return
        }
        val isTriggerVibrate =
            (state == State.RELEASE_SEND && mState == State.NORMAL) || (state == State.RELEASE_CANCEL && mState == State.RELEASE_SEND) || (mState == State.RELEASE_CANCEL && state == State.RELEASE_SEND)
        val isSizeChange =
            (state == State.RELEASE_CANCEL || state == State.RELEASE_SEND) && (mState == State.NORMAL || mState == State.DISABLE) ||
                    (mState == State.RELEASE_CANCEL || mState == State.RELEASE_SEND) && (state == State.NORMAL || state == State.DISABLE)
        this.mState = state
        if (isSizeChange) {
            requestLayout()
        } else {
            invalidate()
        }
        if (isTriggerVibrate) {
            triggerVibrate()
        }
    }

    fun setVoiceWaveEnable(isEnable: Boolean) {
        changeState(if (isEnable) State.NORMAL else State.DISABLE)
    }

    /**
     * 手抬起时回调
     */
    fun setOnConfirmCallback(callBack: ((State, Boolean) -> Unit)?) {
        this.onConfirmCallback = callBack
    }

    /**
     * 按下按钮时的回调
     */
    fun setOnActionDownCallback(callback: (() -> Unit)?) {
        this.onActionDownCallback = callback
    }

    /**
     * 是否需要处理OnMove 和 onUp事件
     */
    fun shouldHandleTouchEvents(shouldHandleTouchEvents: Boolean) {
        this.shouldHandleTouchEvents = shouldHandleTouchEvents
    }

}