package com.alpha.lagouapk.ui.interview.view

import android.content.Context
import android.graphics.Canvas
import android.graphics.Color
import android.graphics.LinearGradient
import android.graphics.Paint
import android.graphics.Shader
import android.util.AttributeSet
import android.view.View
import com.alpha.lagouapk.R
import com.lagou.commonlibs.utils.ResourceUtils


/**
 * Time：2024/8/16 10:06:52
 * Author：wangjunwei
 * Description：
 * 绘制渐变色的圆环
 */
class AiRobotGradientRingView @JvmOverloads constructor(
    context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0
) : View(context, attrs, defStyleAttr) {
 
    private val mPaint = Paint()
    private var shader: Shader? = null
    private val startColor = Color.parseColor("#A9FFBC")
    private val endColor = Color.parseColor("#16C6CB")
    private val dp_7 = ResourceUtils.getDimenSize(R.dimen.dp_7).toFloat()
    init {
        mPaint.style = Paint.Style.STROKE
        mPaint.strokeWidth = dp_7
    }

    override fun onDraw(canvas: Canvas) {
        super.onDraw(canvas)
        val centerX = width / 2f
        val centerY = height / 2f
        val radius = centerX.coerceAtMost(centerY) - dp_7
        if (shader == null){
            shader = LinearGradient(centerX - radius, centerY - radius,
                centerX + radius, centerY + radius, startColor, endColor, Shader.TileMode.CLAMP)
        }
        mPaint.shader = shader
        canvas.drawArc(centerX - radius, centerY - radius, centerX + radius, centerY + radius, 0f, 360f, false, mPaint)
    }
}