package com.common.ui

import android.view.View
import android.view.View.OnClickListener
import com.ApiService
import com.alpha.lagouapk.LagouApplication
import com.alpha.lagouapk.R
import com.alpha.lagouapk.UserRoleManager
import com.alpha.lagouapk.tools.IntentUtils
import com.alpha.lagouapk.ui.user.ResetPasswordActivity
import com.common.bean.json.PasswordStatusBean
import com.common.utils.ViewUtils
import com.common.widget.SettingLinear
import com.lagou.baselib.BaseActivity
import com.lagou.baselib.analytic.AnalyticHelper
import com.lagou.baselib.analytic.definition.c.AID
import com.lagou.baselib.analytic.definition.c.MID
import com.lagou.baselib.net.httpnetback.BaseCallback
import com.lagou.commonlibs.view.LTitleBarLayout
import com.lagou.socketchannel.http.HttpRequest
import com.lagou.socketchannel.http.HttpResponse

/**
 * 账号与安全中心
 */
class AccountSafeCenterActivity:BaseActivity(),OnClickListener {
    private lateinit var mAccountView:SettingLinear
    private lateinit var mPasswordView:SettingLinear
    private var mIsNeedSetPassword:Boolean = false
    private lateinit var mTitleBar:LTitleBarLayout
    override fun getContentLayoutId(): Int = R.layout.activity_account_safe_center

    override fun init() {
        mAccountView = findViewById(R.id.sl_account_manager)
        mPasswordView = findViewById(R.id.sl_modify_password)
        mTitleBar = findViewById(R.id.title_bar)
        ViewUtils.setViewsClick(this,mAccountView,mPasswordView)
        getPasswordIsSet()
        mTitleBar.setLeftImgListener {
            finish()
        }
    }

    override fun onResume() {
        super.onResume()
        if(UserRoleManager.getInstance().isCUser) return
        mPasswordView.setRightTextContent(if (LagouApplication.isShowPwdStrength) getString(R.string.show_password_compromised) else "")
    }

    override fun onClick(v: View?) {
        when(v?.id){
            R.id.sl_account_manager -> {
                IntentUtils.startMineAccount(baseActivity)
                AnalyticHelper.getInstance().sendClickInfo(MID.bP00, AID.dk00)
            }
            R.id.sl_modify_password -> {
                //需要根据是否设置密码判断，是进入修改密码逻辑还是设置密码逻辑  ---
                var type = ResetPasswordActivity.MODIFY_PASSWORD
                if (mIsNeedSetPassword) {
                    type = ResetPasswordActivity.SET_PASSWORD
                } else {
                    AnalyticHelper.getInstance().sendClickInfo(MID.bP00, AID.dp00)
                }
                IntentUtils.startResetPassword(this, type)
            }
        }
    }

    /***
     * 统一注册流程新加，确定当前账号是否设置过密码
     */
    private fun getPasswordIsSet() {
        ApiService.api.queryPasswordStatus()
            .enqueue(object : BaseCallback<PasswordStatusBean?>(false) {
                override fun onSuccess(p0: HttpRequest<*>?, httpResponse: HttpResponse<PasswordStatusBean?>?) {
                    val (hasPassword) = httpResponse?.content ?: return
                    if (isActivityDestroyed) return
                    mIsNeedSetPassword = !hasPassword
                    try {
                        if (mIsNeedSetPassword) {
                            mPasswordView.setLeftTextContent(resources.getString(R.string.set_password));
                        }
                    } catch (e: Exception) {
                        e.printStackTrace()
                    }
                }
            })
    }
}