package com.common.ui

import android.content.Context
import android.util.AttributeSet
import android.widget.HorizontalScrollView

class HorizontalScrollViewCompat @JvmOverloads constructor(context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0):HorizontalScrollView(context,attrs,defStyleAttr) {
    private var mOnScrollChangeListener: OnScrollChangeListener? = null
    private var contentWidth = 0
    override fun onScrollChanged(l: Int, t: Int, oldl: Int, oldt: Int) {
        super.onScrollChanged(l, t, oldl, oldt)
        // 仅在内容宽度超过屏幕宽度且实际发生滚动时才触发
        if (contentWidth > measuredWidth && (l != oldl || t != oldt)) {
            mOnScrollChangeListener?.onScrollChanged(this, l, t, oldl, oldt)
        }
    }
    override fun onMeasure(widthMeasureSpec: Int, heightMeasureSpec: Int) {
        super.onMeasure(widthMeasureSpec, heightMeasureSpec)
        // 测量子视图的宽度
        contentWidth = getChildAt(0)?.measuredWidth ?: 0
    }

    fun setOnScrollChangeListener(listener: OnScrollChangeListener?) {
        mOnScrollChangeListener = listener
    }

    interface OnScrollChangeListener {
        fun onScrollChanged(scrollView: HorizontalScrollView?, l: Int, t: Int, oldl: Int, oldt: Int)
    }
}