package com.lagou.lagouee.ui.dialog.business

import android.os.Bundle
import android.os.Message
import android.view.View.GONE
import android.view.View.INVISIBLE
import android.view.View.OnClickListener
import android.view.View.VISIBLE
import android.widget.TextView
import com.ApiService
import com.alpha.lagouapk.R
import com.lagou.baselib.BaseActivity
import com.lagou.baselib.net.httpnetback.BaseCallback
import com.lagou.commonlibs.event.Event
import com.lagou.commonlibs.event.OkBus
import com.lagou.commonlibs.ui.dialog.BaseBottomSheetDialogFragment
import com.lagou.commonlibs.utils.ResourceUtils
import com.lagou.commonlibs.utils.StringUtil
import com.lagou.lagouee.bean.jsonbean.position.PositionSearchBean
import com.lagou.lagouee.observer.BEventTags
import com.lagou.lagouee.ui.position.PositionSearchActivity
import com.lagou.lagouee.utils.DialogUtils
import com.lagou.lagouee.utils.IntentUtils
import com.lagou.socketchannel.http.HttpRequest
import com.lagou.socketchannel.http.HttpResponse
import kotlinx.android.synthetic.main.dialog_intention_ensure.rel_close
import kotlinx.android.synthetic.main.dialog_intention_ensure.tv_bottom_click
import kotlinx.android.synthetic.main.dialog_intention_ensure.tv_tips
import kotlinx.android.synthetic.main.dialog_intention_ensure.vs_release_position
import kotlinx.android.synthetic.main.dialog_intention_ensure.vs_select_position
import kotlinx.android.synthetic.main.dialog_intention_ensure.vs_single_position

/**
 * Time：2024/10/12 11:43:55
 * Author：wangjunwei
 * Description：
 * 意向确认弹窗
 */
class IntentionEnsureDialog : BaseBottomSheetDialogFragment() {

    /**
     * 选择的职位，后续提交意向确认做准备
     */
    private var positionInfo: PositionSearchBean.ResultBean? = null

    private var positionInfoList: ArrayList<PositionSearchBean.ResultBean?>? = null
    private var cUserid: Long = 0
    private var expectJobId: String = ""
    private var attachmentResumeId: String = ""

    /**
     * 1 也有不止一个职位，选择职位按钮
     */
    private var tvSelectPosition: TextView? = null

    /**
     * 2 只有一个职位的情况
     */
    private var tvPositionName: TextView? = null
    private var tvPositionInfo: TextView? = null
    private var tvChangePosition: TextView? = null

    /**
     * 3 处理没有职位情况
     */
    private var tvReleasePosition: TextView? = null

    /***
     * 选择职位
     */
    private val eventSelectPositionEvent =
        Event { msg: Message? ->
            if (msg != null && msg.obj is PositionSearchBean.ResultBean) {
                val resultBean = msg.obj as PositionSearchBean.ResultBean
                handleSinglePosition(resultBean)
            }
        }

    /**
     * 跳转至发布职位
     */
    private val onReleasePositionListener = OnClickListener {
        IntentUtils.startReleaseActivity(activity)
        dismiss()
    }

    /**
     * 意向确认提交按钮
     */
    private val onSubmitEnsureListener = OnClickListener {
        intentionEnsureSubmit()

        dismiss()
    }

    /**
     * 意向确认提交接口
     */
    private fun intentionEnsureSubmit() {

        ApiService.api.intentionEnsureSubmit(cUserid, positionInfo?.id ?: 0,
            expectJobId, attachmentResumeId).enqueue(object : BaseCallback<Any>() {
            override fun onSuccess(request: HttpRequest<*>?, response: HttpResponse<Any>?) {
                if (activity.isDestroyed){
                    return
                }
                //1 弹出意向确认结果弹窗
                DialogUtils.showIntentionSubmitResultDialog(ResourceUtils.getString(
                    R.string.intention_submit_success_text), true, activity as BaseActivity)

                //2 刷新人才详情页面，已提交意向确认结果
                OkBus.getInstance().onEvent(BEventTags.INTENTION_ENSURE_RESULT_SUCCESS, cUserid)
            }

            override fun onFailure(request: HttpRequest<*>, response: HttpResponse<Any>) {
                // 506 message：意向确认权益已消耗完毕  1008：参数错误
                if (activity.isDestroyed){
                    return
                }

                dismiss()
                if (response.state == 506 || response.state == 1008){
                    DialogUtils.showIntentionSubmitResultDialog(response.message, false, activity as BaseActivity)
                    return
                }

                super.onFailure(request, response)
            }
        })
    }

    companion object {

        const val KEY_POSITION_INFO = "key_position_info"

        const val KEY_USER_ID = "key_user_id"
        const val KEY_EXPECT_JOB_ID = "key_expect_job_id"
        const val KEY_ATTACHMENT_RESUME_ID = "key_attachment_resume_id"

        /**
         * @param positionInfoList 可选择的职位列表信息（第一页）
         */
        @JvmStatic
        fun getInstance(positionInfoList : ArrayList<PositionSearchBean.ResultBean>?,
                        cUserid: Long, expectJobId: String?, attachmentResumeId: String?): IntentionEnsureDialog {
            val arguments = Bundle()
            arguments.apply {
                putSerializable(KEY_POSITION_INFO, positionInfoList)
                putLong(KEY_USER_ID, cUserid)
                putString(KEY_EXPECT_JOB_ID, expectJobId)
                putString(KEY_ATTACHMENT_RESUME_ID, attachmentResumeId)
            }
            val intentionEnsureDialog = IntentionEnsureDialog()
            intentionEnsureDialog.arguments = arguments
            return intentionEnsureDialog
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        positionInfoList = arguments?.getSerializable(KEY_POSITION_INFO) as ArrayList<PositionSearchBean.ResultBean?>
        cUserid = arguments?.getLong(KEY_USER_ID, 0) ?: 0
        expectJobId = arguments?.getString(KEY_EXPECT_JOB_ID) ?: ""
        attachmentResumeId = arguments?.getString(KEY_ATTACHMENT_RESUME_ID, "") ?: ""

        OkBus.getInstance()
            .register(BEventTags.INTENTION_ENSURE_SELECT_POSITION, eventSelectPositionEvent)
    }

    override fun initConfigAtCreateView() {
        super.initConfigAtCreateView()
        dialog!!.setCanceledOnTouchOutside(false)
    }

    override fun fillUI() {
        super.fillUI()
        //1 判断有多个职位的情况
        if ((positionInfoList?.size ?: 0) > 1){
            handleMorePosition()
            return
        }

        //2 单个的职位的情况
        if ((positionInfoList?.size ?: 0) == 1){
            val positionInfo = positionInfoList?.first()
            if (positionInfo != null){
                handleSinglePosition(positionInfo)
                return
            }
        }

        //没有职位的情况
        handleNoPosition()
    }

    /**
     * 1 处理多个职位的情况
     */
    private fun handleMorePosition() {
        vs_release_position.visibility = GONE
        vs_select_position.visibility = VISIBLE
        vs_single_position.visibility = GONE
        tv_tips.visibility = VISIBLE

        tvSelectPosition = vs_select_position as TextView
        tvSelectPosition?.setOnClickListener {
            //跳转到选择职位页面
            IntentUtils.startPositionSearch(
                activity, PositionSearchActivity.DOWNLOAD_MANAGE,
                BEventTags.INTENTION_ENSURE_SELECT_POSITION
            )
        }

        //底部按钮
        tv_bottom_click.isEnabled = false
        tv_bottom_click.alpha = 0.5f
        tv_bottom_click.text = ResourceUtils.getString(R.string.ensure)
        tv_bottom_click.setOnClickListener(onSubmitEnsureListener)
    }

    /**
     * 2 处理单个职位情况
     */
    private fun handleSinglePosition(positionInfo: PositionSearchBean.ResultBean) {
        this.positionInfo = positionInfo

        vs_select_position.visibility = GONE
        vs_single_position.visibility = VISIBLE
        vs_release_position.visibility = GONE
        tv_tips.visibility = VISIBLE
        if (tvPositionName == null){
            tvPositionName = vs_single_position.findViewById(R.id.tv_position_name)
        }
        if (tvPositionInfo == null){
            tvPositionInfo = vs_single_position.findViewById(R.id.tv_position_info)
        }
        if (tvChangePosition == null){
            tvChangePosition = vs_single_position.findViewById(R.id.tv_change_position)
            tvChangePosition?.setOnClickListener {
                //跳转到选择职位页面
                IntentUtils.startPositionSearch(
                    activity, PositionSearchActivity.DOWNLOAD_MANAGE,
                    BEventTags.INTENTION_ENSURE_SELECT_POSITION
                )
            }
        }

        positionInfo.apply {
            tvPositionName?.text = name
            tvPositionInfo?.text = StringUtil.jointString(
                    " | ", city, workYear, education)
        }
        //底部按钮
        tv_bottom_click.isEnabled = true
        tv_bottom_click.alpha = 1f
        tv_bottom_click.text = ResourceUtils.getString(R.string.ensure)
        tv_bottom_click.setOnClickListener(onSubmitEnsureListener)
    }

    /**
     * 处理没有职位的情况
     */
    private fun handleNoPosition() {
        vs_select_position.visibility = GONE
        vs_single_position.visibility = GONE
        vs_release_position.visibility = VISIBLE
        tv_tips.visibility = INVISIBLE
        tvReleasePosition = vs_release_position.findViewById(R.id.tv_release_position)

        //底部按钮
        tv_bottom_click.isEnabled = true
        tv_bottom_click.alpha = 1f
        tv_bottom_click.text = ResourceUtils.getString(R.string.release_position)
        tv_bottom_click.setOnClickListener(onReleasePositionListener)
    }

    override fun initListener() {
        super.initListener()
        //1 弹窗关闭点击事件
        rel_close.setOnClickListener {
            dismiss()
        }
    }

    override fun getLayoutId(): Int {
        return R.layout.dialog_intention_ensure
    }

    override fun getWidth(): Int {
        return screenWidth
    }

    override fun onDestroy() {
        super.onDestroy()
        OkBus.getInstance().unRegister(eventSelectPositionEvent)
    }
}