package com.lagou.lagouee.ui.dialog.business

import android.os.Bundle
import android.text.TextUtils
import android.view.View
import com.alpha.lagouapk.R
import com.lagou.commonlibs.ui.dialog.BaseDialogFragment
import com.lagou.commonlibs.utils.ResourceUtils
import kotlinx.android.synthetic.main.dialog_intention_submit_result.rel_close
import kotlinx.android.synthetic.main.dialog_intention_submit_result.rel_info
import kotlinx.android.synthetic.main.dialog_intention_submit_result.tv_show_text
import kotlinx.android.synthetic.main.dialog_intention_submit_result.tv_title

/**
 * Time：2024/11/11 17:37:36
 * Author：wangjunwei
 * Description：
 * 意向确认弹窗提交结果弹窗
 */
class IntentionSubmitResultDialog : BaseDialogFragment() {

    private var isSuccess: Boolean = false
    private var showText: String? = null
    private val DP_30 = ResourceUtils.getDimenSize(R.dimen.dp_30)
    private val DP_40 = ResourceUtils.getDimenSize(R.dimen.dp_40)

    companion object {
        private const val KEY_IS_SUCCESS: String = "key_is_success"
        private const val KEY_SHOW_TEXT: String = "key_show_text"

        /**
         * @param text 显示的文案
         * @param isSuccess 成功还是失败
         */
        @JvmStatic
        fun getInstance(text: String, isSuccess: Boolean): IntentionSubmitResultDialog {
            val bundle = Bundle()
            bundle.apply {
                putString(KEY_SHOW_TEXT, text)
                putBoolean(KEY_IS_SUCCESS, isSuccess)
            }
            val intentionSubmitResultDialog = IntentionSubmitResultDialog()
            intentionSubmitResultDialog.arguments = bundle

            return intentionSubmitResultDialog
        }
    }

    override fun getLayoutId(): Int {
        return R.layout.dialog_intention_submit_result
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        arguments?.apply {
            showText = getString(KEY_SHOW_TEXT)
            isSuccess = getBoolean(KEY_IS_SUCCESS)
        }
    }

    override fun fillUI() {
        super.fillUI()
        if (isSuccess) {
            //1 成功情况
            tv_title.setTextColor(ResourceUtils.getColor(R.color.color_00b38a))
            tv_title.text = ResourceUtils.getString(R.string.submit_successfull)
            tv_title.setCompoundDrawablesWithIntrinsicBounds(
                ResourceUtils.getDrawable(
                    R.drawable.delivery_im_successful_48
                ), null, null, null
            )

            tv_show_text.text = showText
        } else {
            //2 失败情况
            tv_title.setTextColor(ResourceUtils.getColor(R.color.color_FF4D4F))
            tv_title.text = ResourceUtils.getString(R.string.submit_fail)
            tv_title.setCompoundDrawablesWithIntrinsicBounds(
                ResourceUtils.getDrawable(
                    R.drawable.delivery_im_fail_48
                ), null, null, null
            )

            tv_show_text.text = showText
        }

        //处理特殊情况，showText 为空的情况
        if (TextUtils.isEmpty(showText)) {
            tv_show_text.visibility = View.GONE
            rel_info.setPadding(0, DP_40, 0, DP_40)
        } else {
            tv_show_text.visibility = View.VISIBLE
            rel_info.setPadding(0, DP_30, 0, DP_30)
        }
    }


    override fun initListener() {
        super.initListener()
        rel_close.setOnClickListener {
            dismiss()
        }
    }
    override fun getWidth(): Int {
        return (screenWidth * 0.8).toInt()
    }

}